package com.hisense.hiaposcmp.activity;


import android.app.Activity;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;

import androidx.appcompat.app.AppCompatActivity;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.anno.ActivityViewInject;
import com.hisense.hiaposcmp.boot.ActivityCollector;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.pos.scanner.CameraScanner;
import com.hisense.hiaposcmp.pos.scanner.StdScanner;
import com.hisense.hiaposcmp.util.AntiHijackingUtils;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.LaserScanner;
import com.hisense.pos.scanner.UmsCameraScanner;

import java.lang.reflect.Field;

import butterknife.ButterKnife;

/**
 * Activity 基类，用于Activity的统一管理。
 *
 * @author fujiaxin         2017-03-14
 * @author dukunbiao(null)  2019-07-22  重构
 */
public class BaseActivity extends AppCompatActivity {

    private int layoutId = -1;
    protected IScanner mScanner;
    private static long lastClickTime = 0;

    private static final int SPACE_TIME = 1500;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        displayInjectLayout();
        displayInjectView();

        ButterKnife.bind(this);

        ActivityCollector.AddActivity(this);
    }

    public static boolean isFastClick() {
        long currentTime = System.currentTimeMillis();
        boolean isFastClick;
        isFastClick = (currentTime - lastClickTime < SPACE_TIME);
        lastClickTime = currentTime;
        LoggerUtils.getLogger().debug("快速点击" + lastClickTime);
        return isFastClick;
    }

    public static boolean isFastClick(int spaceTime) {
        long currentTime = System.currentTimeMillis();
        boolean isFastClick;
        isFastClick = (currentTime - lastClickTime < spaceTime);
        lastClickTime = currentTime;
        LoggerUtils.getLogger().debug("快速点击" + lastClickTime);
        return isFastClick;
    }

    private void displayInjectLayout() {
        Class<?> clazz = this.getClass();
        if (clazz.isAnnotationPresent(ActivityLayoutInject.class)) {
            ActivityLayoutInject inject = clazz.getAnnotation(ActivityLayoutInject.class);
            layoutId = inject.value();
            setContentView(layoutId);
        }
    }

    private void displayInjectView() {
        if (layoutId <= 0) {
            return;
        }

        Class<?> clazz = this.getClass();
        Field[] fields = clazz.getDeclaredFields();
        try {
            for (Field field : fields) {
                if (field.isAnnotationPresent(ActivityViewInject.class)) {
                    field.setAccessible(true);
                    ActivityViewInject inject = field.getAnnotation(ActivityViewInject.class);
                    field.set(this, this.findViewById(inject.value()));
                }
            }
        } catch (Exception e) {

        }
    }

    protected IScanner getScanner() {
        return mScanner;
    }

    /**
     * 返回一个扫描器的实例。
     *
     * @param activity 调用的 Activity
     * @return {@code IScanner} 扫描器的实例
     */
    protected IScanner getScanner(Activity activity) {
        if (GeneralSettings.INSTANCE.getDeviceType().isHK316()) {
            return new StdScanner(activity);
        } else if (GeneralSettings.INSTANCE.getDeviceType().isHI98() &&
                GeneralSettings.INSTANCE.getScannerType().isLaser()) {
            return new LaserScanner(activity);
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isUMS()) {
            return new UmsCameraScanner(activity);
        } else {
            return new CameraScanner(activity);
        }
    }

    protected void onScan(String s) {
        if (mScanner != null) {
            mScanner.close();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mScanner != null) {
            mScanner.close();
        }
        ActivityCollector.removeActivity(this);
    }

    @Override
    protected void onResume() {
        Log.d("ActivityTrace", getClass().getSimpleName());
        boolean safe = AntiHijackingUtils.checkActivity(this);
        if (!safe) {
            // 程序退到后台，进行风险警告
//            Toast.makeText(this, "程序退到后台，若非本人操作请退出",
//                    Toast.LENGTH_LONG).show();
        }

        super.onResume();
    }

    @Override
    protected void onPause() {
        boolean safe = AntiHijackingUtils.checkActivity(this);
        if (!safe) {
            // 程序退到后台，进行风险警告
//            Toast.makeText(this, "程序退到后台，若非本人操作请退出",
//                    Toast.LENGTH_LONG).show();
        }
        super.onPause();
    }

    @Override
    protected void onStop() {
        boolean safe = AntiHijackingUtils.checkActivity(this);
        if (!safe) {
            // 程序退到后台，进行风险警告
//            Toast.makeText(this, "程序退到后台，若非本人操作请退出",
//                    Toast.LENGTH_LONG).show();
        }
        super.onStop();
    }

    protected String getVersion() {
        String versionName = "";

        try {
            PackageManager pm = this.getPackageManager();
            PackageInfo pi = pm.getPackageInfo(this.getPackageName(), PackageManager.GET_ACTIVITIES);

            if (pi != null) {
                versionName = pi.versionName == null ? "null" : pi.versionName;
            }
        } catch (PackageManager.NameNotFoundException e) {

        }

        return versionName;
    }

    /**
     * 缩放动画
     *
     * @param view 待缩放控件
     */
    public void executeAnimation(View view) {
        Animation scaleAnimation = AnimationUtils.loadAnimation(this, R.anim.scale_anim);
        view.startAnimation(scaleAnimation);
    }
}
