/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.refund.viewModel;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.ReturnSalePluModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.model.SalePluModel;
import com.hisense.hipos.plu.PluDscType;

import java.util.Locale;

import lombok.Data;


/**
 * 退货商品 ViewModel。
 *
 * @author xieliyan    2017-05-10
 * @author zhangsai    2018-03-07 HIPOSAPP-345 修复退货时清除退款金额闪退问题
 * @author zhangsai    2018-07-24 HIPOSAPP-699 优化退货商品逻辑
 * @author zhangsai    2018-12-26 HIPOSAPP-939 新增检查原销售商品是否包含普通/会员促销的方法
 * @author wangyaogang 2019-01-21 HIPOSAPP-970 修复选择退货商品界面,对复选框操作时,其他商品数量恢复最大值的问题
 * @author wangyaogang 2019-11-18 HIAPOSCMP-302  getQuantity()数据格式化
 */
@Data
public class ReturnItemViewModel {
    /**
     * 最小商品数量
     */
    private static final String MIN_COUNT = "0";
    /**
     * 退货数量
     */
    private String count;
    /**
     * 最大退货数量
     */
    private String maxCount;
    /**
     * 是否选中
     */
    private boolean checked;
    /**
     * 原交易商品
     */
    private SalePluModel saleItem;
    /**
     * 退货数量
     */
    private ReturnSalePluModel returnSaleItem;
    /**
     * 暂存在退货列表中的序号
     */
    private int position;


    /**
     * 使用 {@code returnSaleItem} 初始化一个 {@code ReturnItemViewModel} 的实例。
     *
     * @param returnSaleItem 退货商品
     */
    public ReturnItemViewModel(ReturnSalePluModel returnSaleItem) {
        this.checked = true;
        this.saleItem = returnSaleItem.getSalePluModel();
        this.returnSaleItem(returnSaleItem, returnSaleItem.getQuantity());
    }

    /**
     * 返回原交易商品。
     *
     * @return 原交易商品
     */
    public SalePluModel saleItem() {
        return this.saleItem;
    }

    /**
     * 返回退货商品。
     *
     * @return 退货商品
     */
    public ReturnSalePluModel returnSaleItem() {
        return this.returnSaleItem;
    }

    /**
     * 设置退货商品。
     *
     * @param returnSaleItem 退货商品
     */
    public void returnSaleItem(ReturnSalePluModel returnSaleItem) {
        this.returnSaleItem(returnSaleItem, returnSaleItem.getQuantity());
    }

    /**
     * 设置退货商品。
     *
     * @param returnSaleItem 退货商品
     * @param count          退货数量
     */
    public void returnSaleItem(ReturnSalePluModel returnSaleItem, String count) {
        this.returnSaleItem = returnSaleItem;
        this.count = this.maxCount = saleItem.getQuantity();
        if (BigDecimalUtils.isGreater0(count) && BigDecimalUtils.isLess(count, maxCount)) {
            this.count = count;
        }
    }

    /**
     * 返回商品名称。
     *
     * @return 商品名称
     */
    public String pluName() {
        return this.saleItem.getPluName();
    }

    /**
     * 返回原交易流水商品的合计实收金额。
     *
     * @return 原交易流水商品的合计实收金额
     */
    public String total() {
        return this.saleItem.getTotalRecv();
    }

    /**
     * 返回退货商品的合计退款金额。
     *
     * @return 退货商品的合计退款金额
     */
    public String totalRefund() {
        return this.returnSaleItem == null ? "0" : this.returnSaleItem.getTotalRecv();
    }

    /**
     * 返回商品单位。
     *
     * @return 商品单位
     */
    public String unit() {
        return this.saleItem.getUnit();
    }

    /**
     * 返回商品规格。
     *
     * @return 商品规格
     */
    public String spec() {
        return this.saleItem.getUnit();
    }

    /**
     * 返回商品的当前退货数量。
     *
     * @return 商品的当前退货数量
     */
    public String count() {
        return this.count;
    }

    /**
     * 多退一件。
     */
    public void returnMore() {
        if (testReturnMore()) {
            this.count = BigDecimalUtils.add(this.count, "1");
        }
    }

    public boolean returnWeightPlu(String quality) {
        if (BigDecimalUtils.isGreater(quality, this.maxCount)) {
            return false;
        }

        this.count = quality;
        return true;
    }

    /**
     * 少退一件。
     */
    public void returnLess() {
        if (testReturnLess()) {
            this.count = BigDecimalUtils.subtract(this.count, "1");
        }
    }

    /**
     * 检查商品能否再退一件。
     *
     * @return true 表示可以再退一件；false 表示不可以
     */
    public boolean testReturnMore() {
        if (BigDecimalUtils.isLess(this.count, maxCount)) {
            return true;
        } else {
            return false;
        }
    }

    public boolean test() {
        return BigDecimalUtils.isEquals0(maxCount);
    }

    /**
     * 检查商品能否少退一件。
     *
     * @return true 表示可以少退一件；false 表示不可以
     */
    public boolean testReturnLess() {
        if (BigDecimalUtils.isGreater(this.count, MIN_COUNT)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 返回销售商品是否包含普通优惠。
     *
     * @return 如果包含普通优惠则返回 true
     */
    public boolean hasNormalDsc() {
        for (SalePluDscModel dsc : this.saleItem.getPluDscs()) {
            if (dsc.getDscType().equals(PluDscType.NORMAL)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 返回销售商品是否包含会员优惠。
     *
     * @return 如果包含会员优惠则返回 true
     */
    public boolean hasVipDsc() {

        for (SalePluDscModel dsc : this.saleItem.getPluDscs()) {
            if (dsc.getDscType().equals(PluDscType.VIP)) {
                return true;
            }
        }

        return false;
    }

    /**
     * 返回商品是否被选中。
     *
     * @return true 表示选中状态；false 表示未选中
     */
    public boolean isChecked() {
        return this.checked;
    }

    /**
     * 更改商品的选中状态。
     *
     * @param checked true 表示选中；false 表示不选
     */
    public void changeCheckState(boolean checked) {
        this.checked = checked;
        if (!checked) {
            this.count = this.maxCount = MIN_COUNT;
            this.returnSaleItem = null;
        }
    }

    /**
     * 选中商品。
     */
    public void check() {
        this.changeCheckState(true);
    }

    /**
     * 取消商品选中状态。
     */
    public void unCheck() {
        this.changeCheckState(false);
    }
}
