/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.boot;

import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Build;
import android.os.Environment;
import android.os.Looper;
import android.util.Log;
import android.widget.Toast;

import com.hisense.common.util.CompressUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.BuildConfig;
import com.hisense.hiaposcmp.activity.common.BootUpActivity;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.lidroid.xutils.HttpUtils;
import com.lidroid.xutils.exception.HttpException;
import com.lidroid.xutils.http.RequestParams;
import com.lidroid.xutils.http.ResponseInfo;
import com.lidroid.xutils.http.callback.RequestCallBack;
import com.lidroid.xutils.http.client.HttpRequest;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.Thread.UncaughtExceptionHandler;
import java.lang.reflect.Field;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * UncaughtException处理类,当程序发生Uncaught异常的时候,有该类来接管程序,并记录发送错误报告.
 *
 * @author user
 */
public class CrashHandler implements UncaughtExceptionHandler {

    public static final String TAG = "CrashHandler";

    // CrashHandler 实例
    private static CrashHandler INSTANCE = new CrashHandler();

    // 程序的 Context 对象
    private Context mContext;

    // 系统默认的 UncaughtException 处理类
    private Thread.UncaughtExceptionHandler mDefaultHandler;

    // 用来存储设备信息和异常信息
    private Map<String, String> infos = new HashMap<String, String>();

    // 用于格式化日期,作为日志文件名的一部分
    private DateFormat formatter = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");


    /**
     * 保证只有一个 CrashHandler 实例
     */
    private CrashHandler() {
    }

    /**
     * 获取 CrashHandler 实例 ,单例模式
     */
    public static CrashHandler getInstance() {
        return INSTANCE;
    }

    /**
     * 初始化
     *
     * @param context
     */
    public void init(Context context) {
        mContext = context;

        // 获取系统默认的 UncaughtException 处理器
        mDefaultHandler = Thread.getDefaultUncaughtExceptionHandler();

        // 设置该 CrashHandler 为程序的默认处理器
        Thread.setDefaultUncaughtExceptionHandler(this);
    }

    /**
     * 当 UncaughtException 发生时会转入该函数来处理
     */
    @Override
    public void uncaughtException(Thread thread, Throwable ex) {
        if (!handleException(ex) && mDefaultHandler != null) {
            // 如果用户没有处理则让系统默认的异常处理器来处理
            mDefaultHandler.uncaughtException(thread, ex);
        } else {
            try {
                Thread.sleep(3000);
            } catch (InterruptedException e) {
                Log.e(TAG, "error : ", e);
            }

            // 退出程序,注释下面的重启启动程序代码
//            android.os.Process.killProcess(android.os.Process.myPid());
//            System.exit(1);

            // 重新启动程序，注释上面的退出程序
            if (CommonUtils.isWriteOk()) {
                LoggerUtils.getLogger().error("应用闪退crash，跳转到指定页面");
            }

            Intent intent = new Intent();
           /* if (CommonUtils.isUniformStyle()) {
                intent.setClass(mContext, BootUpActivity.class);
            } else {
                intent.setClass(mContext, MainActivity.class);
            }*/
            intent.setClass(mContext, BootUpActivity.class);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            mContext.startActivity(intent);
            android.os.Process.killProcess(android.os.Process.myPid());
        }
    }

    /**
     * 自定义错误处理，收集错误信息，发送错误报告等操作均在此完成
     *
     * @param ex
     * @return true：如果处理了该异常信息；否则返回 false
     */
    private boolean handleException(Throwable ex) {
        if (ex == null) {
            return false;
        }

        /**
         * 如果是调试时(非正式发布),则把错误日志打印出来
         */
        if (BuildConfig.DEBUG) {
            Log.v(TAG, "crash log:");
            ex.printStackTrace();
        }

        // 使用 Toast 来显示异常信息
        new Thread() {
            @Override
            public void run() {
                Looper.prepare();
                Toast.makeText(mContext, "很抱歉，程序出现异常，即将退出。", Toast.LENGTH_LONG).show();
                Looper.loop();
            }
        }.start();

        // 收集设备参数信息
        collectDeviceInfo(mContext);
        // 保存日志文件
        if (CommonUtils.isWriteOk()) {
            saveCrashInfo2File(ex);
        }
        return true;
    }

    /**
     * 收集设备参数信息
     *
     * @param ctx
     */
    public void collectDeviceInfo(Context ctx) {
        try {
            PackageManager pm = ctx.getPackageManager();
            PackageInfo pi = pm.getPackageInfo(ctx.getPackageName(), PackageManager.GET_ACTIVITIES);

            if (pi != null) {
                String versionName = pi.versionName == null ? "null" : pi.versionName;
                String versionCode = pi.versionCode + "";
                infos.put("versionName", versionName);
                infos.put("versionCode", versionCode);
            }
        } catch (NameNotFoundException e) {
            Log.e(TAG, "an error occured when collect package info", e);
        }

        Field[] fields = Build.class.getDeclaredFields();
        for (Field field : fields) {
            try {
                field.setAccessible(true);
                infos.put(field.getName(), field.get(null).toString());
                Log.d(TAG, field.getName() + " : " + field.get(null));
            } catch (Exception e) {
                Log.e(TAG, "an error occured when collect crash info", e);
            }
        }
    }

    /**
     * 保存错误信息到文件中
     * *
     *
     * @param ex
     * @return 返回文件名称, 便于将文件传送到服务器
     */
    private String saveCrashInfo2File(Throwable ex) {
        StringBuffer sb = new StringBuffer();
        for (Map.Entry<String, String> entry : infos.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            sb.append(key + "=" + value + "\n");
        }

        Writer writer = new StringWriter();
        PrintWriter printWriter = new PrintWriter(writer);
        ex.printStackTrace(printWriter);
        Throwable cause = ex.getCause();
        while (cause != null) {
            cause.printStackTrace(printWriter);
            cause = cause.getCause();
        }
        printWriter.close();

        String result = writer.toString();
        sb.append(result);
        try {
            long timestamp = System.currentTimeMillis();
            String time = formatter.format(new Date());
            final String fileName = "crash-" + time + "-" + timestamp + ".log";

            if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
                final String path = Config.log_path + "crash/";
                File dir = new File(path);
                if (!dir.exists()) {
                    dir.mkdirs();
                }
                FileOutputStream fos = new FileOutputStream(path + fileName);
                fos.write(sb.toString().getBytes());
                fos.close();

                new Thread() {
                    @Override
                    public void run() {
                        Looper.prepare();
                        upload(path + fileName);
                        Looper.loop();
                    }
                }.start();
            }


            return fileName;
        } catch (Exception e) {
            Log.e("TradeLog", "an error occured while writing file...", e);
        }
        return null;
    }

    /**
     * 上传crash文件
     * *
     *
     * @param s 文件全路径名
     */

    public static void upload(final String s) {
        HttpUtils mHttpUtils = new HttpUtils(15 * 1000);//5000-->联网超时时间-->可以无参
        //设置缓存大小
        int size = (int) (Runtime.getRuntime()
                .totalMemory() / 8);
        mHttpUtils.configHttpCacheSize(size);
        //设置编码格式
        mHttpUtils.configResponseTextCharset("utf-8");
        //设置重连次数,如果链接服务失败,再次尝试
        mHttpUtils.configRequestRetryCount(3);
        //设置多线程迸发(线程池)
        mHttpUtils.configRequestThreadPoolSize(4);
        //设置缓存数据失效的时间
        mHttpUtils.configDefaultHttpCacheExpiry(1000 * 60 * 30);
        String url = Config.base_url + "/api/log/upload";
        RequestParams params = new RequestParams();
        if (StringUtils.isEmpty(Config.org_code) || StringUtils.isEmpty(Config.pos_no)) {
                LoggerUtils.getLogger().debug("crash日志未成功上传到中间件服务器，崩溃后，组织编码等为空");
            return;
        }
        params.addBodyParameter("orgCode", Config.org_code);
        params.addBodyParameter("posNo", Config.pos_no);
        params.addBodyParameter("file", new File(s));
        mHttpUtils.send(HttpRequest.HttpMethod.POST, url, params, new RequestCallBack<String>() {
            @Override
            public void onSuccess(ResponseInfo<String> responseInfo) {

                if (responseInfo != null) {
                    try {
                        JSONObject object = new JSONObject(responseInfo.result);
                        int code = 1;
                        if (object.has("code")) {
                            code = object.getInt("code");
                        }
                        if (code == 1) {
                            LoggerUtils.getLogger().info("crash日志成功上传到中间件服务器");
                     //       CompressUtils.deleteAll(s);
                        } else {
                            LoggerUtils.getLogger().info("crash日志未成功上传到中间件服务器" + responseInfo.result);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        LoggerUtils.getLogger().error("crash日志未成功上传到中间件服务器" + e.toString());
                    }
                }

            }

            @Override
            public void onFailure(HttpException error, String msg) {
                LoggerUtils.getLogger().error("crash日志未成功上传到中间件服务器" + msg);
            }
        });
    }

}