/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.layout;

import android.content.Context;
import android.content.res.TypedArray;
import android.text.method.HideReturnsTransformationMethod;
import android.text.method.PasswordTransformationMethod;
import android.util.AttributeSet;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.view.KeyBoardView;


/**
 * 包含图标、清除按钮的组合 {@code EditText} 控件。
 *
 * @author zhangsai    2018-01-18
 * @author zhangsai    2018-08-21 新增默认按钮
 * @author wangyaogang 2019-01-03 HIPOSAPP-875 优化键盘使用逻辑
 */
public class CombinedEditText extends LinearLayout implements View.OnClickListener, View.OnFocusChangeListener, View.OnTouchListener {

    private ImageView imgIcon;              // 图标
    private EditText edit;                  // 文本编辑器
    private ImageButton imgClearText;       // 文本清除按钮
    private ImageButton imgDefault;         // 默认按钮
    private ImageButton imgShowPassword;    // 密码显示按钮
    private View vwBottomLine;              // 底部细线
    private View vwBottomLineFocused;       // 底部细线（激活时）

    private TypedArray attributes;              // 控件属性
    private SwitchViewWatcher clearTextWatcher;  // 文本监视器
    private KeyBoardView keyboard;              // 自定义软键盘

    private boolean isClearTextShown;       // 文本清除按钮是否显示
    private boolean isShowPasswordShown;    // 密码显示按钮是否显示


    /**
     * 初始化一个 {@code CombinedEditText} 的实例。
     *
     * @param context Context
     * @param attrs   AttributeSet
     */
    public CombinedEditText(Context context, AttributeSet attrs) {
        super(context, attrs);

        LayoutInflater.from(context).inflate(R.layout.combined_edit_text, this, true);

        imgIcon = (ImageView) findViewById(R.id.imgIcon);
        edit = (EditText) findViewById(R.id.edtText);
        imgClearText = (ImageButton) findViewById(R.id.imgClearText);
        imgDefault = (ImageButton) findViewById(R.id.imgDefault);
        imgShowPassword = (ImageButton) findViewById(R.id.imgShowPassword);
        vwBottomLine = findViewById(R.id.vwBottomLine);
        vwBottomLineFocused = findViewById(R.id.vwBottomLineFocused);

        int inputType = -1;
        this.attributes = context.obtainStyledAttributes(attrs, R.styleable.CombinedEditText);
        if (this.attributes != null) {
            this.setIcon(R.styleable.CombinedEditText_icon);

            inputType = this.attributes.getInteger(R.styleable.CombinedEditText_android_inputType, -1);
            if (inputType != -1) {
                edit.setInputType(inputType);
            }

            CharSequence hint = this.attributes.getString(R.styleable.CombinedEditText_android_hint);
            edit.setHint(hint);

            float textSize = this.attributes.getDimension(R.styleable.CombinedEditText_android_textSize, -1);
            if (textSize != -1) {
                edit.setTextSize(TypedValue.COMPLEX_UNIT_PX, textSize);
            }
        }

        if (inputType == 18) {
            isClearTextShown = false;
            isShowPasswordShown = true;
            imgShowPassword.setVisibility(View.VISIBLE);
            imgClearText.setVisibility(View.GONE);
        } else {
            isClearTextShown = true;
            isShowPasswordShown = false;
            imgShowPassword.setVisibility(View.GONE);
            imgClearText.setVisibility(View.VISIBLE);
        }

        if (isClearTextShown) {
            int defaultImage = this.attributes.getResourceId(R.styleable.CombinedEditText_defaultImage, -1);
            if (defaultImage != -1) {
                if (GeneralSettings.INSTANCE.isScanVisibility()) {
                    clearTextWatcher = new SwitchViewWatcher(imgClearText, imgDefault, GONE);
                } else {
                    clearTextWatcher = new SwitchViewWatcher(imgClearText, imgDefault);
                }
                this.setIcon(imgDefault, R.styleable.CombinedEditText_defaultImage);
            } else {
                clearTextWatcher = new SwitchViewWatcher(imgClearText);
            }
            edit.addTextChangedListener(clearTextWatcher);
        }
        edit.setOnFocusChangeListener(this);
        edit.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (keyboard != null) {
                    keyboard.showKeyboard();
                }
            }
        });

        imgClearText.setOnClickListener(this);
        imgIcon.setOnClickListener(this);

        if (isShowPasswordShown) {
            clearTextWatcher = new SwitchViewWatcher(imgClearText, imgShowPassword, GONE);
            edit.addTextChangedListener(clearTextWatcher);
        }
        imgShowPassword.setOnTouchListener(this);
    }

    /**
     * 设置hint
     */
    public void setHint(String text) {
        edit.setHint(text);
    }

    /**
     * 获取文本编辑器
     *
     * @return
     */
    public EditText getEdit() {
        return this.edit;
    }

    /**
     * 设置点击默认按钮的回调函数。
     *
     * @param l 单击事件回调函数
     */
    @Override
    public void setOnClickListener(OnClickListener l) {
        imgDefault.setOnClickListener(l);
    }

    /**
     * 返回输入的字符串序列。
     *
     * @return 字符串序列
     */
    public CharSequence getText() {
        return this.edit.getText();
    }

    /**
     * 返回 {@code KeyBoardView} 自定义软键盘的实例。如果未设置软键盘，则初始化一个新的实例。
     *
     * @return 自定义软键盘的实例
     */
    public KeyBoardView getKeyboard() {
        if (this.keyboard == null) {
            KeyBoardView instance = new KeyBoardView(this.edit);
            instance.showKeyboard();
            this.setKeyboard(instance);
        }
        return this.keyboard;
    }

    /**
     * 设置自定义软键盘的实例。
     *
     * @param keyboard 自定义软键盘的实例
     */
    public void setKeyboard(KeyBoardView keyboard) {
        this.keyboard = keyboard;
        SoftInput.disable(this.edit);
    }

    /**
     * 设置编辑器文本。
     *
     * @param s 字符串序列
     */
    public void setText(CharSequence s) {
        this.edit.setText(s);
        this.edit.setSelection(this.edit.length());
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.imgClearText:
                edit.setText("");
                break;
            case R.id.imgIcon:
                edit.requestFocus();
                edit.setSelection(edit.length());
                this.changeKeyboard();
                break;
        }
    }

    @Override
    public void onFocusChange(View v, boolean hasFocus) {
        if (hasFocus) {
            this.setIcon(R.styleable.CombinedEditText_iconFocused);
            vwBottomLine.setVisibility(View.GONE);
            vwBottomLineFocused.setVisibility(View.VISIBLE);
            this.updateClearTextVisibility();
            this.updateShowPasswordVisibility();
            this.changeKeyboard();
        } else {
            this.setIcon(R.styleable.CombinedEditText_icon);
            vwBottomLine.setVisibility(View.VISIBLE);
            vwBottomLineFocused.setVisibility(View.GONE);
            imgClearText.setVisibility(View.GONE);
            imgShowPassword.setVisibility(View.GONE);
        }
    }

    @Override
    public boolean onTouch(View v, MotionEvent event) {
        if (v.getId() == R.id.imgShowPassword) {
            this.showPassword(event);
        }

        return false;
    }

    /**
     * 是否展示默认图标
     */
    public void setDefaultIconVisibility(int visibility) {
        imgDefault.setVisibility(visibility);
    }

    /**
     * 切换自定义软键盘的文本编辑器。
     */
    private void changeKeyboard() {
        if (this.keyboard != null) {
            this.keyboard.change(this.edit);
            this.keyboard.showKeyboard();
        }
    }

    private static boolean isTypeofPassword(int inputTypeAttr, int inputType) {
        return (inputTypeAttr & inputType) == inputType;
    }

    /**
     * 设置图标。
     *
     * @param index 图标资源的索引
     */
    private void setIcon(int index) {
        this.setIcon(imgIcon, index);
    }

    /**
     * 设置图标。
     *
     * @param v     图标视图
     * @param index 图标资源的索引
     */
    private void setIcon(ImageView v, int index) {
        int iconDrawable = this.attributes.getResourceId(index, -1);
        if (iconDrawable != -1) {
            v.setImageResource(iconDrawable);
        }
    }

    /**
     * 显示密码。
     *
     * @param event MotionEvent
     */
    private void showPassword(MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            edit.setTransformationMethod(HideReturnsTransformationMethod.getInstance());
        } else if (event.getAction() == MotionEvent.ACTION_UP) {
            edit.setTransformationMethod(PasswordTransformationMethod.getInstance());
        }
        edit.postInvalidate();
    }

    /**
     * 更新文本清除按钮显示状态。
     */
    private void updateClearTextVisibility() {
        if (isClearTextShown && this.clearTextWatcher != null) {
            this.clearTextWatcher.update();
        }
    }

    /**
     * 更新密码显示按钮显示状态。
     */
    private void updateShowPasswordVisibility() {
        if (isShowPasswordShown) {
            if (edit.getText().length() > 0) {
                imgShowPassword.setVisibility(View.VISIBLE);
            } else {
                imgShowPassword.setVisibility(View.GONE);
            }
        }
    }
}
