package com.hisense.hiaposcmp.pos.ccb;

import android.graphics.Bitmap;
import android.graphics.Color;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * 图片转换转换工具<br/>
 * 利用算法把PNG和JPG等格式的图片转换成打印机可打印的单色bmp图片数据。
 *
 * @author xieliyan
 */
public class ImageTransformer {

    private ImageTransformer() {
    }

    /**
     * 转换成单色bmp图片数据
     *
     * @param bitmap - bitmap图片数据，PNG和JPG等格式
     * @return 单色bmp图片数据
     */
    public static ByteArrayOutputStream convert1BitBmp(Bitmap bitmap) {
        if (bitmap == null) {
            throw new IllegalArgumentException();
        }

        // 位图大小
        int nBmpWidth = bitmap.getWidth();
        int nBmpHeight = bitmap.getHeight();
        // 图像数据大小
        int wid = nBmpWidth % 32;
        if (wid > 0) {
            wid = 32 - nBmpWidth % 32;
        }
        wid += nBmpWidth;
        // 宽度必须是4的倍数，位图数据是以4个字节为一组来处理的。
        int bufferSize = nBmpHeight * wid / 8;
        ByteArrayOutputStream os;
        try {
            // 存储文件名
            os = new ByteArrayOutputStream();
            // bmp文件头
            int bfType = 0x4d42;
            long bfSize = 14 + 40 + 2 * 4 + bufferSize;
            int bfReserved1 = 0;
            int bfReserved2 = 0;
            long bfOffBits = 14 + 40 + 2 * 4;
            // 保存bmp文件头
            writeWord(os, bfType);
            writeDword(os, bfSize);
            writeWord(os, bfReserved1);
            writeWord(os, bfReserved2);
            writeDword(os, bfOffBits);
            // bmp信息头
            long biSize = 40L;
            int biPlanes = 1;
            int biBitCount = 1;
            long biCompression = 0L;
            long biXpelsPerMeter = 0L;
            long biYPelsPerMeter = 0L;
            long biClrUsed = 0L;
            long biClrImportant = 2L;
            // 保存bmp信息头
            writeDword(os, biSize);
            writeLong(os, nBmpWidth);
            writeLong(os, nBmpHeight);
            writeWord(os, biPlanes);
            writeWord(os, biBitCount);
            writeDword(os, biCompression);
            writeDword(os, bufferSize);
            writeLong(os, biXpelsPerMeter);
            writeLong(os, biYPelsPerMeter);
            writeDword(os, biClrUsed);
            writeDword(os, biClrImportant);

            // 调色板
            byte rgbBlue0 = (byte) 0xff;
            byte rgbGreen0 = (byte) 0xff;
            byte rgbRed0 = (byte) 0xff;
            byte rgbReserved0 = 0x0;
            byte rgbBlue1 = (byte) 0x00;
            byte rgbGreen1 = (byte) 0x00;
            byte rgbRed1 = (byte) 0x00;
            byte rgbReserved1 = (byte) 0x00;
            // 保存调色板
            writeByte(os, rgbBlue0);
            writeByte(os, rgbGreen0);
            writeByte(os, rgbRed0);
            writeByte(os, rgbReserved0);
            writeByte(os, rgbBlue1);
            writeByte(os, rgbGreen1);
            writeByte(os, rgbRed1);
            writeByte(os, rgbReserved1);

            // 像素扫描
            int threshold = getThreshold(bitmap);
            byte bmpData[] = new byte[bufferSize];
            for (int r = 0; r < nBmpHeight; ++r) {
                for (int c = 0; c < nBmpWidth; ) {
                    int color0 = getPixel(bitmap, c++, r, threshold);

                    int color1 = getPixel(bitmap, c++, r, threshold);

                    int color2 = getPixel(bitmap, c++, r, threshold);

                    int color3 = getPixel(bitmap, c++, r, threshold);

                    int color4 = getPixel(bitmap, c++, r, threshold);

                    int color5 = getPixel(bitmap, c++, r, threshold);

                    int color6 = getPixel(bitmap, c++, r, threshold);

                    int color7 = getPixel(bitmap, c, r, threshold);


                    bmpData[(nBmpHeight - r - 1) * (wid / 8) + (c++) / 8] = (byte) ((color0 << 7) | (color1 << 6)
                            | (color2 << 5) | (color3 << 4) | (color4 << 3) | (color5 << 2) | (color6 << 1) | (color7));

                }
            }
            os.write(bmpData);
            os.flush();
            return os;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    private static int getPixel(Bitmap bitmap, int x, int y, int threshHold) {
        if (x < bitmap.getWidth() && y < bitmap.getHeight()) {
            int clr = bitmap.getPixel(x, y);
            int gray = (int) ((double) Color.red(clr) * 0.3 + (double) Color.green(clr) * 0.59
                    + (double) Color.blue(clr) * 0.11);
            return gray > threshHold ? 0 : 1;
        } else {
            return 1;
        }
    }

    private static int getThreshold(Bitmap bmp) {
        double[] histGram = new double[256];
        for (int i = 0; i != 256; i++) {
            histGram[i] = 0;
        }

        for (int c = 0, r = bmp.getHeight() - 1; c < bmp.getHeight(); ++c, --r) {
            for (int wRow = 0; wRow < bmp.getWidth(); wRow++) {
                // 获取像素点
                int clr = bmp.getPixel(wRow, c);
                int red = Color.red(clr);
                int blue = Color.blue(clr);
                int green = Color.green(clr);

                // 用公式X = 0.3×R+0.59×G+0.11×B计算出X代替原来的RGB
                int gray = (int) ((float) red * 0.3 + (float) green * 0.59 + (float) blue * 0.11);
                histGram[gray]++;
            }
        }

        int k;
        double m1, m2, sum, cSum, fMax, sb;
        int threshValue = 1;
        sum = cSum = 0.0;
        int n = 0, n1, n2;
        for (k = 0; k <= 255; k++) {
            sum += (double) k * histGram[k];
            n += histGram[k];
        }
        fMax = -1.0;
        n1 = 0;
        for (k = 0; k < 255; k++) {
            n1 += histGram[k];
            if (n1 == 0) {
                continue;
            }
            n2 = n - n1;
            if (n2 == 0) {
                break;
            }
            cSum += (double) k * histGram[k];
            m1 = cSum / n1;
            m2 = (sum - cSum) / n2;
            sb = (double) n1 * (double) n2 * (m1 - m2) * (m1 - m2);
            if (sb > fMax) {
                fMax = sb;
                threshValue = k;
            }
        }
        return threshValue;
    }

    private static void writeByte(OutputStream stream, int value) throws IOException {
        byte[] b = new byte[1];
        b[0] = (byte) (value & 0xff);
        stream.write(b);
    }

    private static void writeWord(OutputStream stream, int value) throws IOException {
        byte[] b = new byte[2];
        b[0] = (byte) (value & 0xff);
        b[1] = (byte) (value >> 8 & 0xff);
        stream.write(b);
    }

    private static void writeDword(OutputStream stream, long value) throws IOException {
        byte[] b = new byte[4];
        b[0] = (byte) (value & 0xff);
        b[1] = (byte) (value >> 8 & 0xff);
        b[2] = (byte) (value >> 16 & 0xff);
        b[3] = (byte) (value >> 24 & 0xff);
        stream.write(b);
    }

    private static void writeLong(OutputStream stream, long value) throws IOException {
        byte[] b = new byte[4];
        b[0] = (byte) (value & 0xff);
        b[1] = (byte) (value >> 8 & 0xff);
        b[2] = (byte) (value >> 16 & 0xff);
        b[3] = (byte) (value >> 24 & 0xff);
        stream.write(b);
    }
}
