/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.service.payment;

import android.content.Context;
import android.database.Cursor;

import com.hisense.database.sqlite.QueryCallback;
import com.hisense.database.sqlite.SQLiteRepository;
import com.hisense.hiaposcmp.db.PadPosDatabase;
import com.hisense.hipos.payment.PayMethod;

import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * 支付方式资源库。
 *
 * @author zhangsai 2018-01-12 HIPOSAPP-272  支付方式按使用频次排序
 * @author zhangsai 2019-06-17 HIPOSAPP-1174 使用 PadPosDatabase 初始化 SQLite 数据库
 */
public class PayMethodRepository extends SQLiteRepository {

    /**
     * 初始化一个 {@code PayMethodRepository} 的实例。
     *
     * @param context Context
     */
    public PayMethodRepository(Context context) {
        super(new PadPosDatabase(context));
    }


    /**
     * 按使用频次对支付方式进行排序。
     *
     * @param payMethods 支付方式列表
     * @return 按使用频次由高到低排序后的支付方式列表
     */
    public List<PayMethod> sortByFrequency(final List<PayMethod> payMethods) {
        if (payMethods == null || payMethods.size() == 0) {
            return payMethods;
        }

        List<PayMethod> result = null;

        // 排序前支付方式集合
        final HashMap<String, PayMethod> map = new HashMap<>();
        for (PayMethod method : payMethods) {
            map.put(method.code(), method);
        }

        // 排序后集合
        final LinkedHashMap<String, PayMethod> sortedPayMethods = new LinkedHashMap<>();

        try {
            String sql = "SELECT Code,Freq FROM tPayMethod ORDER BY Freq DESC, Code";

            // 按频次排序
            this.executeCursor(sql, new QueryCallback() {
                @Override
                public void onSuccess(Cursor cursor) {
                    do {
                        String code = cursor.getString(0);
                        PayMethod method = map.get(code);
                        if (method != null) {
                            sortedPayMethods.put(code, map.get(code));
                        }
                    } while (cursor.moveToNext());
                }
            });

            // 添加本地数据库不存在支付方式
            for (PayMethod method : payMethods) {
                if (!sortedPayMethods.containsKey(method.code())) {
                    sortedPayMethods.put(method.code(), method);
                }
            }

            result = Arrays.asList(sortedPayMethods.values().toArray(new PayMethod[sortedPayMethods.size()]));
        } catch (Exception e) {

        } finally {
            map.clear();
            sortedPayMethods.clear();
        }

        return result;
    }

    /**
     * 更新 {@code method} 指定支付方式的使用频次。
     *
     * @param method 支付方式
     */
    public void updateFrequency(PayMethod method) {
        String sql = "REPLACE INTO tPayMethod (Code, Freq) VALUES (?,?)";
        int freq = this.getFrequency(method);
        this.execute(sql, method.code(), String.valueOf(freq + 1));
    }

    /**
     * 返回 {@code method} 指定支付方式的使用频次。
     *
     * @param method 支付方式
     * @return 支付方式的使用频次。默认为 0。
     */
    private int getFrequency(PayMethod method) {
        String sql = "SELECT Freq FROM tPayMethod WHERE Code = ? ";
        String value = this.executeScalar(sql, method.code());
        return this.getInteger(value, 0);
    }
}
