/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.service.payment;

import android.app.Activity;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;

import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.BuildConfig;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hipos.payment.PayMethod;

import java.util.ArrayList;
import java.util.List;

/**
 * 支付方式工具类。
 *
 * @author zhangsai 2017-12-22
 * @author zhangsai 2018-01-16 HIPOSAPP-277 调整支付方式按钮布局
 * @author dukunbiao(null) 2018-05-15 适配自助，支付方式布局调整
 */
public final class PaymentHelper {

    private static LinearLayout.LayoutParams lpDefault = new LinearLayout.LayoutParams(
            LinearLayout.LayoutParams.MATCH_PARENT,
            LinearLayout.LayoutParams.WRAP_CONTENT);

    private static LinearLayout.LayoutParams lpPayMethod = new LinearLayout.LayoutParams(
            0,
            LinearLayout.LayoutParams.MATCH_PARENT,
            1f);

    private static List<PayMethod> mCommonUsedPayMethods;           // 常用支付方式列表

    /**
     * 根据支付方式的个数创建一个包含一行、两行或包含展开按钮的精简布局。
     *
     * @param activity   布局所在的 Activity 实例
     * @param payMethods 支付方式
     * @param listener   支付方式点击的处理程序
     */
    public static void createCompactLayout(Activity activity, List<PayMethod> payMethods, View.OnClickListener listener) {
        if (mCommonUsedPayMethods == null) {
            mCommonUsedPayMethods = new ArrayList<>();
        } else {
            mCommonUsedPayMethods.clear();
        }

        RelativeLayout lytPayMethodsArea = activity.findViewById(R.id.lytPayMethodsArea);
        LinearLayout lytExpand = activity.findViewById(R.id.lytExpand);
        if (payMethods.size() <= 4) {
            // 显示一行
            lytPayMethodsArea.getLayoutParams().height = CommonUtils.dp2Px(activity, 64);
            lytExpand.setVisibility(View.GONE);
            mCommonUsedPayMethods.addAll(payMethods);
        } else if (payMethods.size() <= 8) {
            // 显示两行
            lytExpand.setVisibility(View.GONE);
            mCommonUsedPayMethods.addAll(payMethods);
        } else {
            // 显示两行，以及展开按钮
            lytPayMethodsArea.getLayoutParams().height = CommonUtils.dp2Px(activity, 158);
            lytExpand.setVisibility(View.VISIBLE);
            if (payMethods.size() < 12) {
                mCommonUsedPayMethods.addAll(payMethods);
            } else {
                mCommonUsedPayMethods.addAll(payMethods.subList(0, 12));
            }
        }

        createLayout(activity, mCommonUsedPayMethods, listener, null);
    }

    public static void createFullLayout(Activity activity, List<PayMethod> payMethods, View.OnClickListener listener) {
        PayMethod defaultSelected = null;
        if (payMethods != null && payMethods.size() > 0) {
            defaultSelected = payMethods.get(0);
        }

        createLayout(activity, payMethods, listener, defaultSelected);
    }

    /**
     * 创建支付方式布局。
     * 1、支付方式依次排列显示。
     * 2、每行最多显示 4 个支付方式，超出 4 个支付方式后折行。
     *
     * @param activity   布局所在的 Activity 实例
     * @param payMethods 支付方式
     * @param listener   支付方式点击的处理程序
     * @param selected   当前选中的支付方式。null 表示当前未选中任何支付方式
     * @return 选中的支付方式的布局控件。
     */
    private static LinearLayout createLayout(Activity activity, List<PayMethod> payMethods,
                                             View.OnClickListener listener, PayMethod selected) {
        LinearLayout lytSelected = null;
        LinearLayout lytLine = null;

        lpDefault.height = CommonUtils.dp2Px(activity, 64);
        lpPayMethod.height = CommonUtils.dp2Px(activity, 64);

        LinearLayout lytPayMethods = activity.findViewById(R.id.lytPayMethods);
        lytPayMethods.removeAllViews();

        try {
            for (int i = 0; i < payMethods.size(); i++) {
                PayMethod payMethod = payMethods.get(i);

                LinearLayout lytItem = createItemLayout(activity);
                lytItem.setOnClickListener(listener);
                lytItem.setTag(payMethod);

                lytItem.addView(createEmptyView(activity));

                ImageView imgItem = createImageButton(activity, payMethod);
                lytItem.addView(imgItem);

                TextView textView = createTextView(activity, payMethod);
                lytItem.addView(textView);

                if (selected != null && selected.code().equals(payMethod.code())) {
                    lytSelected = lytItem;
                }

                if (i % 4 == 0) {
                    lytLine = createLineLayout(activity);
                    lytPayMethods.addView(lytLine);
                }
                lytLine.addView(lytItem);
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("动态产生支付方式布局异常:" + e.toString());
        }

        return lytSelected;
    }

    /**
     * 创建支付方式按钮布局。
     *
     * @param activity Activity
     * @return LinearLayout
     */
    private static LinearLayout createItemLayout(Activity activity) {
        LinearLayout lytItem = new LinearLayout(activity);
        lytItem.setLayoutParams(lpPayMethod);
        lytItem.setOrientation(LinearLayout.VERTICAL);
        lytItem.setBackgroundResource(R.drawable.border);

        return lytItem;
    }

    /**
     * 创建支付方式按钮占位视图。
     *
     * @param activity Activity
     * @return View
     */
    private static View createEmptyView(Activity activity) {
        View view = new View(activity);
        view.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, CommonUtils.dp2Px(activity, 8)));

        return view;
    }

    /**
     * 创建支付方式按钮图标视图。
     *
     * @param activity  Activity
     * @param payMethod 对应的支付方式
     * @return ImageView
     */
    private static ImageView createImageButton(Activity activity, PayMethod payMethod) {
        ImageView imgPayMethod = new ImageView(activity);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                CommonUtils.dp2Px(activity, 24));
        imgPayMethod.setLayoutParams(layoutParams);
        imgPayMethod.setImageDrawable(ContextCompat.getDrawable(activity, PayMethodService.getIconByCode(payMethod)));

        return imgPayMethod;
    }

    /**
     * 创建支付方式按钮文本视图。
     *
     * @param activity  Activity
     * @param payMethod 对应的支付方式
     * @return TextView
     */
    private static TextView createTextView(Activity activity, PayMethod payMethod) {
        TextView textView = new TextView(activity);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                CommonUtils.dp2Px(activity, 32));
        textView.setLayoutParams(layoutParams);
        textView.setGravity(Gravity.CENTER);
        textView.setTextSize(14);

        textView.setText(StringUtils.substringValue(payMethod.name(), 6, "L"));

        return textView;
    }

    /**
     * 创建支付方式行布局。每行最多显示 4 个支付方式。
     *
     * @param activity Activity
     * @return LinearLayout
     */
    private static LinearLayout createLineLayout(Activity activity) {
        LinearLayout lytLine = new LinearLayout(activity);
        lytLine.setWeightSum(4);
        lytLine.setLayoutParams(lpDefault);
        lytLine.setOrientation(LinearLayout.HORIZONTAL);

        return lytLine;
    }

    /**
     * 选中选择的支付方式，并重新布局。
     *
     * @param activity 布局所在的 Activity 实例
     * @param method   选择的支付方式
     * @param listener 支付方式点击的处理程序
     */
    public static void selectPayMethod(Activity activity, PayMethod method, View.OnClickListener listener) {
        // 检查选择的支付方式是否在常用支付方式列表内，及其位置。
        int index = -1;
        for (int i = 0; i < mCommonUsedPayMethods.size(); i++) {
            if (mCommonUsedPayMethods.get(i).code().equals(method.code())) {
                index = i;
                break;
            }
        }

        if (index == -1) {
            // 如果不存在，则直接添加到常用支付方式列表可见位置的末尾；
            mCommonUsedPayMethods.remove(7);
            mCommonUsedPayMethods.add(7, method);
        } else if (index > 7) {
            // 如果位置在可见范围之外，则移动到可见位置的末尾；
            mCommonUsedPayMethods.remove(index);
            mCommonUsedPayMethods.add(7, method);
        } else {
            // 其它情况表示选择的支付方式在可见范围内，无需调整
            // NO CODE HERE.
        }

        LinearLayout lytSelected = PaymentHelper.createLayout(
                activity, mCommonUsedPayMethods, listener, method);

        listener.onClick(lytSelected);
    }

    /**
     * 创建蚂蚁金服支付方式布局。
     *
     * @param activity   布局所在的 Activity 实例
     * @param payMethods 支付方式
     * @param listener   支付方式点击的处理程序
     * @return 支付方式完整布局。
     */
    public static void createAliLayout(Activity activity, List<PayMethod> payMethods, View.OnClickListener listener) {
        PayMethod defaultSelected = null;
        if (payMethods != null && payMethods.size() > 0) {
            defaultSelected = payMethods.get(0);
        }

        createAliLayout(activity, payMethods, listener, defaultSelected);
    }

    /**
     * 创建蚂蚁金服支付方式布局。
     * 只显示刷脸付、支付宝支付
     *
     * @param activity   布局所在的 Activity 实例
     * @param payMethods 支付方式
     * @param listener   支付方式点击的处理程序
     * @param selected   当前选中的支付方式。null 表示当前未选中任何支付方式
     * @return 选中的支付方式的布局控件。
     */
    private static LinearLayout createAliLayout(Activity activity, List<PayMethod> payMethods,
                                                View.OnClickListener listener, PayMethod selected) {
        LinearLayout lytSelected = null;
        LinearLayout lytPayMethodsAnt = activity.findViewById(R.id.layoutPayment);
        lytPayMethodsAnt.removeAllViews();

        try {
            for (int i = 0; i < payMethods.size(); i++) {
                PayMethod payMethod = payMethods.get(i);

                if (!payMethod.isAliFacePay() && !payMethod.isAliPay()) {
                    continue;
                }
                //此处做一个分支选择（普通大屏竖版自助 或者 8自连锁商米T2机器横屏版台式自助）
                LinearLayout lytItem ;
                if(CommonUtils.isAliLandStyle()){
                    lytItem = createAliLandItemLayout(activity, i);
                } else{
                    lytItem = createAliItemLayout(activity, i);
                }

                lytItem.setOnClickListener(listener);
                lytItem.setTag(payMethod);

                RelativeLayout relativeItem = createAliItemRelative(activity, i);
                lytItem.addView(relativeItem);

                ImageView imgItem = createAliImageButton(activity, payMethod);
                relativeItem.addView(imgItem);

                TextView textItem = createAliTextView(activity, payMethod);
                relativeItem.addView(textItem);

                lytPayMethodsAnt.addView(lytItem);
                if (selected != null && selected.code().equals(payMethod.code())) {
                    lytSelected = lytItem;
                }
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("自助动态产生支付方式布局异常:" + e.toString());
        }

        return lytSelected;
    }

    /**
     * 创建蚂蚁金服支付方式按钮布局。
     *
     * @param activity Activity
     * @return LinearLayout
     */
    private static LinearLayout createAliItemLayout(Activity activity, int buttonIndex) {
        float dpValue = 729;
        if (BuildConfig.DEBUG) {
            dpValue = 129;
        }

        LinearLayout lytItem = new LinearLayout(activity);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(CommonUtils.dp2Px(activity, 729),
                CommonUtils.dp2Px(activity, dpValue));
        if (buttonIndex != 0) {
            layoutParams.setMargins(0, 48, 0, 0);
        }

        lytItem.setLayoutParams(layoutParams);
        lytItem.setOrientation(LinearLayout.HORIZONTAL);
        lytItem.setBackgroundResource(R.drawable.border);

        return lytItem;
    }

    /**
     * 创建蚂蚁金服支付方式按钮布局。(适用8字连锁商米T2机器横屏版)
     *
     * @param activity Activity
     * @return LinearLayout
     */
    private static LinearLayout createAliLandItemLayout(Activity activity, int buttonIndex) {
        float dpValue = 729;
        if (BuildConfig.DEBUG) {
            dpValue = 129;
        }

        LinearLayout lytItem = new LinearLayout(activity);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(450,
                450);
        if (buttonIndex != 0) {
            layoutParams.setMargins(300, 200, 0, 0);
        }

        lytItem.setLayoutParams(layoutParams);
        lytItem.setOrientation(LinearLayout.HORIZONTAL);
        lytItem.setBackgroundResource(R.drawable.border);

        return lytItem;
    }


    /**
     * 创建蚂蚁金服支付方式按钮布局。
     *
     * @param activity Activity
     * @return LinearLayout
     */
    private static RelativeLayout createAliItemRelative(Activity activity, int buttonIndex) {
        RelativeLayout lytItem = new RelativeLayout(activity);
        RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        lytItem.setLayoutParams(layoutParams);

        return lytItem;
    }

    /**
     * 创建蚂蚁金服支付方式按钮图标视图。
     *
     * @param activity  Activity
     * @param payMethod 对应的支付方式
     * @return ImageView
     */
    private static ImageView createAliImageButton(Activity activity, PayMethod payMethod) {
        ImageView imgPayMethod = new ImageView(activity);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        imgPayMethod.setLayoutParams(layoutParams);

        imgPayMethod.setImageDrawable(ContextCompat.getDrawable(activity, PayMethodService.getAliPayMethodIcon(payMethod)));

        return imgPayMethod;
    }

    private static TextView createAliTextView(Activity activity, PayMethod payMethod) {
        TextView textView = new TextView(activity);

        if ((payMethod.isAliFacePay()) && PayMethodService.isAliFacePayLimit()) {
            LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
            textView.setLayoutParams(layoutParams);
            textView.setGravity(Gravity.CENTER);
            textView.setTextSize(40);
            textView.setText("\n" + "消费总金额超过" + GeneralSettings.INSTANCE.getAliFacePayLimit() + "元" + "\n" + "不支持刷脸支付");
        }

        return textView;
    }

    /**
     * 创建微信付方式布局。
     * 只显示刷脸付、微信支付
     *
     * @param activity   布局所在的 Activity 实例
     * @param payMethods 支付方式
     * @param listener   支付方式点击的处理程序
     * @param selected   当前选中的支付方式。null 表示当前未选中任何支付方式
     * @return 选中的支付方式的布局控件。
     */
    private static LinearLayout createWxLayout(Activity activity, List<PayMethod> payMethods,
                                               View.OnClickListener listener, PayMethod selected) {
        LinearLayout lytSelected = null;

        LinearLayout lytPayMethodsWx = activity.findViewById(R.id.linPayment);
        lytPayMethodsWx.removeAllViews();

        try {
            for (int i = 0; i < payMethods.size(); i++) {
                PayMethod payMethod = payMethods.get(i);

                if (!payMethod.isWxPay() && !payMethod.isWxFacePay()) {
                    continue;
                }

                LinearLayout lytItem = createWxItemLayout(activity, i);

                LinearLayout lytPay = createWxPayLayout(activity, payMethod);
                lytPay.setOnClickListener(listener);
                lytPay.setTag(payMethod);

                ImageView imgItem = getWxPayMethodIcon(activity, payMethod);
                lytPay.addView(imgItem);

                TextView textItem = createWxTextView(activity, payMethod);
                lytPay.addView(textItem);

                lytItem.addView(lytPay);

                lytPayMethodsWx.addView(lytItem);
                if (selected != null && selected.code().equals(payMethod.code())) {
                    lytSelected = lytItem;
                }
            }

        } catch (Exception e) {
            LoggerUtils.getLogger().error("自助动态产生支付方式布局异常:" + e.getMessage());
        }

        return lytSelected;
    }

    /**
     * 创建微信支付方式按钮布局
     *
     * @param activity
     * @param buttonIndex
     * @return
     */
    private static LinearLayout createWxItemLayout(Activity activity, int buttonIndex) {
        LinearLayout lytItem = new LinearLayout(activity);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(CommonUtils.dp2Px(activity, 0), ViewGroup.LayoutParams.MATCH_PARENT);
        layoutParams.weight = 1;
        layoutParams.gravity = Gravity.CENTER;

        lytItem.setLayoutParams(layoutParams);
        lytItem.setOrientation(LinearLayout.VERTICAL);

        return lytItem;
    }

    /**
     * 创建微信支付方式按钮布局
     *
     * @param activity
     * @param payMethod
     * @return
     */
    private static LinearLayout createWxPayLayout(Activity activity, PayMethod payMethod) {
        LinearLayout lytPayItem = new LinearLayout(activity);

        if (payMethod.isWxFacePay()) {
            LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(
                    CommonUtils.dp2Px(activity, 217), ViewGroup.LayoutParams.MATCH_PARENT);

            lytPayItem.setLayoutParams(layoutParams);
            lytPayItem.setOrientation(LinearLayout.VERTICAL);
        } else if (payMethod.isWxPay()) {
            LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(
                    CommonUtils.dp2Px(activity, 214), ViewGroup.LayoutParams.MATCH_PARENT);

            lytPayItem.setLayoutParams(layoutParams);
            lytPayItem.setOrientation(LinearLayout.VERTICAL);
        } else {

        }

        return lytPayItem;
    }

    private static ImageView getWxPayMethodIcon(Activity activity, PayMethod payMethod) {
        ImageView imgPayMethod = new ImageView(activity);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        imgPayMethod.setLayoutParams(layoutParams);
        imgPayMethod.setImageDrawable(ContextCompat.getDrawable(activity, PayMethodService.getWxPayMethodIcon(payMethod)));

        return imgPayMethod;
    }

    /**
     * 创建微信支付方式文字提示
     *
     * @param activity
     * @param payMethod
     * @return
     */
    private static TextView createWxTextView(Activity activity, PayMethod payMethod) {
        TextView textView = new TextView(activity);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        textView.setLayoutParams(layoutParams);
        layoutParams.setMargins(0, 48, 0, 0);
        textView.setGravity(Gravity.CENTER);
        textView.setTextSize(45);

        if (payMethod.isWxFacePay()) {
            textView.setText("刷脸支付");
        } else if (payMethod.isWxPay()) {
            textView.setText("扫码支付");
        }

        return textView;
    }
}
