
package com.hisense.hiaposcmp.service.sale;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.database.SQLException;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;

import com.hisense.common.boot.IteratorCallback;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.pay.AliConstants;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.hipos.webapi.sale.SaleSubmitResultData;
import com.hisense.webapi.Callback;
import com.hisense.webapi.WebApiException;

import org.greenrobot.eventbus.EventBus;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * 交易流水应用服务。
 *
 * @author zhangsai 2018-12-24 HIPOSAPP-931
 * @author guolina  2019-04-10 HIPOSAPP-1065 支持清理本地三个月前已上传流水信息
 * @author guolina  2019-10-21 HIPOSAPP-1317 利客来未上传流水自动上传
 * @author guolina  2019-11-07 HIPOSAPP-1338 使用EventBus在自动上传流水结束后刷新销售界面红点
 * @author wangyaogang 2020-03-23 HIPOSAPP-1392 调整删除过期流水逻辑（改为删除销售日期三个月之前流水，不管是否成功）
 */
public class SaleApplicationService {

    private Context context;
    private SaleRepository saleRepository;

    /**
     * 初始化一个 {@code SaleApplicationService} 的实例。
     *
     * @param context Context
     */
    public SaleApplicationService(Context context) {
        this.context = context;
        this.saleRepository = new SaleRepository(context);
    }

    /**
     * 获取本地未上传流水,拿到需要的地方单独上传,以便使用相应风格的控件
     *
     * @return 流水集合
     */
    public List<BaseSaleModel> getNotUploadSales() {
        List<BaseSaleModel> saleModelList = new ArrayList<>();
        Collection<String> notUploadSales = saleRepository.getNotUploadSaleNo();
        for (String notUploadSale : notUploadSales) {
            saleModelList.add(saleRepository.get(notUploadSale));
        }
        return saleModelList;
    }

    /**
     * 返回本地未上传流水条数
     *
     * @return 本地未上传流水条数。
     */
    public int localUnUploadSize() {
        final Collection<String> notUploadSales = saleRepository.getNotUploadSaleNo();

        return notUploadSales.size();
    }

    /**
     * 对于未上传流水，采用逐笔上传的方式
     *
     * @return 如果存在本地未上传流水，则返回 true。否则返回 false。
     */
    public boolean uploadLocalData() {
        LoggerUtils.getLogger().debug("重新上传流水");

        final Collection<String> notUploadSales = saleRepository.getNotUploadSaleNo();
        if (notUploadSales.size() > 0) {
            new AlertDialog.Builder(this.context)
                    .setTitle("流水重传")
                    .setMessage(String.format(Locale.CHINESE, "发现 %d 条本地流水。是否上传流水？",
                            notUploadSales.size()))
                    .setPositiveButton("确定", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            final ProgressDialog progressDialog = ProgressDialog.show(
                                    SaleApplicationService.this.context, "流水重传", "请耐心等待…");

                            final Iterator<String> iterator = notUploadSales.iterator();
                            uploadAsync(iterator, progressDialog, new IteratorCallback() {
                                @Override
                                public void onNext(IteratorCallback callback) {
                                    uploadAsync(iterator, progressDialog, callback);
                                }

                                @Override
                                public void onFinish() {
                                    progressDialog.dismiss();
                                }
                            });
                        }
                    })
                    .setNegativeButton("取消", null)
                    .show();
        }

        return notUploadSales.size() > 0;
    }


    /**
     * 对于未上传流水，自动采用逐笔上传的方式
     *
     * @return 如果存在本地未上传流水，则返回 true。否则返回 false。
     */
    public boolean uploadLocalDataAuto() {
        LoggerUtils.getLogger().debug("自动重新上传流水");

        final Collection<String> notUploadSales = saleRepository.getNotUploadSaleNo();
        if (notUploadSales.size() > 0) {

            final Iterator<String> iterator = notUploadSales.iterator();
            uploadAsyncAuto(iterator, new IteratorCallback() {
                @Override
                public void onNext(IteratorCallback callback) {
                    uploadAsyncAuto(iterator, callback);
                    EventBus.getDefault().postSticky("refresh");
                }

                @Override
                public void onFinish() {
                    EventBus.getDefault().postSticky("refresh");
                }
            });
        }

        return notUploadSales.size() > 0;
    }

    /**
     * 异步上传流水。
     *
     * @param iterator 本地未上传流水的流水号迭代器
     * @param callback 迭代完成后的回调方法
     */
    private void uploadAsync(final Iterator<String> iterator, final ProgressDialog progressDialog,
                             final IteratorCallback callback) {
        final String saleNo = iterator.next();
        final BaseSaleModel sale = saleRepository.get(saleNo);

        progressDialog.setMessage("正在上传流水：" + saleNo);

        this.upload(sale, new Callback<SaleSubmitResultData>() {
            @Override
            public void onSuccess(SaleSubmitResultData o) {
                CommonTools.toastShort("流水重传成功：" + saleNo);
                LoggerUtils.getLogger().debug("流水重传成功" + saleNo);
                if (iterator.hasNext()) {
                    callback.onNext(callback);
                } else {
                    callback.onFinish();
                }
            }

            @Override
            public void onError(final Throwable throwable) {
                CommonTools.toastShort(String.format(Locale.CHINESE, "流水重传失败：%s[%s]",
                        throwable.getMessage(), sale.getSaleNo()));
                LoggerUtils.getLogger().debug(String.format(Locale.CHINESE, "流水重传失败：%s[%s]",
                        throwable.getMessage(), sale.getSaleNo()));
                if (iterator.hasNext()) {
                    callback.onNext(callback);
                } else {
                    callback.onFinish();
                }
            }
        });
    }


    /**
     * 自动异步上传流水。
     *
     * @param iterator 本地未上传流水的流水号迭代器
     * @param callback 迭代完成后的回调方法
     */
    private void uploadAsyncAuto(final Iterator<String> iterator,
                                 final IteratorCallback callback) {
        final String saleNo = iterator.next();
        final BaseSaleModel sale = saleRepository.get(saleNo);


        this.upload(sale, new Callback<SaleSubmitResultData>() {
            @Override
            public void onSuccess(SaleSubmitResultData o) {
                LoggerUtils.getLogger().debug("流水自动重传成功" + saleNo);
                if (iterator.hasNext()) {
                    callback.onNext(callback);
                } else {
                    callback.onFinish();
                }
            }

            @Override
            public void onError(final Throwable throwable) {
                LoggerUtils.getLogger().debug(String.format(Locale.CHINESE, "流水自动重传失败：%s[%s]",
                        throwable.getMessage(), sale.getSaleNo()));
                if (iterator.hasNext()) {
                    callback.onNext(callback);
                } else {
                    callback.onFinish();
                }
            }
        });
    }

    /**
     * 上传流水。
     *
     * @param sale     实现 {@code ISale} 接口的销售/退货流水
     * @param callback 执行上传流水后的回调方法
     */
    public void upload(final BaseSaleModel sale, final Callback<SaleSubmitResultData> callback) {
        try {
            SaleState saleState = saleRepository.getState(sale.getSaleNo());
            if (!saleState.equals(SaleState.VIP_FAILURE)) { // 流水上传失败（含会员流水）
                if (sale instanceof SaleModel) {
                    submit(false, (SaleModel) sale, callback);
                } else {
                    returns(false, (ReturnSaleModel) sale, callback);
                }
            } else { // 会员流水上传失败
                if (sale instanceof SaleModel) {
                    submit(true, (SaleModel) sale, callback);
                } else {
                    returns(true, (ReturnSaleModel) sale, callback);
                }
            }
        } catch (SQLException e) {
            LoggerUtils.getLogger().error("SQL异常：" + e.getMessage());
        }
    }

    /**
     * 上传销售流水。
     *
     * @param callback 执行上传流水后的回调方法
     */
    private void submit(boolean isToVip, SaleModel sale, final Callback<SaleSubmitResultData> callback) {
        boolean openRpt = GeneralSettings.INSTANCE.isOpenRpt();
        SaleService.submit(openRpt, isToVip, sale, new Callback<SaleSubmitResultData>() {
            @Override
            public void onSuccess(SaleSubmitResultData resultData) {
                saleRepository.updateState(sale.getSaleNo(), SaleState.SUCCESS);
                if (callback != null) {
                    callback.onSuccess(resultData);
                }
            }

            @Override
            public void onError(Throwable throwable) {
                checkError(sale, throwable, callback);
            }
        });
    }

    /**
     * 上传退货流水。
     *
     * @param isToVip  是否会员流水
     * @param callback 执行上传流水后的回调方法
     */
    private void returns(boolean isToVip, ReturnSaleModel sale, final Callback<SaleSubmitResultData> callback) {
        boolean openRpt = GeneralSettings.INSTANCE.isOpenRpt();
        SaleService.returns(openRpt, isToVip, sale, new Callback<Object>() {
            @Override
            public void onSuccess(Object o) {
                saleRepository.updateState(sale.getSaleNo(), SaleState.SUCCESS);
                saleRepository.updateState(sale.oldSaleNo(), SaleState.RETURN);
                if (callback != null) {
                    callback.onSuccess(null);
                }
            }

            @Override
            public void onError(Throwable throwable) {
                saleRepository.updateState(sale.oldSaleNo(), SaleState.RETURN);
                checkError(sale, throwable, callback);
            }
        });
    }

    /**
     * 检查并处理上传流水接口返回的异常。
     *
     * @param throwable 上传流水接口返回的异常
     * @param callback  执行上传流水后的回调方法
     */
    private void checkError(BaseSaleModel saleModel, Throwable throwable, final Callback<SaleSubmitResultData> callback) {
        WebApiException ex = (WebApiException) throwable;

        try {
            if (ex.code() == AliConstants.NOT_FOUND_IGNORE) {
                //code: NOT_FOUND_IGNORE,表示无此接口
                saleRepository.updateState(saleModel.getSaleNo(), SaleState.SUCCESS);
                if (callback != null) {
                    callback.onError(throwable);
                }
                return;
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("调用会员流水上传接口解析异常：" + e.toString());
        }

        if (ex.code() == 411) {
            if (saleModel.getVip() != null) {
                // 返回 411 表示 交易流水上传成功，会员流水上传失败
                saleRepository.updateState(saleModel.getSaleNo(), SaleState.VIP_FAILURE);
                if (callback != null) {
                    callback.onError(throwable);
                }
            } else {
                // 如果是非会员交易，返回 411 表示流水上传成功
                saleRepository.updateState(saleModel.getSaleNo(), SaleState.SUCCESS);
                if (callback != null) {
                    callback.onSuccess(null);
                }
            }

        } else if (ex.code() == 412) {
            // 上传流水重复时修改本地流水为完成
            saleRepository.updateState(saleModel.getSaleNo(), SaleState.SUCCESS);
            if (callback != null) {
                callback.onError(throwable);
            }

        } else {
            saleRepository.updateState(saleModel.getSaleNo(), SaleState.FAILURE);
            if (callback != null) {
                callback.onError(throwable);
            }
        }
    }

    /**
     * 对于3个月前已上传流水，采用逐笔删除的方式
     *
     * @return 如果存在3个月前已上传流水，则返回 true。否则返回 false。
     */
    public boolean deleteUploadedLocalData() {
        LoggerUtils.getLogger().info("删除本地3个月前已上传流水");

        final ArrayList<String> deleteSales = new ArrayList<>();

        Map<String, String> map;
        map = saleRepository.getUploadedSaleNo();

        for (Map.Entry<String, String> entry : map.entrySet()) {
            deleteSales.add(entry.getKey());
            LoggerUtils.getLogger().debug("超过90天的已上传流水saleNo = " + entry.getKey() + ", SaleDate = " + entry.getValue());
        }

        if (deleteSales.size() > 0) {
            ProgressDialog progressDialog = ProgressDialog.show(
                    SaleApplicationService.this.context, "流水删除", "请耐心等待…");

            Iterator<String> iterator = deleteSales.iterator();
            deleteAsync(iterator, progressDialog, new IteratorCallback() {
                @Override
                public void onNext(IteratorCallback callback) {
                    deleteAsync(iterator, progressDialog, callback);
                }

                @Override
                public void onFinish() {
                    progressDialog.dismiss();
                    LoggerUtils.getLogger().info("全部流水删除成功");
                }
            });
        }

        return deleteSales.size() > 0;
    }

    /**
     * 删除流水。
     *
     * @param iterator 本地已上传流水的流水号迭代器
     * @param callback 迭代完成后的回调方法
     */
    private void deleteAsync(final Iterator<String> iterator, final ProgressDialog progressDialog,
                             final IteratorCallback callback) {
        final String saleNo = iterator.next();

        progressDialog.setMessage("正在删除流水：" + saleNo);

        saleRepository.delete(saleNo);
        Toast.makeText(App.getContext(), "流水删除成功：" + saleNo, Toast.LENGTH_SHORT).show();

        LoggerUtils.getLogger().info("流水删除成功" + saleNo);
        if (iterator.hasNext()) {
            callback.onNext(callback);
        } else {
            callback.onFinish();
        }
    }

    public void reSubmit(UniCallback callback) {

        List<BaseSaleModel> notUploadSales = getNotUploadSales();
        if (ListUtils.isEmpty(notUploadSales)) {
            callback.onSuccess("流水已全部上传");
            return;
        }

        for (BaseSaleModel notUploadSale : notUploadSales) {
            try {
                String saleNo = notUploadSale.getSaleNo();
                CommonTools.toastShort("正在上传流水：" + saleNo + "，请稍后");
                upload(notUploadSale, new Callback<SaleSubmitResultData>() {
                    @Override
                    public void onSuccess(SaleSubmitResultData submitResultData) {
                        LoggerUtils.getLogger().debug("流水重传成功:" + saleNo);
                        callback.onSuccess("流水重传成功:" + saleNo);
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        LoggerUtils.getLogger().debug("流水重传失败:" + throwable.toString());
                        callback.onError("流水重传失败:" + throwable.getMessage());
                    }
                });
            } catch (Exception e) {
                LoggerUtils.getLogger().debug("获取重传流水信息异常:" + e.toString());
                callback.onError("获取重传流水信息异常:" + e.getMessage());
                break;
            }
        }


    }
}
