package com.hisense.common.util;

import com.hisense.common.constant.UtilConstants;

import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.util.UUID;
import java.util.regex.Pattern;

/**
 * String 工具类
 *
 * @author dukunbiao(null) 2018-06-01
 */
public class StringUtils {

    public static final String PAD_TYPE_RIGHT = "R";
    public static final String PAD_TYPE_LEFT = "L";

    private static Pattern PATTERN_IS_NUMERIC = Pattern.compile("-?[0-9]*+.?[0-9]*");
    private static Pattern PATTERN_IS_INTEGER = Pattern.compile("-?[0-9]*");

    /**
     * 字符串补全，补空格
     *
     * @param str 待补全字符串
     * @param len 补全后长度
     * @param tag R 右补全   L 左补全
     * @return 补全后字符串
     */
    public static String stringPadding(String str, int len, String tag) {
        if (str == null) {
            str = "";
        }
        int length = 0;
        try {
            length = str.getBytes(UtilConstants.CHARSET_GBK).length;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        if (length > len) {
            try {
                return StringUtils.subStringByBytes(str, len, UtilConstants.CHARSET_GBK, "", tag);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        }

        len = len - (length - str.length());

        String padFormat = "%s";
        if (PAD_TYPE_LEFT.equals(tag)) {
            padFormat = "%" + len + "s";
        } else if (PAD_TYPE_RIGHT.equals(tag)) {
            padFormat = "%-" + len + "s";
        }

        return String.format(padFormat, str);
    }

    /**
     * 字符串补全
     *
     * @param str  待补全字符串
     * @param len  补全后长度
     * @param fill 填充字符
     * @param tag  R 右补全   L 左补全
     * @return 补全后字符串
     */
    public static String stringPadding(String str, int len, String fill, String tag) {
        if (str == null) {
            str = "";
        }

        int length = 0;
        try {
            length = str.getBytes(UtilConstants.CHARSET_GBK).length;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        if (length > len) {
            try {
                return StringUtils.subStringByBytes(str, len, UtilConstants.CHARSET_GBK, fill, tag);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        } else if (length == len) {
            return str;
        }

        StringBuffer res = new StringBuffer();
        StringBuffer bf = new StringBuffer(str);

        if (PAD_TYPE_RIGHT.equals(tag)) {   //右补位
            for (int i = 0; i < len - length; i++) {
                res = bf.append(fill);
            }
        } else if (PAD_TYPE_LEFT.equals(tag)) { //左补位
            StringBuffer bf1 = new StringBuffer();
            for (int i = 0; i < len - length; i++) {
                res = bf1.append(fill);
            }
            res.append(bf);
        }
        return res.toString();
    }

    public static String intToString(int i) {
        return String.format("%s", i);
    }

    /**
     * 检查字符串是否为空
     * <p>
     * SysUtils.isEmpty(null) = true
     * SysUtils.isEmpty("") = true
     * SysUtils.isEmpty("   ") = true
     * SysUtils.isEmpty("abc") = false
     *
     * @param value
     * @return
     */
    public static boolean isEmpty(String value) {
        int strLength;
        if (value == null || (strLength = value.length()) == 0) {
            return true;
        }

        for (int i = 0; i < strLength; i++) {
            if (!Character.isWhitespace(value.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * 检查字符串列表是否不为空
     *
     * @param values
     * @return
     */
    public static boolean areNotEmpty(String... values) {
        boolean result = true;

        if (values == null || values.length == 0) {
            result = false;
        } else {
            for (String value : values) {
                result &= !isEmpty(value);
            }
        }

        return result;
    }

    /**
     * 检查对象是否为数字型字符串,包含负数开头的
     *
     * @param obj
     * @return
     */
    public static boolean isNumeric(Object obj) {
        if (obj == null) {
            return false;
        }
        char[] chars = obj.toString().toCharArray();
        int length = chars.length;
        if (length < 1) {
            return false;
        }

        int i = 0;
        if (length > 1 && chars[0] == '-') {
            i = 1;
        }

        for (; i < length; i++) {
            if (!Character.isDigit(chars[i])) {
                return false;
            }
        }
        return true;
    }

    /**
     * String转int，格式不正确则返回默认值
     *
     * @param value        字符串
     * @param defaultValue 默认值
     * @return 数值
     */
    public static int toInt(String value, int defaultValue) {
        try {
            if ((!isEmpty(value)) && (isInteger(BigDecimalUtils.format(value)))) {
                return Integer.parseInt(BigDecimalUtils.format(value));
            }
        } catch (NumberFormatException e) {
            return defaultValue;
        }

        return defaultValue;
    }

    public static double toDouble(String value, double defaultValue) {
        if ((!isEmpty(value)) && (isNumeric(value))) {
            return Double.parseDouble(value);
        }

        return defaultValue;
    }

    /**
     * 把通用字符编码的字符串转化为汉字编码。
     *
     * @param value
     * @return
     */
    public static String unicodeToChinese(String value) {
        StringBuilder out = new StringBuilder();
        if (!isEmpty(value)) {
            for (int i = 0; i < value.length(); i++) {
                out.append(value.charAt(i));
            }
        }
        return out.toString();
    }

    /**
     * 过滤不可见字符
     *
     * @param value
     * @return
     */
    public static String stripNonValidXMLCharacters(String value) {
        if (value == null || ("".equals(value))) {
            return "";
        }

        StringBuilder out = new StringBuilder();
        char current;
        for (int i = 0; i < value.length(); i++) {
            current = value.charAt(i);
            if ((current == 0x9) || (current == 0xA) || (current == 0xD)
                    || ((current >= 0x20) && (current <= 0xD7FF))
                    || ((current >= 0xE000) && (current <= 0xFFFD))
                    || ((current >= 0x10000) && (current <= 0x10FFFF))) {
                out.append(current);
            }
        }
        return out.toString();
    }

    /**
     * 检查String是否为数值型字符串，包含整数、小数
     *
     * @param value 待检查字符串
     * @return true/false
     */
    public static boolean isNumeric(String value) {
        return (!isEmpty(value)) && PATTERN_IS_NUMERIC.matcher(value).matches();
    }

    /**
     * 检查String是否是int型字符串
     *
     * @param value 待检查字符串
     * @return true/false
     */
    public static boolean isInteger(String value) {
        return (!isEmpty(value)) && PATTERN_IS_INTEGER.matcher(value).matches();
    }

    /**
     * 截取指定长度字符串，不足则返回完整字符串
     *
     * @param value 字符串
     * @param len   截取长度
     * @param tag   R 右对齐截取   L 左对齐截取
     * @return
     */
    public static String substringValue(String value, int len, String tag) {
        if (isEmpty(value)) {
            return "";
        }

        if (value.length() <= len) {
            return value;
        }

        if (PAD_TYPE_LEFT.equals(tag)) {
            return value.substring(0, len);
        } else {
            return value.substring(value.length() - len);
        }
    }

    public static String randomUUID() {
        return UUID.randomUUID().toString();
    }

    public static String randomUUID(int i) {
        return substringValue(UUID.randomUUID().toString(), i, PAD_TYPE_LEFT);
    }

    /**
     * 分转元
     *
     * @param amount 金额（分）
     * @return 金额（元，两位小数）
     */
    public static String changeF2Y(String amount) {
        if (StringUtils.isEmpty(amount)) {
            return "0.00";
        }
        BigDecimal yuan = (new BigDecimal(amount)).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);

        return yuan.toString();
    }

    public static String changeF2Y(int amount) {
        BigDecimal yuan = (new BigDecimal(amount)).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);

        return yuan.toString();
    }

    /**
     * 元转分
     *
     * @param amount 金额（元）
     * @return 金额（分，整数）
     */
    public static String changeY2F(String amount) {
        if (StringUtils.isEmpty(amount)) {
            return "0";
        }
        BigDecimal fen = (new BigDecimal(amount)).multiply(new BigDecimal("100")).setScale(0, BigDecimal.ROUND_HALF_UP);

        return fen.toString();
    }

    public static String subStringByBytes(String str, int bytes, String charSetName, String fill, String tag) throws UnsupportedEncodingException {
        String subAfter = str;
        int temp = str.length();
        try {
            //直到截取的字符串的字节数  和   需要的 截取的字节数相等位为止
            while (bytes < subAfter.getBytes(charSetName).length) {
                subAfter = subAfter.substring(0, --temp);
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //汉字可能会被多截取一位
        if (subAfter.getBytes(charSetName).length < bytes) {

            StringBuffer res = new StringBuffer();
            StringBuffer bf = new StringBuffer(subAfter);

            if (PAD_TYPE_RIGHT.equals(tag)) {   //右补位
                for (int i = 0; i < bytes - subAfter.getBytes(charSetName).length; i++) {
                    res = bf.append(fill);
                }
            } else if (PAD_TYPE_LEFT.equals(tag)) { //左补位
                StringBuffer bf1 = new StringBuffer();
                for (int i = 0; i < bytes - subAfter.getBytes(charSetName).length; i++) {
                    res = bf1.append(fill);
                }
                res.append(bf);
            }

            return res.toString();
        } else {
            return subAfter;
        }
    }

    public static String getMiddleString(String all, String start, String end) {
        int beginIdx = all.indexOf(start) + start.length();
        int endIdx = all.indexOf(end);
        return all.substring(beginIdx, endIdx);
    }

    /**
     * 隐藏字符
     *
     * @param value
     * @param mask
     * @return
     */
    public static String mask(String value, String mask) {
        int markStart = -1;
        for (int i = 0; i < mask.length(); i++) {
            if (mask.charAt(i) == '*') {
                markStart = i;
                break;
            }
            if (mask.charAt(i) != 'x' && mask.charAt(i) != 'X') {
                markStart = i;
                break;
            }
        }

        if (markStart == -1 || markStart >= value.length()) {
            return value;
        }

        int markEnd = markStart + 1;
        for (int i = mask.length() - 1; i > markStart + 1; i--) {
            if (mask.charAt(i) == '*') {
                markEnd = i + 1;
                break;
            }
            if (mask.charAt(i) != 'x' && mask.charAt(i) != 'X') {
                markEnd = i + 1;
                break;
            }
        }

        if (mask.length() - markEnd + markStart >= value.length()) {
            return value;
        }

        StringBuilder str = new StringBuilder();
        str.append(value.substring(0, markStart));

        markEnd += value.length() - mask.length();

        str.append("****");    //固定拼接4位*


        if (markEnd < value.length()) {
            str.append(value.substring(markEnd));
        }

        return str.toString();
    }

    /**
     * 将double类型数据转化为指定格式的 字符串
     *
     * @param number double类型数据
     * @param format 格式化字符串
     * @return 转换后的字符串
     */
    public static String doubleFormat(double number, String format) {
        return String.format(format, number);
    }

    /**
     * 将double保留2位小数，并转化为字符串
     *
     * @param number double类型数据
     * @return 转换后的字符串
     */
    public static String doubleFormat(double number) {
        return doubleFormat(number, "%.2f");
    }

    public static String trimToEmpty(String value) {
        return value == null ? "" : value.trim();
    }

    public static String asterisk(String phone) {
        return asterisk(phone, 3, 4);
    }

    public static String asterisk(String phone, int start, int length) {
        String phoneHidden = "";

        if (isEmpty(phone) || start < 0 || length < 0) {
            return phoneHidden;
        }

        String asterisking = "";
        for (int j = 0; j < length; j++) {
            asterisking = asterisking + "*";

        }
        StringBuilder sb = new StringBuilder(phone);
        sb.replace(start, length + start, asterisking);
        return sb.toString();
    }

    /**
     * 是否是分割线
     *
     * @param str
     * @return boolean
     */
    public static boolean isCutLine(String str) {
        if (org.apache.commons.lang3.StringUtils.isWhitespace(str)|| str.length() < 2) {
            return false;
        }
        char x = str.charAt(0);
        for (char c : str.substring(0, str.length() - 2).toCharArray()) {
            if (x != c) {
                return false;
            }
        }
        return true;
    }

    public static String ternaryOperator(String value) {

        return isEmpty(value) ? "0" : value;
    }
}
