package com.hisense.common.util;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.xml.sax.InputSource;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * XML 工具类
 *
 * @author dukunbiao(null)  2018-07-26
 */
public class XmlUtils {

    private XmlUtils() {

    }

    /**
     * 检查String是否为XML格式
     *
     * @param value 待检查字符串
     * @return true/false
     */
    public static boolean isXml(String value) {
        try {
            DocumentHelper.parseText(value);
        } catch (DocumentException e) {
            return false;
        }
        return true;
    }

    /**
     * Map转XML，支持fastjson
     *
     * @param map  待转化Map
     * @param root 根节点
     * @return XML格式字符串
     */
    public static String mapToXml(Map<String, String> map, String root) {
        StringBuilder sb = new StringBuilder();
        sb.append("<").append(root).append(">");
        sb.append(mapToXml(map));
        sb.append("</").append(root).append(">");
        return sb.toString();
    }

    public static String mapToXml(Map<String, String> map, String root, String namespaces) {
        StringBuilder sb = new StringBuilder();
        sb.append("<").append(root).append(" ").append(namespaces).append(">");
        sb.append(mapToXml(map));
        sb.append("</").append(root).append(">");
        return sb.toString();
    }

    private static String mapToXml(Map<String, String> map) {
        StringBuilder sb = new StringBuilder();
        List<String> keys = new ArrayList<>(map.keySet());
        for (String key : keys) {
            sb.append("<").append(key).append(">");
            sb.append(map.get(key));
            sb.append("</").append(key).append(">");
        }

        return sb.toString();
    }

    /**
     * Map List转XML，支持fastjson
     *
     * @param maps      待转化Map List
     * @param root      根节点
     * @param childRoot 子节点
     * @return XML格式字符串
     */
    public static String mapsToXml(List<Map<String, String>> maps, String root, String childRoot) {
        StringBuilder sb = new StringBuilder();
        sb.append("<").append(root).append(">");
        for (Map<String, String> map : maps) {
            sb.append(mapToXml(map, childRoot));
        }
        sb.append("</").append(root).append(">");

        return sb.toString();
    }

    public static Map<String, String> xmlToMap(String value, String charset) throws Exception {
        if (!isXml(value)) {
            return null;
        }

        byte[] xmlBytes = value.getBytes();
        SAXReader reader = new SAXReader(false);
        InputSource source = new InputSource(new ByteArrayInputStream(xmlBytes));
        source.setEncoding(charset);
        Document doc = reader.read(source);
        return xmlToMap(doc.getRootElement());
    }

    private static Map<String, String> xmlToMap(Element element) {
        Map<String, String> rest = new HashMap<>();
        List<Element> els = element.elements();
        for (Element el : els) {
            rest.put(el.getName().toLowerCase(), el.getTextTrim());
        }
        return rest;
    }

    /**
     * XML转JSON，支持Map<String, Object>接收
     *
     * @param xml 待转化XML
     * @return JSON
     * @throws Exception
     */
    public static JSONObject xmlToJSON(String xml) throws Exception {
        return elementToJSON(stringToDocument(xml).getRootElement());
    }

    /**
     * XML转 Java Bean
     *
     * @param xml
     * @param clazz
     * @param <T>
     * @return
     * @throws Exception
     */
    public static <T> T xmlToJavaBean(String xml, Class<T> clazz) throws Exception {
        return JSONObject.parseObject(xmlToJSON(xml).toString(), clazz);
    }

    private static Document stringToDocument(String xml) throws Exception {
        return DocumentHelper.parseText(xml);
    }

    private static JSONObject elementToJSON(Element element) {
        JSONObject jsonObject = new JSONObject();
        List<Attribute> attributes = element.attributes();
        for (Attribute attribute : attributes) {
            jsonObject.put(attribute.getName(), attribute.getValue());
        }

        List<Element> elements = element.elements();
        if (!ListUtils.isEmpty(elements)) {
            for (Element e : elements) {
                if (e.attributes().isEmpty() && e.elements().isEmpty()) {
                    jsonObject.put(e.getName(), e.getTextTrim());
                } else {
                    if (!jsonObject.containsKey(e.getName())) {
                        jsonObject.put(e.getName(), new JSONArray());
                    }
                    ((JSONArray) jsonObject.get(e.getName())).add(elementToJSON(e));
                }
            }
        }

        return jsonObject;
    }

    /**
     * String型获取XML value值
     *
     * @param xml XML
     * @param key XML节点
     * @return value值
     * @throws Exception
     */
    public static String getXmlString(String xml, String key) throws Exception {
        if (StringUtils.isEmpty(xml) || StringUtils.isEmpty(key)) {
            return "";
        }

        Document document = DocumentHelper.parseText(xml);
        return ((Element) (document.getRootElement().elements(key).get(0))).getText();
    }

    /**
     * int型获取XML value值
     *
     * @param xml XML
     * @param key XML节点
     * @return value值
     * @throws Exception
     */
    public static int getXmlInt(String xml, String key) throws Exception {
        if (StringUtils.isEmpty(xml) || StringUtils.isEmpty(key)) {
            return 0;
        }

        return Integer.valueOf(getXmlString(xml, key));
    }

    /**
     * int型获取XML value值，无法转化则返回默认值
     *
     * @param xml          XML
     * @param key          XML节点
     * @param defaultValue 默认值
     * @return value值
     */
    public static int getXmlInt(String xml, String key, int defaultValue) {
        if (StringUtils.isEmpty(xml) || StringUtils.isEmpty(key)) {
            return defaultValue;
        }

        try {
            return Integer.valueOf(getXmlString(xml, key));
        } catch (Exception e) {
            return defaultValue;
        }
    }

    /**
     * double型获取XML value值
     *
     * @param xml XML
     * @param key XML节点
     * @return value值
     * @throws Exception
     */
    public static double getXmlDouble(String xml, String key) throws Exception {
        if (StringUtils.isEmpty(xml) || StringUtils.isEmpty(key)) {
            return 0d;
        }

        return Double.valueOf(getXmlString(xml, key));
    }

    /**
     * double型获取XML value值，无法转化则返回默认值
     *
     * @param xml          XML
     * @param key          XML节点
     * @param defaultValue 默认值
     * @return value值
     */
    public static double getXmlDouble(String xml, String key, double defaultValue) {
        if (StringUtils.isEmpty(xml) || StringUtils.isEmpty(key)) {
            return defaultValue;
        }

        try {
            return Double.valueOf(getXmlString(xml, key));
        } catch (Exception e) {
            return defaultValue;
        }
    }
}
