/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.card;

import java.io.Serializable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 卡号。
 *
 * @author zhangsai    2017-10-09
 * @author guolina     2018-12-03 HIPOSAPP-858  会员卡号支持19手机号段
 * @author guolina     2019-04-09 HIPOSAPP-1074 支持储值卡/积分动态码支付
 * @author wangyaogang 2019-08-06 HIPOSAPP-1228 手机号匹配规则修改
 */
public class CardNo implements Serializable {

    // 手机号判定规则,为了避免后续增加新的号段,暂定为1开头,共11位数字
//    private final static String REGEX_MOBILE = "^1[3-9]\\d{9}$";
    private final static String REGEX_MOBILE = "^1\\d{10}$";
    private final static Pattern REGEX_MOBILE_PATTERN = Pattern.compile(REGEX_MOBILE);

    // 卡号类型
    private final CardNoType type;
    // 卡号
    private final String value;

    /**
     * 初始化一个 {@code CardNo} 的实例。
     *
     * @param type  卡号类型。
     * @param value 卡号。
     */
    private CardNo(CardNoType type, String value) {
        this.type = type;
        this.value = value;
    }


    /**
     * 返回卡号类型。
     *
     * @return 卡号类型。
     */
    public CardNoType getType() {
        return this.type;
    }

    /**
     * 返回卡号类型编码。
     *
     * @return 卡号类型编码。
     */
    public int getTypeCode() {
        return this.type.getTypeCode();
    }

    /**
     * 返回卡号值。
     *
     * @return 卡号值。
     */
    public String value() {
        return this.value;
    }


    /**
     * 使用 {@code CardNoType.Default} 初始化一个 {@code CardNo} 的实例。
     *
     * @param cardNo 卡号。
     */
    public static CardNo Default(String cardNo) {
        return new CardNo(CardNoType.Default, cardNo);
    }

    /**
     * 使用 {@code CardNoType.CardFaceNo} 初始化一个 {@code CardNo} 的实例。
     *
     * @param cardFaceNo 卡面号。
     */
    public static CardNo FaceNo(String cardFaceNo) {
        return new CardNo(CardNoType.CardFaceNo, cardFaceNo);
    }

    /**
     * 使用 {@code CardNoType.Mobile} 初始化一个 {@code CardNo} 的实例。
     *
     * @param mobileNo 手机号。
     */
    public static CardNo Mobile(String mobileNo) {
        return new CardNo(CardNoType.Mobile, mobileNo);
    }

    /**
     * 使用 {@code CardNoType.PayCode} 初始化一个 {@code CardNo} 的实例。
     *
     * @param PayCode 付款码。
     */
    public static CardNo PayCode(String PayCode) {
        return new CardNo(CardNoType.PayCode, PayCode);
    }

    /**
     * 初始化一个 {@code CardNo} 的实例。
     * <p>
     * 如果检测到 {@code value} 是手机号，则返回一个类型为 {@code CardNoType.Mobile} 的 {@code CardNo} 实例；
     * 否则，返回一个类型为 {@code CardNoType.CardFaceNo} 的 {@code CardNo} 实例。
     *
     * @param value 卡号
     * @return {@code CardNoType.Mobile} 或 {@code CardNoType.CardFaceNo} 类型的 {@code CardNo} 的实例。
     */
    public static CardNo MobileOrDefault(String value) {
        Matcher matcher = REGEX_MOBILE_PATTERN.matcher(value);

        if (matcher.matches()) {
            return CardNo.Mobile(value);
        } else {
            return CardNo.FaceNo(value);
        }
    }

    /**
     * 初始化一个 {@code CardNo} 的实例。
     * <p>
     * 如果检测到 {@code value} 是手机号，则返回一个类型为 {@code CardNoType.Mobile} 的 {@code CardNo} 实例；
     * 否则，返回一个指定 {@code CardNoType} 类型的 {@code CardNo} 实例。
     *
     * @param value 卡号
     * @return {@code CardNoType.Mobile} 或指定类型的 {@code CardNo} 的实例。
     */
    public static CardNo MobileOrDefault(String value, CardNoType type) {
        Matcher matcher = REGEX_MOBILE_PATTERN.matcher(value);

        if (matcher.matches()) {
            return CardNo.Mobile(value);
        } else {
            return new CardNo(type, value);
        }
    }

    /**
     * 使用 {@code CardNoType.VipCode} 初始化一个 {@code CardNo} 的实例。
     *
     * @param vipCode 会员卡号。
     */
    public static CardNo VipCode(String vipCode) {
        return new CardNo(CardNoType.VipCode, vipCode);
    }

    /**
     * 使用 {@code CardNoType.VipCode} 初始化一个 {@code CardNo} 的实例。
     *
     * @param value 会员卡号 type 会员卡类型
     */
    public static CardNo cardNo(String value,CardNoType type) {
        return new CardNo(type, value);
    }
}
