/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.coupon;

import com.hisense.crm.card.CardNo;
import com.hisense.hipos.coupon.Coupon;
import com.hisense.hipos.coupon.ICoupon;
import com.hisense.hipos.coupon.UsableCoupon;
import com.hisense.hipos.user.User;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.util.Collection;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * 赠券服务。
 *
 * @author zhangsai    2017-10-16
 * @author zhangsai    2017-11-22 增加一段式赠券计算与核销
 * @author zhangsai    2018-02-13 HIPOSAPP-330 增加赠券查询锁定
 * @author zhangsai    2018-02-13 HIPOSAPP-331 增加赠券解锁
 * @author zhangsai    2018-02-26 HIPOSAPP-332 增加赠券异步核销
 * @author xieliyan    2018-04-10 HIPOSAPP-346 增加赠券撤销
 * @author guolina     2018-09-10 HIPOSAPP-766 修复赠券核销(coupon/use)返回数据类型与服务不一致的问题
 * @author guolina     2019-03-25 HIPOSAPP-1048 修复手动添加会员券再次选中时未传校验码导致的用券失败
 * @author yaofangchao 2019-05-22 增加促销方案查询
 * @author wangyaogang 2019-06-13 HIPOSAPP-1134 一段式核销接口返回剩余可用券信息
 * @author wangyaogang 2019-06-27 HIPOSAPP-1181 生成券信息时,若可用金额大于券的面值,进行提示
 */
public class CouponService {

    private CouponService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 计算赠券可用余额及商品分摊。
     *
     * @param coupon     赠券
     * @param cardFaceNo 会员卡面号
     * @param user       收银员
     * @param callback   赠券计算的回调方法
     */
    public static void calc(final Coupon coupon, final String cardFaceNo, final User user, final com.hisense.webapi.Callback<UsableCoupon> callback) {
        Request<CouponCalcRequestData> request = CouponCalcRequestData.request(coupon, cardFaceNo, user);

        WebApi.requestAsync(WebApi.create(ICouponService.class).calc(request), new Callback<Result<CouponCalcResultData>>() {
            @Override
            public void onResponse(Call<Result<CouponCalcResultData>> call, Response<Result<CouponCalcResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("计算赠券可用金额返回参数为空"));
                    return;
                }

                try {
                    callback.onSuccess(CouponCalcResultData.handleResult(response.body().data, coupon.no(),coupon.checkCode()));
                } catch (Exception e) {
                    callback.onError(new Throwable("计算赠券可用金额接口返回参数错误" + e.getMessage()));
                }
            }

            @Override
            public void onFailure(Call<Result<CouponCalcResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, CouponCalcResultData.class);
    }

    /**
     * 多张券计算赠券可用余额及商品分摊。
     *
     * @param coupons    赠券
     * @param cardFaceNo 会员卡面号
     * @param user       收银员
     * @return 赠券的可用余额及商品分摊
     * @throws WebApiException 如果 WebAPI 执行过程中产生错误或返回错误。
     */
    public static List<UsableCoupon> calcMulti(final List<Coupon> coupons, String cardFaceNo, User user) throws WebApiException {
        Request<CouponCalcMultiRequestData> request = CouponCalcMultiRequestData
                .request(coupons, cardFaceNo, user);

        List<CouponCalcMultiResultData> resultData = WebApi.request(WebApi.create(ICouponService.class).calcMulti(request));

        return CouponCalcMultiResultData.handleResult(resultData, coupons);
    }

    /**
     * 多张券计算赠券可用余额及商品分摊。
     *
     * @param coupons    赠券
     * @param cardFaceNo 会员卡面号
     * @param user       收银员
     * @param callback   赠券计算的回调方法
     */
    public static void calcMulti(final List<Coupon> coupons, final String cardFaceNo, final User user,
                                 final com.hisense.webapi.Callback<List<UsableCoupon>> callback) {
        Request<CouponCalcMultiRequestData> request = CouponCalcMultiRequestData
                .request(coupons, cardFaceNo, user);

        WebApi.requestAsync(WebApi.create(ICouponService.class).calcMulti(request), new Callback<Result<List<CouponCalcMultiResultData>>>() {
            @Override
            public void onResponse(Call<Result<List<CouponCalcMultiResultData>>> call, Response<Result<List<CouponCalcMultiResultData>>> response) {
                try {
                    List<UsableCoupon> usableCoupons = CouponCalcMultiResultData
                            .handleResult(response.body().data, coupons);

                    callback.onSuccess(usableCoupons);
                } catch (Exception e) {
                    callback.onError(new Throwable(e.getMessage()));
                }
            }

            @Override
            public void onFailure(Call<Result<List<CouponCalcMultiResultData>>> call, Throwable t) {
                callback.onError(t);
            }
        });
    }
    /**
     * 一段式赠券核销。
     *
     * @param usableCoupons 待核销的赠券集合
     * @param callback      赠券核销的回调方法
     */
    public static void use1(List<UsableCoupon> usableCoupons, final com.hisense.webapi.Callback<Collection<ICoupon>> callback) {
        Request request = CouponUse1RequestData.request(usableCoupons);
        WebApi.requestAsync(WebApi.create(ICouponService.class).use1(request), new Callback<Result<CouponUse1ResultData>>() {
            @Override
            public void onResponse(Call<Result<CouponUse1ResultData>> call, Response<Result<CouponUse1ResultData>> response) {
                callback.onSuccess(CouponUse1ResultData.handleResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<CouponUse1ResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, CouponUse1ResultData.class);
    }

    /**
     * 一段式赠券撤销。
     *
     * @param usableCoupons 待撤销的赠券集合
     * @param callback      赠券撤销的回调方法
     */
    public static void cancel(List<UsableCoupon> usableCoupons, final com.hisense.webapi.Callback<Object> callback) {
        Request request = CouponUse1RequestData.request(usableCoupons);

        WebApi.requestAsync(WebApi.create(ICouponService.class).cancel(request), new Callback<Result<Object>>() {
            @Override
            public void onResponse(Call<Result<Object>> call, Response<Result<Object>> response) {
                callback.onSuccess(new Object());
            }

            @Override
            public void onFailure(Call<Result<Object>> call, Throwable t) {
                callback.onError(t);
            }
        }, Object.class);
    }

    /**
     * 赠券原路退回预判。
     *
     * @param saleNo   流水号
     * @param callback 赠券原路退回预判的回调方法
     */
    public static void couponReturnPredict(String saleNo, final com.hisense.webapi.Callback<CouponUsedData> callback) {
        Request<CouponReturnRequestData> request = CouponReturnRequestData.request(saleNo);

        WebApi.requestAsync(WebApi.create(ICouponService.class).couponReturnPredict(request), new Callback<Result<CouponReturnResultData>>() {
            @Override
            public void onResponse(Call<Result<CouponReturnResultData>> call, Response<Result<CouponReturnResultData>> response) {
                CouponUsedData couponUsedData = CouponReturnResultData.handleResult(response.body().data);
                callback.onSuccess(couponUsedData);
            }

            @Override
            public void onFailure(Call<Result<CouponReturnResultData>> call, Throwable t) {
                callback.onError(t);
            }
        });
    }

    /**
     * 赠券原路退回确认。
     *
     * @param saleNo   流水号
     * @param callback 赠券原路退回确认的回调方法
     */
    public static void couponReturnConfirm(String saleNo, final com.hisense.webapi.Callback<String> callback) {
        Request<CouponReturnRequestData> request = CouponReturnRequestData.request(saleNo);

        WebApi.requestAsync(WebApi.create(ICouponService.class).couponReturnConfirm(request), new Callback<Result<String>>() {
            @Override
            public void onResponse(Call<Result<String>> call, Response<Result<String>> response) {
                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<String>> call, Throwable t) {
                callback.onError(t);
            }
        });
    }

    /**
     * 会员券查询接口。
     *
     * @param cardNo   会员卡号
     * @param saleTime 销售日期
     * @param plu      商品信息
     */
    public static List<CouponPluQueryResultData> couponPluQuery(CardNo cardNo, String saleTime, List<CouponQueryPlu> plu) throws WebApiException {
        Request<CouponPluQueryRequestData> request = CouponPluQueryRequestData.request(cardNo, saleTime, plu);
        return WebApi.request(WebApi.create(ICouponService.class).couponPluQuery(request));
    }

    /**
     * 促销方案查询接口。
     */
    public static List<CouponProjectQueryResultData> couponProjectQuery() throws WebApiException {
        Request<CouponProjectQueryRequestData> request = CouponProjectQueryRequestData.request();
        return WebApi.request(WebApi.create(ICouponService.class).couponProjectQuery(request));
    }
}
