/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.webapi.pay;

import com.hisense.common.util.DateUtils;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.webapi.Context;
import com.hisense.webapi.Callback;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Random;
import java.util.concurrent.TimeUnit;

import retrofit2.Call;
import retrofit2.Response;

/**
 * 支付服务
 *
 * @author wangyaogang 2018-11-27
 * @author wangyaogang 2018-12-08 HIPOSAPP-855 付款界面中控制微信和支付宝支付不允许串刷，避免对账混乱。
 * @author zhangsai    2018-12-19 HIPOSAPP-698 交易商品类型改为 ISaleItem
 * @author wangyaogang 2019-03-06 HIPOSAPP-1043 修复支付扫码时扫非微信/支付宝付款码造成的系统异常
 * @author wangyaogang 2019-05-09 HIPOSAPP-1086 支持银联扫码支付
 * @author wangyaogang 2019-05-23 HIPOSAPP-1123 请求海信支付时增加交易查询避免网络问题导致单边账
 * @author wangyaogang 2019-06-05 HIPOSAPP-1151 海信pay支付,撤销和退款时超时时间设置为2分钟
 * @author zhangsai    2019-06-12 HIPOSAPP-1141 getOutTradeNo() 调整为 newOutTradeNo() 并增加重载方法
 * @author wangyaogang 2019-08-08 HIPOSAPP-1259 对接海信支付退款结果查询接口
 * @author wangyaogang 2019-11-27 HIAPOSCMP-284 海信支付打印签购单
 */
public class PayService {


    /**
     * 返回一个新的商户订单号。
     *
     * @param oldTradeNo 原商户订单号
     * @param returnSale 退货流水
     * @return 一个新的商户订单号
     */
    public static String newOutTradeNo(String oldTradeNo, ReturnSaleModel returnSale) {
        return newOutTradeNo(oldTradeNo.substring(0, 2), returnSale.getSaleNo());
    }


    /**
     * 返回一个新的商户订单号。
     *
     * @param scanCode 付款码
     * @param saleNo   交易流水号
     * @return 一个新的商户订单号
     */
    private static String newOutTradeNo(String scanCode, String saleNo) {
        return scanCode.substring(0, 2) + saleNo + String.format(Locale.CHINESE, "%03d", new Random().nextInt(1000)) + Context.orgCode();
    }

    /**
     * 下单支付
     *
     * @throws WebApiException
     */
    public static void payPay(String authCode, String openid, String outTradeNo, String totalPaid, String payRemark, String saleTime,
                              String scene, String subject, String terminalParams, String terminalId, final Callback<PayResultData> callback) throws WebApiException {
        Request<PayRequestData> request;
        try {
            request = PayRequestData.request(authCode, openid, outTradeNo, totalPaid,
                    payRemark, saleTime, scene, subject, terminalParams, terminalId);
        } catch (Exception e) {
            callback.onError(new Throwable("组装请求参数失败" + e.getMessage()));
            return;
        }

        IPayService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPayService.class);
        WebApi.requestAsync(service.pay(request), new retrofit2.Callback<Result<PayResultData>>() {
            @Override
            public void onResponse(Call<Result<PayResultData>> call, Response<Result<PayResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("交易支付接口返回参数为空"));
                    return;
                }

                callback.onSuccess(PayResultData.handleResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<PayResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, PayResultData.class);
    }

    /**
     * 交易支付查询
     *
     * @throws WebApiException
     */
    public static void payQuery(String outTradeNo, String payRemark, String scene, String saleTime, String terminalId, final Callback<PayResultData> callback) {
        Request<PayRequestData> request = PayRequestData.requestQuery(outTradeNo, payRemark, scene, saleTime, terminalId);
        IPayService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPayService.class);
        WebApi.requestAsync(service.query(request), new retrofit2.Callback<Result<PayResultData>>() {
            @Override
            public void onResponse(Call<Result<PayResultData>> call, Response<Result<PayResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("交易支付查询接口返回参数为空"));
                    return;
                }

                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<PayResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, PayResultData.class);
    }

    /**
     * 撤销交易。
     *
     * @param terminalId 终端号（银联支付）
     * @param callback   撤销的回调方法
     */
    public static void cancel(SalePayModel payModel, String saleNo, String terminalId, final Callback<CancelResultData> callback) {
        Request<PayRequestData> request = PayRequestData.requestCancel(payModel, saleNo, terminalId);
        IPayService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPayService.class);
        WebApi.requestAsync(service.cancel(request), new retrofit2.Callback<Result<CancelResultData>>() {
            @Override
            public void onResponse(Call<Result<CancelResultData>> call, Response<Result<CancelResultData>> response) {
                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<CancelResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, CancelResultData.class);
    }

    /**
     * 退款。
     *
     * @param outTradeNo       原商户订单号
     * @param sale             退货流水
     * @param payRemark        支付方式备注
     * @param refundOutTradeNo 本笔交易商户订单号
     * @param total            原交易金额
     * @param totalRefund      本笔交易退款金额
     * @param terminalId       终端号（银联支付）
     * @param callback         退款的回调方法
     */
    public static void refund(String couponNo, String outTradeNo, ReturnSaleModel sale,
                              String payRemark, String refundOutTradeNo,
                              String total, String totalRefund,
                              String terminalId,
                              final Callback<PayResultData> callback) {
        Request<PayRequestData> request = PayRequestData.requestRefund(couponNo, outTradeNo, sale,
                payRemark, refundOutTradeNo, total, totalRefund, terminalId);
        IPayService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPayService.class);
        WebApi.requestAsync(service.refund(request), new retrofit2.Callback<Result<PayResultData>>() {
            @Override
            public void onResponse(Call<Result<PayResultData>> call, Response<Result<PayResultData>> response) {
                callback.onSuccess(PayResultData.handleRefundResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<PayResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, PayResultData.class);
    }

    /**
     * 退款。
     *
     * @param outTradeNo       原商户订单号
     * @param sale             退货流水
     * @param payRemark        支付方式备注
     * @param refundOutTradeNo 本笔交易商户订单号
     * @param total            原交易金额
     * @param totalRefund      本笔交易退款金额
     * @param terminalId       终端号（银联支付）
     * @param callback         退款的回调方法
     */
    public static void refund(String couponNo, String tradeNo, String outTradeNo, ReturnSaleModel sale,
                              String payRemark, String refundOutTradeNo,
                              String total, String totalRefund,
                              String terminalId,
                              final Callback<PayResultData> callback) {
        Request<PayRequestData> request = PayRequestData.requestRefund(couponNo, tradeNo, outTradeNo, sale,
                payRemark, refundOutTradeNo, total, totalRefund, terminalId);
        IPayService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPayService.class);
        WebApi.requestAsync(service.refund(request), new retrofit2.Callback<Result<PayResultData>>() {
            @Override
            public void onResponse(Call<Result<PayResultData>> call, Response<Result<PayResultData>> response) {
                callback.onSuccess(PayResultData.handleRefundResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<PayResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, PayResultData.class);
    }

    /**
     * 交易支付上传
     *
     * @throws WebApiException
     */
    public static void payUploadOrders(String openId, String tradeNo, PayConfigInfo payCfg,
                                       List<PayChannel> payChannel, TerminalInfo terminal) throws WebApiException {
//        Request<UploadOrdersRequestData> request = UploadOrdersRequestData.request(openId, tradeNo,
//                payCfg, sale, payChannel, terminal);
//        WebApi.request(WebApi.create(IPayService.class).uploadOrders(request));
    }

    /**
     * 退款状态查询
     *
     * @param outTradeNo
     * @param sale
     * @param payRemark
     * @param refundOutTradeNo
     * @param terminalId
     * @param callback
     */
    public static void refundQuery(String outTradeNo, ReturnSaleModel sale,
                                   String payRemark, String refundOutTradeNo,
                                   String terminalId,String total,String totalRefund,
                                   final Callback<PayResultData> callback) {
        Request<PayRequestData> request = PayRequestData.requestRefundQuery(outTradeNo, sale,
                payRemark, refundOutTradeNo, terminalId,total,totalRefund);
        IPayService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPayService.class);
        WebApi.requestAsync(service.refundQuery(request), new retrofit2.Callback<Result<PayResultData>>() {
            @Override
            public void onResponse(Call<Result<PayResultData>> call, Response<Result<PayResultData>> response) {
                callback.onSuccess(PayResultData.handleRefundResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<PayResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, PayResultData.class);
    }


    /**
     * 海信支付签购单模板
     * 1.销售票号 outTradeNo?   支付traceNo存的outTradeNo,但是退款时traceNo存的tradeNo,所以此处取payNo,
     * 两处统一都为outTradeNo
     * 2.payment不为null,为支付,撤销和退款,使用当前时间;
     * 3.salePayment不为null,为重打印,使用sale中的销售时间;
     * 4.两者同时有且仅有一个
     * 5.取消的金额人为置成了负值, 退货因为本地是正且流水查询取了绝对值所以根据sale添加符号
     *
     * @return string集合
     */
    public static ArrayList<String> getPrintContent(BaseSaleModel sale, SalePayModel payment, SalePayModel salePayment) {
        ArrayList<String> printList = new ArrayList<>();

        for (int i = 0; i < 2; i++) {
            printList.add("cmd_centerStart");
            printList.add("海信支付消费存根联");
            printList.add("cmd_centerEnd");
            printList.add("cmd_alignRightStart");
            printList.add(((i == 0) ? "顾客联" : "商户联"));
            printList.add("cmd_alignRightEnd");
            if (payment != null) {
                printList.add("销售票号：" + payment.getBuyerLogonId() + "(" + payment.getPayCode() + payment.getPayName() + ")");
            } else {
                printList.add("销售票号：" + salePayment.getBuyerLogonId() + "(" + salePayment.getPayCode() + salePayment.getPayName() + ")");
            }
            printList.add("收银员：" + sale.getCashier().code());
            if (payment != null) {
                printList.add("销售时间：" + DateUtils.dateFormat());
            } else {
                printList.add("销售时间：" + sale.getSaleTime());
            }
            printList.add("店号：" + Context.shopCode());
            printList.add("----------------------------------------------");
            printList.add("消费(RMB)");
            printList.add("cmd_centerStart");
            if (payment != null) {
                //撤销传入金额 手动赋了负值
                printList.add(((sale instanceof ReturnSaleModel) ? "" + (-Double.parseDouble(payment.getTotalPaid())) : payment.getTotalPaid()));
            } else {
                //todo 此时流水查询应该自带符号了,需要验证下
                printList.add(salePayment.getTotalPaid());
            }
            printList.add("cmd_centerEnd");
            printList.add("----------------------------------------------");
            printList.add("顾客签名：");
            printList.add("");
            printList.add("");
            printList.add("cmd_paperCut");
        }
        return printList;
    }
}
