/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.plu;

import com.hisense.common.util.ListUtils;
import com.hisense.crm.vip.Vip;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;

import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * 商品服务。
 *
 * @author zhangsai        2017-09-02
 * @author zhangsai        2018-01-08 增加商品查询和优惠查询异步处理方法。
 * @author wangyaogang     2018-12-06 HIPOSAPP-885 商品查询接口调整
 * @author guolina         2019-01-09 HIPOSAPP-916 查询商品优惠接口调整
 * @author guolina         2019-04-09 HIPOSAPP-1071 增加商品范围设置异常捕获，防止闪退
 * @author dukunbiao(null) 2019-08-20  重构
 * @author wangyaogang     2020-04-10 HIAPOSCMP-660 会员促销接口，流水查询接口参数调整
 * @author guolina         2020-04-21 HIAPOSCMP-649 利客来易收银允许改价需求
 */
public class PluService {

    private PluService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 查询商品。
     *
     * @param pluCode  商品编码。
     * @param shopCode 专柜编码。
     * @param callback 商品查询的回调方法
     */
    public static void pluQuery(final String pluCode, final String shopCode,
                                final com.hisense.webapi.Callback<BaseSalePluModel> callback) {
        pluQuery("1",null, null, pluCode, shopCode, callback);
    }

    /**
     * 查询商品。
     *
     * @param pluCode  商品编码。
     * @param shopCode 专柜编码。
     * @param callback 商品查询的回调方法
     */
    public static void pluQuery(final String queryType, final String saleNo, final Vip vip, final String pluCode, final String shopCode,
                                final com.hisense.webapi.Callback<BaseSalePluModel> callback) {
        Request<PluQueryRequestData> request = PluQueryRequestData.request(queryType, saleNo, vip, pluCode, shopCode);

        WebApi.requestAsync(WebApi.create(IPluService.class).query(request), new Callback<Result<List<PluQueryResultData>>>() {
            @Override
            public void onResponse(Call<Result<List<PluQueryResultData>>> call, Response<Result<List<PluQueryResultData>>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("商品查询返回参数为空"));
                    return;
                }

                try {
                    PluQueryResultData resultData;
                    if (response.body().data instanceof PluQueryResultData) {
                        resultData = (PluQueryResultData) response.body().data;
                    } else {
                        if (ListUtils.isEmpty(response.body().data)) {
                            callback.onError(new Throwable("商品查询返回参数为空"));
                            return;
                        }
                        resultData = response.body().data.get(0);
                    }

                    callback.onSuccess(PluQueryResultData.handleResult(resultData));
                } catch (Exception e) {
                    callback.onError(new Throwable(e.getMessage()));
                }
            }

            @Override
            public void onFailure(Call<Result<List<PluQueryResultData>>> call, Throwable t) {
                callback.onError(t);
            }
        }, PluQueryResultData.class);
    }

    /**
     * 查询商品优惠。
     *
     * @param callback  商品优惠查询的回调方法
     * @param shopId    专柜ID。
     * @param vip       会员卡。
     * @param pluModels 商品。
     */
    public static void pluDiscount(final String shopId, final String saleNo, final Vip vip, final List<BaseSalePluModel> pluModels,
                                   final com.hisense.webapi.Callback<List<SalePluDscModel>> callback) {
        Request<PluDiscountRequestData> request = PluDiscountRequestData.request(shopId, saleNo, vip, pluModels);

        WebApi.requestAsync(WebApi.create(IPluService.class).discount(request), new Callback<Result<List<PluDiscountResultData>>>() {
            @Override
            public void onResponse(Call<Result<List<PluDiscountResultData>>> call, Response<Result<List<PluDiscountResultData>>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("会员促销商品查询返回参数为空"));
                    return;
                }

                callback.onSuccess(PluDiscountResultData.handleResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<List<PluDiscountResultData>>> call, Throwable t) {
                callback.onError(t);
            }
        }, PluDiscountResultData.class);
    }

    /**
     * 调整商品数量 海信Java中间件
     *
     * @param pluModel 商品
     * @param saleNo   流水号
     * @param callback 调整商品数量的回调方法
     */
    public static void pluAdjust(final BaseSalePluModel pluModel, final String quantity, final String saleNo, final String operator,
                                 final com.hisense.webapi.Callback<PluAdjustResultData> callback) {
        Request<PluRequestData> request = PluRequestData.request(pluModel, quantity, saleNo, operator);

        WebApi.requestAsync(WebApi.create(IPluService.class).pluAdjust(request), new Callback<Result<PluAdjustResultData>>() {
            @Override
            public void onResponse(Call<Result<PluAdjustResultData>> call, Response<Result<PluAdjustResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("调整商品数量返回参数为空"));
                    return;
                }

                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<PluAdjustResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, PluAdjustResultData.class);
    }


    /**
     * 删除商品
     *
     * @param saleNo   流水号
     * @param callback 重新获取整笔交易明细的回调方法
     */
    public static void pluDelete(final String saleNo, final BaseSalePluModel pluModel, final com.hisense.webapi.Callback<PluDeleteResultData> callback) {
        Request<PluDeleteRequestData> request = PluDeleteRequestData.request(saleNo, pluModel);
        WebApi.requestAsync(WebApi.create(IPluService.class).pluDelete(request), new Callback<Result<PluDeleteResultData>>() {
            @Override
            public void onResponse(Call<Result<PluDeleteResultData>> call, Response<Result<PluDeleteResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("删除商品返回参数为空"));
                    return;
                }

                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<PluDeleteResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, PluDeleteResultData.class);
    }
}
