/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.coupon;

import java.io.Serializable;
import java.security.InvalidParameterException;

import lombok.Getter;

/**
 * 赠券。
 *
 * @author zhangsai    2017-10-17
 * @author zhangsai    2017-11-22 增加赠券属性；使用 Builder 模式替换原来的构造函数方式来构建赠券。
 * @author zhangsai    2018-06-15 HIPOSAPP-538 增加开始日期/是否打印/备注等满送赠券属性
 * @author zhangsai    2018-11-13 HIPOSAPP-822 继承自新增的 CouponBase 基类，用于支持礼品券。
 * @author guolina     2019-03-25 HIPOSAPP-1048 修复手动添加会员券再次选中时未传校验码导致的用券失败
 * @author wangyaogang 2019-06-12 HIPOSAPP-1114 增加qType券类型字段
 * @author wangyaogang 2019-06-24 HIPOSAPP-1090 增加活动描述字段
 * @author wangyaogang 2019-06-28 HIPOSAPP-1182 修改异常提示信息
 */
@Getter
public class Coupon extends CouponBase implements Serializable {
    // 赠券的面额
    private double value;
    // 余额
    private double balance;
    // 可核销金额
    private double available;
    // 校验码
    private String checkCode;

    /**
     * 初始化一个 {@code Coupon} 的实例。
     */
    private Coupon() {
        // NO CODE HERE
    }

    /**
     * 返回校验码。
     *
     * @return 校验码
     */
    @Override
    public String checkCode() {
        return this.checkCode;
    }

    /**
     * 返回赠券的面额。
     *
     * @return 面额
     */
    public double value() {
        return this.value;
    }

    /**
     * 返回可核销金额。
     *
     * @return 可核销金额
     */
    public double available() {
        return this.available;
    }

    /**
     * 设置可核销金额。
     *
     * @param value 可核销金额
     */
    public void available(double value) {
        if (value < 0) {
            throw new InvalidParameterException("value 的值不能小于零");
        }
        this.available = value;
    }

    /**
     * 返回券余额。
     */
    public double balance() {
        return this.balance;
    }

    /**
     * 使用 Builder 模式初始化一个 {@code Coupon} 的实例。
     */
    public static class Builder implements Serializable {
        // 券号
        private String qNo;
        // 赠券的面额
        private double value;
        // 余额
        private double balance;
        // 可核销金额
        private double available = -1;
        // 校验码
        private String checkCode;
        // 名称
        private String name;
        // 开始日期
        private String beginDate;
        // 截止日期
        private String endDate;
        // 赠券类型
        private CouponType qType;

        // 是否打印
        private boolean isPrint;
        // 备注
        private String remark;
        //活动描述
        private String couponDesc;


        /**
         * 初始化一个 {@code Coupon.Builder} 的实例。
         */
        public Builder() {
            // 赠券默认类型为金额券
            this.qType = CouponType.Cash;
        }

        /**
         * 设置券类型
         *
         * @param value 券类型
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder type(CouponType value) {
            this.qType = value;
            return this;
        }

        /**
         * 设置券号。
         *
         * @param qNo 券号
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder coupon(String qNo) {
            this.qNo = qNo;
            return this;
        }

        /**
         * 设置券号、校验码。
         *
         * @param qNo       券号
         * @param checkCode 校验码
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder coupon(String qNo, String checkCode) {
            this.qNo = qNo;
            this.checkCode = checkCode;
            return this;
        }

        /**
         * 设置券号、校验码和名称。
         *
         * @param qNo       券号
         * @param checkCode 校验码
         * @param name      券名称
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder coupon(String qNo, String checkCode, String name) {
            this.qNo = qNo;
            this.checkCode = checkCode;
            this.name = name;
            return this;
        }

        /**
         * 设置券类型。
         *
         * @param qNo  券号
         * @param type 券类型
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder coupon(String qNo, CouponType type) {
            this.qNo = qNo;
            this.qType = type;
            return this;
        }

        /**
         * 设置校验码。
         *
         * @param checkCode 校验码
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder checkCode(String checkCode) {
            this.checkCode = checkCode;
            return this;
        }

        /**
         * 设置赠券面额。
         *
         * @param value 赠券面额
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder values(double value) {
            this.value = value;
            return this;
        }

        /**
         * 设置赠券余额。
         *
         * @param value 赠券余额
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder balance(double value) {
            this.balance = value;
            return this;
        }

        /**
         * 设置可核销金额。
         *
         * @param value 可核销金额
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder available(double value) {
            this.available = value;
            return this;
        }

        /**
         * 设置赠券使用开始日期。
         *
         * @param date 赠券使用开始日期
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder beginAt(String date) {
            this.beginDate = date;
            return this;
        }

        /**
         * 设置赠券使用截止日期。
         *
         * @param date 赠券使用截止日期
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder endsAt(String date) {
            this.endDate = date;
            return this;
        }

        /**
         * 设置是否打印赠券。
         *
         * @param value true 表示打印；false 表示不打印
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder isPrint(boolean value) {
            this.isPrint = value;
            return this;
        }

        /**
         * 设置备注。
         *
         * @param value 备注
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder remark(String value) {
            this.remark = value;
            return this;
        }

        /**
         * 设置活动描述。
         *
         * @param couponDesc 活动描述
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder couponDesc(String couponDesc) {
            this.couponDesc = couponDesc;
            return this;
        }

        /**
         * 构建一个 {@code Coupon} 的实例。
         */
        public Coupon build() {
            if (this.balance > this.value) {
                throw new IllegalArgumentException("券的余额不能大于面额");
            }

            Coupon coupon = new Coupon();
            coupon.qNo = this.qNo;
            coupon.name = this.name;
            coupon.checkCode = this.checkCode;
            coupon.value = this.value;
            coupon.balance = this.balance;
            coupon.beginDate = this.beginDate;
            coupon.endDate = this.endDate;
            coupon.isPrint = this.isPrint;
            coupon.remark = this.remark;


            if (this.available == -1) {
                // 如果未设置available可用金额，则认为可用金额与赠券余额相同。
                coupon.available(this.balance);
            } else {
                coupon.available(this.available);
            }
            coupon.qType = this.qType;
            coupon.couponDesc = this.couponDesc;
            return coupon;
        }
    }
}
