/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.coupon;

import com.hisense.hipos.payment.PayMethod;

import java.io.Serializable;
import java.math.BigDecimal;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 可核销赠券。
 *
 * @author zhangsai 2017-11-22
 * @author zhangsai 2017-12-20 HIPOSAPP-245 允许部分核销，即实际核销金额小于 available 可核销金额
 */
public class UsableCoupon implements Serializable {
    // 赠券
    private Coupon coupon;
    // 促销赠券编号
    private String proNo;
    // 限制支付方式
    private List<String> payLimits;
    // 赠券商品分摊
    private Map<Integer, SaleItemShare> shares;
    // 核销金额
    private double useValue;

    // 所属待核销赠券列表
    private UsableCouponList parent;


    /**
     * 初始化一个 {@code UsableCoupon} 的实例。
     *
     * @param proNo     促销赠券编号
     * @param coupon    赠券
     * @param payLimits 限制支付方式。不同支付方式编码使用分号分隔
     */
    public UsableCoupon(String proNo, Coupon coupon, String payLimits) {
        this.proNo = proNo;
        this.coupon = coupon;
        if (payLimits != null && payLimits.trim().length() != 0) {
            this.payLimits = Arrays.asList(payLimits.split(";"));
        } else {
            this.payLimits = new ArrayList<>();
        }
        this.shares = new HashMap<>();
    }

    /**
     * 返回赠券。
     *
     * @return 赠券
     */
    public Coupon coupon() {
        return this.coupon;
    }

    /**
     * 返回促销赠券编号。
     *
     * @return 促销赠券编号
     */
    public String proNo() {
        return this.proNo;
    }

    /**
     * 返回限制的支付方式。
     *
     * @return 限制的支付方式
     */
    public Collection<String> payLimits() {
        return Collections.unmodifiableList(this.payLimits);
    }

    /**
     * 返回赠券分摊商品集合。
     *
     * @return 赠券分摊商品集合
     */
    public Collection<SaleItemShare> shares() {
        return this.shares.values();
    }

    /**
     * 返回 {@code itemNo} 指定行号位置的赠券商品分摊。
     *
     * @param itemNo 商品行号
     * @return {@code itemNo} 指定行号位置的赠券商品分摊
     */
    public SaleItemShare shares(int itemNo) {
        return this.shares.get(itemNo);
    }

    /**
     * 返回可核销金额。
     *
     * @return 可核销金额
     */
    public double available() {
        return this.coupon.available();
    }

    /**
     * 核销金额。
     *
     * @return 核销金额
     */
    public double useValue() {
        return this.useValue;
    }

    /**
     * 返回指定支付方式是否受限。
     *
     * @param method 支付方式
     * @return 受限时返回 true；不受限制时返回 false。
     */
    public boolean isLimit(PayMethod method) {
        return method != null && this.payLimits.contains(method.code());
    }

    /**
     * 返回赠券是否已使用。
     *
     * @return 如果赠券已使用则返回 true；否则返回 false。
     */
    public boolean isUsed() {
        return this.useValue > 0;
    }

    /**
     * 添加一个赠券商品分摊。
     *
     * @param share 商品分摊
     */
    public void addShare(SaleItemShare share) {
        if (share != null) {
            this.shares.put(share.itemNo(), share);
        }
    }

    /**
     * 设置所属的核销赠券列表。
     *
     * @param usableCouponList 赠券列表
     */
    void parent(UsableCouponList usableCouponList) {
        if (usableCouponList != null && parent != null) {
            throw new InvalidParameterException("已添加到其它列表中");
        }

        this.parent = usableCouponList;
    }

    /**
     * 核销。
     *
     * @param value 核销金额
     */
    public void use(double value) {
        BigDecimal p = BigDecimal.valueOf(this.useValue);
        BigDecimal q = BigDecimal.valueOf(value);
        this.useValue = p.add(q).doubleValue();
    }

    /**
     * 取消核销。
     */
    public void cancel() {
        this.useValue = 0;
    }
}
