/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.coupon;

import com.hisense.hipos.model.MainSaleModel;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * 可核销赠券列表，用于管理待核销赠券。
 *
 * @author zhangsai 2017-11-24
 */
public class UsableCouponList implements Serializable {

    // 可核销赠券列表
    private List<UsableCoupon> usableCoupons;
    // 可核销金额合计
    private double total;

    public UsableCouponList() {
        this.usableCoupons = new ArrayList<>();
    }

    /**
     * 添加一个新的 {@code UsableCoupon} 赠券。
     *
     * @param coupon 可核销赠券
     * @return 赠券在列表中的索引位置
     */
    public int add(UsableCoupon coupon) {
        if (this.usableCoupons.add(coupon)) {
            coupon.parent(this);
            BigDecimal p = BigDecimal.valueOf(this.total);
            BigDecimal q = BigDecimal.valueOf(coupon.available());
            this.total = p.add(q).doubleValue();

            return this.usableCoupons.size() - 1;
        } else {
            coupon.parent(null);
            return -1;
        }
    }

    /**
     * 返回可核销赠券集合。
     *
     * @return 可核销赠券集合
     */
    public List<UsableCoupon> items() {
        return this.usableCoupons;
    }

    /**
     * 返回 {@code i} 指定位置的可核销赠券。
     *
     * @param i 赠券在列表中的索引位置
     * @return {@code i} 指定位置的赠券
     */
    public UsableCoupon items(int i) {
        return this.usableCoupons.get(i);
    }

    /**
     * 移除 {@code i} 指定位置及其以后的所有可核销赠券。
     *
     * @param i 要移除赠券在列表中的索引位置
     */
    public Collection<UsableCoupon> remove(int i) {
        List<UsableCoupon> usableCoupons = new ArrayList<>();
        for (int j = this.usableCoupons.size() - 1; j >= 0; j--) {
            if (j >= i) {
                usableCoupons.add(this.items(j));
                this.remove(this.items(j));
            }
        }
        return Collections.unmodifiableList(usableCoupons);
    }

    /**
     * 移除最后添加的可核销赠券。
     */
    public UsableCoupon removeLast() {
        Collection<UsableCoupon> removedCoupons = this.remove(this.usableCoupons.size() - 1);
        return (removedCoupons.size() > 0) ? (UsableCoupon) removedCoupons.toArray()[0] : null;
    }

    /**
     * 移除指定赠券。
     *
     * @param item 要移除的赠券
     */
    public void remove(UsableCoupon item) {
        if (this.usableCoupons.remove(item)) {
            item.parent(null);
            BigDecimal p = BigDecimal.valueOf(this.total);
            BigDecimal q = BigDecimal.valueOf(item.available());
            this.total = p.subtract(q).doubleValue();
        }
    }

    /**
     * 核销赠券。
     *
     * @param usableCoupon 可核销赠券
     */
    public void use(UsableCoupon usableCoupon) {
        if (usableCoupon == null) {
            return;
        }

        if (this.add(usableCoupon) != -1) {
            // 核销商品分摊
            for (SaleItemShare itemShare : usableCoupon.shares()) {
                usableCoupon.use(MainSaleModel.getInstance().getSaleModel().getPlu(itemShare.itemNo() - 1).use(itemShare));
            }
        }
    }

    /**
     * 取消 {@code i} 指定位置及其位置之后的所有赠券核销。
     *
     * @param i 赠券列表的指定位置
     */
    public void cancel(int i) {
        if (i < 0 || i >= this.size()) {
            return;
        }

        for (int j = this.size() - 1; j >= i; j--) {
            this.cancelLast();
        }
    }

    /**
     * 取消所有赠券核销。
     */
    public void cancelAll() {
        this.cancel(0);
    }

    /**
     * 取消末尾位置的赠券核销。
     */
    public void cancelLast() {
        UsableCoupon removedCoupon = this.removeLast();
        if (removedCoupon != null) {
            removedCoupon.cancel();

            // 核销商品分摊
            for (SaleItemShare itemShare : removedCoupon.shares()) {
                MainSaleModel.getInstance().getSaleModel().getPlu(itemShare.itemNo() - 1).cancel(itemShare);
            }
        }
    }

    /**
     * 返回列表的长度。
     *
     * @return 列表长度
     */
    public int size() {
        return this.usableCoupons.size();
    }

    /**
     * 返回可核销金额合计。
     *
     * @return 可核销金额合计
     */
    public double total() {
        return this.total;
    }
}
