/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.invoice.models;

import com.hisense.common.util.DateUtils;
import com.hisense.hipos.sale.models.ISale;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Random;

/**
 * 电子发票。
 *
 * @author xieliyan 2017-09-15 HIPOSAPP-133
 * @author zhangsai 2018-07-02 HIPOSAPP-626
 * @author zhangsai 2018-08-17 HIPOSAPP-745
 * @author zhangsai 2018-12-19 HIPOSAPP-698 交易类型改为 ISale
 * @author zhangsai 2019-01-03 HIPOSAPP-958 新增random()生成提取码方法；默认电子发票配置改为 EInvoiceDefault
 */
public abstract class EInvoice implements IEInvoice {
    /**
     * 山西太原
     */
    public static final int CITY_1401 = 1401; // 山西山姆士(航信)
    /**
     * 山东青岛
     */
    public static final int CITY_3702 = 3702; // 青岛海信广场(航信)
    /**
     * 默认
     */
    public static final int CITY_9999 = 9999;

    String baseUrl; // 电子发票地址


    /**
     * 初始化一个 {@code EInvoice} 的实例。
     *
     * @param baseUrl 电子发票地址
     */
    EInvoice(String baseUrl) {
        this.baseUrl = baseUrl;
    }


    /**
     * 格式化电子发票提取地址。
     *
     * @param sale 交易流水
     * @return 该笔流水的电子发票提取地址
     */
    @Override
    public String formatUrl(ISale sale) {
        try {
            return this.formatUrl(sale, DateUtils.dateParse(sale.saleTime()));
        } catch (Exception ex) {
            return null;
        }
    }

    /**
     * 格式化电子发票提取地址。
     *
     * @param sale     交易流水
     * @param saleTime 交易完成时间
     * @return 该笔流水的电子发票提取地址
     */
    @Override
    public abstract String formatUrl(ISale sale, Date saleTime);

    /**
     * 随机生成电子发票提取码。
     *
     * @return 电子发票提取码
     */
    @Override
    public String random() {
        return EInvoice.random(4);
    }

    /**
     * 根据 {@code config} 创建一个 {@code IEInvoice} 的实例。
     *
     * @param config 电子发票配置
     * @return {@code IEInvoice} 的实例
     */
    public static IEInvoice create(EInvoiceConfig config) {
        switch (config.cityCode()) {
            case CITY_1401:
                return new EInvoice1401(config.baseUrl());
            case CITY_3702:
                return new EInvoice3702(config.baseUrl());
        }

        if (!config.baseUrl().contains("[") && !config.baseUrl().contains("]")) {
            return new EInvoice3702(config.baseUrl());
        }

        return new EInvoiceDefault(config.baseUrl());
    }

    /**
     * 随机生成 {@param digits} 要求位数的提取码。
     *
     * @param digits 提取码的位数
     * @return 提取码
     */
    public static String random(int digits) {
        Random random = new Random();
        long d = random.nextLong();
        return EInvoice.format(d, digits);
    }

    /**
     * 转换数字并截取 {@param digits} 位数的字符串。
     *
     * @param d      随机数
     * @param digits 提取码的位数
     * @return 长度为 4 的字符串，长度不足 4 位的前面补零
     */
    static String format(long d, int digits) {
        String formatted = String.format(Locale.CHINESE, "%0" + digits + "d", d);
        return (formatted.length() > digits) ? formatted.substring(formatted.length() - digits) : formatted;
    }

    /**
     * 将日期转换为指定格式的字符串。
     *
     * @param dateString    日期字符串
     * @param oldDateFormat 原日期格式
     * @param newDateFormat 转换后的日期格式
     * @return 指定格式的日期字符串
     */
    public static String formatDate(String dateString, String oldDateFormat, String newDateFormat) {
        try {
            SimpleDateFormat oldFormatter = new SimpleDateFormat(oldDateFormat, Locale.CHINESE);
            SimpleDateFormat newFormatter = new SimpleDateFormat(newDateFormat, Locale.CHINESE);
            return newFormatter.format(oldFormatter.parse(dateString));
        } catch (ParseException ex) {
            return dateString;
        }
    }
}
