package com.hisense.hipos.invoice.models;

import com.hisense.common.constant.UtilConstants;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hipos.sale.models.ISale;
import com.hisense.security.DES;

import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.UUID;
import java.util.regex.MatchResult;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 电子发票默认实现。
 *
 * @author zhangsai    2018-08-17 HIPOSAPP-745
 * @author zhangsai    2019-01-03 HIPOSAPP-958 对接POS中间件电子发票二维码地址配置规则
 * @author wangyaogang 2019-06-27 HIPOSAPP-1173 温州时代-发票增加到月底有效期
 */
public class EInvoiceDefault extends EInvoice implements IEInvoice {

    private static final String REGEX_ACC_DATE = "\\[accDate(:(.+?)?)?\\]";
    private static final String REGEX_ENCRYPTION = "\\{ENCRYPT(:(.+?)?)?(:(.+?)?)?\\}";
    private static final String REGEX_ENCRYPTION_VALUE = "\\{(.+?)?\\}";
    private static final String REGEX_RANDOM = "\\[random(:(\\d+)?)?\\]";
    private static final String REGEX_RANDOMV2 = "\\[randomV2(:(\\d+)?)?\\]";
    private static final String REGEX_SALE_TIME = "\\[date(:(.+?)?)?\\]";
    private static final String REGEX_TIMESTAMP_IN_MILLI_SECONDS = "\\[timestampMillis(:(\\d+?)?)?\\]";
    private static final String REGEX_TIMESTAMP_IN_SECONDS = "\\[timestamp(:(\\d+?)?)?\\]";
    private static final String REGEX_TIME_TO_END_OF_MONTH_IN_MILLI_SECONDS = "\\[timeToEndOfMonth(:(\\d+?)?)?\\]";
    private static final String REGEX_TOTAL_RECV = "\\[totalRecv(:(.+?)?)?\\]";
    private static final Pattern REGEX_ACC_DATE_PATTERN = Pattern.compile(REGEX_ACC_DATE);
    private static final Pattern REGEX_ENCRYPTION_PATTERN = Pattern.compile(REGEX_ENCRYPTION);
    private static final Pattern REGEX_ENCRYPTION_VALUE_PATTERN = Pattern.compile(REGEX_ENCRYPTION_VALUE);
    private static final Pattern REGEX_RANDOM_PATTERN = Pattern.compile(REGEX_RANDOM);
    private static final Pattern REGEX_RANDOM_PATTERNV2 = Pattern.compile(REGEX_RANDOMV2);
    private static final Pattern REGEX_SALE_TIME_PATTERN = Pattern.compile(REGEX_SALE_TIME);
    private static final Pattern REGEX_TIMESTAMP_IN_MILLI_SECONDS_PATTERN = Pattern.compile(REGEX_TIMESTAMP_IN_MILLI_SECONDS);
    private static final Pattern REGEX_TIMESTAMP_IN_SECONDS_PATTERN = Pattern.compile(REGEX_TIMESTAMP_IN_SECONDS);
    private static final Pattern REGEX_TIME_TO_END_OF_MONTH_IN_MILLI_SECONDS_PATTERN = Pattern.compile(REGEX_TIME_TO_END_OF_MONTH_IN_MILLI_SECONDS);
    private static final Pattern REGEX_TOTAL_RECV_PATTERN = Pattern.compile(REGEX_TOTAL_RECV);

    private Date currentTime;

    /**
     * 初始化一个 {@code EInvoiceDefault} 的实例。
     *
     * @param baseUrl 电子发票地址
     */
    EInvoiceDefault(String baseUrl) {
        super(transform(baseUrl));

        this.currentTime = new Date();
    }

    EInvoiceDefault(String baseUrl, Date currentTime) {
        this(baseUrl);

        this.currentTime = currentTime;
    }

    /**
     * 格式化电子发票提取地址。
     *
     * @param sale     交易流水
     * @param saleTime 交易完成时间
     * @return 该笔流水的电子发票提取地址
     */
    @Override
    public String formatUrl(ISale sale, Date saleTime) {
        return this.parseEncryption(sale, saleTime);
    }

    private String formatUrl(String url, ISale sale, Date saleTime) {
        // 电子发票二维码打印格式配置参数：
        //   【POS前台】 【POS中间件】             【含义】
        // - saleno     [saleNo]                表示流水号
        // - orgcode    [orgCode]               表示组织编码
        // - posno      [posNo]                 表示收款机号
        // - sjsnum     [random:4]              表示随机数（0000-9999）
        // - yyyy       [date:yyyyMMddHHmmss]   表示销售日期，格式：yyyymmddhhmmss
        // - jzdate     [accDate]               表示结账日期
        // - sstotal    [totalRecv]             表示实收金额
        // -            [timestampMillis:7]     表示时间戳（精确到毫秒）

        // 解析 DES KEY
        // 替换地址中的 DES KEY
        // 解析 {}
        // 处理 {}
        // 替换 {}

        // 处理发票地址
        url = replace(url, "[saleNo]", sale.saleNo());
        url = replace(url, "[orgCode]", sale.orgCode());
        url = replace(url, "[posNo]", sale.posNo());
        url = replaceAccDate(url, sale.accDate(), "yyyy-MM-dd", "yyyyMMdd");
        url = replaceRandom(url, sale.udp().get(4));
        url = replaceRandomV2(url, sale.udp().get(4));
        url = replaceSaleTime(url, sale.saleTime(), UtilConstants.DATE_FORMAT, "yyyyMMddHHmmss");
        url = replaceTimestampInMilliSeconds(url, this.currentTime, 7);
        url = replaceTimestampInSeconds(url, this.currentTime, 7);
        url = replaceTotalRecv(url, sale.total(), "2f");
        url = replaceTimeToEndOfMonthMilliSeconds(url, 0);

        return url;
    }

    /**
     * 随机生成电子发票提取码。
     *
     * @return 电子发票提取码
     */
    @Override
    public String random() {
        return extractRandom(this.baseUrl, 4);
    }

    /**
     * 根据发票地址配置提取随机数。
     *
     * @param url           电子发票地址
     * @param defaultDigits 发票二维码地址中随机数的默认位数
     * @return 随机数。如果地址中配置了随机数位数，则返回配置位数的随机数，例如：[random:6]返回6位随机数；如果未配置
     * 随机数位数，则返回默认位数的随机数。
     */
    public static String extractRandom(String url, int defaultDigits) {
        LoggerUtils.getLogger().debug("shifenglei_Default");
        Matcher matcher = REGEX_RANDOM_PATTERN.matcher(url);
        Matcher matcherV2 = REGEX_RANDOM_PATTERNV2.matcher(url);
        if (matcher.find()) {
            int digits = Integer.valueOf(matchFirstGroup(matcher, String.valueOf(defaultDigits)));
            return EInvoice.random(digits);
        }

        if (matcherV2 != null && matcherV2.find()) {
            int digits = Integer.valueOf(matchFirstGroup(matcherV2, String.valueOf(defaultDigits)));
            String guid = UUID.randomUUID().toString().replaceAll("-", "").replace("{","").replace("}","").toLowerCase();

            LoggerUtils.getLogger().debug("shifenglei "+guid.substring(0,digits));

            return guid.substring(0,digits);
        }
        return EInvoice.random(4);
    }

    /**
     * 替换电子发票地址参数
     *
     * @param url   电子发票地址
     * @param key   参数名称
     * @param value 参数值
     * @return 替换指定地址参数后的发票地址
     */
    private static String replace(String url, String key, String value) {
        if (url.contains(key)) {
            return url.replace(key, value);
        } else {
            return url;
        }
    }

    /**
     * 替换结账日期。
     *
     * @param url           电子发票二维码地址
     * @param accDate       结账日期
     * @param oldDateFormat 结账日期的格式
     * @param defaultFormat 发票二维码地址中结账日期的默认格式
     * @return 替换结账日期参数后的电子发票二维码地址
     */
    public static String replaceAccDate(String url, String accDate, String oldDateFormat, String defaultFormat) {
        Matcher matcher = REGEX_ACC_DATE_PATTERN.matcher(url);
        if (matcher.find()) {
            String format = matchFirstGroup(matcher, defaultFormat);
            return matcher.replaceAll(formatDate(accDate, oldDateFormat, format));
        }
        return url;
    }

    /**
     * 替换销售时间。
     *
     * @param url           电子发票二维码地址
     * @param saleTime      销售时间
     * @param oldDateFormat 销售时间的格式
     * @param defaultFormat 发票二维码地址中销售时间的默认格式
     * @return 替换销售时间参数后的电子发票二维码地址
     */
    public static String replaceSaleTime(String url, String saleTime, String oldDateFormat, String defaultFormat) {
        Matcher matcher = REGEX_SALE_TIME_PATTERN.matcher(url);
        if (matcher.find()) {
            String format = matchFirstGroup(matcher, defaultFormat);
            return matcher.replaceAll(formatDate(saleTime, oldDateFormat, format));
        }
        return url;
    }

    /**
     * 替换随机数。
     *
     * @param url   电子发票二维码地址
     * @param value 随机数
     * @return 替换随机数参数后的电子发票二维码地址
     */
    public static String replaceRandom(String url, String value) {
        Matcher matcher = REGEX_RANDOM_PATTERN.matcher(url);
        if (matcher.find()) {
            return matcher.replaceAll(value);
        }
        return url;
    }

    /**
     * 替换随机数。
     *
     * @param url   电子发票二维码地址
     * @param value 随机数
     * @return 替换随机数参数后的电子发票二维码地址
     */
    public static String replaceRandomV2(String url, String value) {
        Matcher matcher = REGEX_RANDOM_PATTERNV2.matcher(url);
        if (matcher.find()) {
            return matcher.replaceAll(value);
        }
        return url;
    }

    /**
     * 替换时间戳。
     *
     * @param url         电子发票二维码地址
     * @param current     当前日期
     * @param defaultDays 时间戳距离当前日期的天数
     * @return 替换时间戳参数后的电子发票二维码地址
     */
    public static String replaceTimestampInMilliSeconds(String url, Date current, int defaultDays) {
        Matcher matcher = REGEX_TIMESTAMP_IN_MILLI_SECONDS_PATTERN.matcher(url);
        if (matcher.find()) {
            int days = Integer.valueOf(matchFirstGroup(matcher, String.valueOf(defaultDays)));
            GregorianCalendar calendar = new GregorianCalendar();
            calendar.setTime(current);
            calendar.add(Calendar.DATE, days);
            long timestamp = calendar.getTime().getTime();
            return matcher.replaceAll(String.valueOf(timestamp));
        }
        return url;
    }

    /**
     * 替换时间戳。
     *
     * @param url         电子发票二维码地址
     * @param current     当前日期
     * @param defaultDays 时间戳距离当前日期的天数
     * @return 替换时间戳参数后的电子发票二维码地址
     */
    public static String replaceTimestampInSeconds(String url, Date current, int defaultDays) {
        Matcher matcher = REGEX_TIMESTAMP_IN_SECONDS_PATTERN.matcher(url);
        if (matcher.find()) {
            int days = Integer.valueOf(matchFirstGroup(matcher, String.valueOf(defaultDays)));
            GregorianCalendar calendar = new GregorianCalendar();
            calendar.setTime(current);
            calendar.add(Calendar.DATE, days);
            long timestamp = new BigDecimal(calendar.getTime().getTime())
                    .divide(BigDecimal.TEN, BigDecimal.ROUND_DOWN)
                    .divide(BigDecimal.TEN, BigDecimal.ROUND_DOWN)
                    .divide(BigDecimal.TEN, BigDecimal.ROUND_DOWN).longValue();
            return matcher.replaceAll(String.valueOf(timestamp));
        }
        return url;
    }

    private String replaceTimeToEndOfMonthMilliSeconds(String url, int defaultDays) {
        Matcher matcher = REGEX_TIME_TO_END_OF_MONTH_IN_MILLI_SECONDS_PATTERN.matcher(url);
        if (matcher.find()) {
            int days = Integer.valueOf(matchFirstGroup(matcher, String.valueOf(defaultDays)));
            String formatted;
            if (days == 0) {
                // 到月底
                Calendar calendar = Calendar.getInstance();
                calendar.add(Calendar.MONTH, 1);
                calendar.set(Calendar.DAY_OF_MONTH, 1);
                calendar.set(Calendar.HOUR_OF_DAY, 0);
                calendar.set(Calendar.MINUTE, 0);
                calendar.set(Calendar.SECOND, 0);
                calendar.set(Calendar.MILLISECOND, 0);
                formatted = String.valueOf(calendar.getTimeInMillis());
            } else {
                //指定天数
                Calendar calendar = Calendar.getInstance();
                calendar.add(Calendar.DAY_OF_MONTH, days);
                formatted = String.valueOf(calendar.getTimeInMillis());
            }
            return matcher.replaceAll(formatted);
        }
        return url;
    }

    /**
     * 替换实收金额。
     *
     * @param url   电子发票二维码地址
     * @param total 实收金额
     * @return 替换实收金额参数后的电子发票二维码地址
     */
    public static String replaceTotalRecv(String url, double total) {
        return replaceTotalRecv(url, total, "2f");
    }

    /**
     * 替换实收金额。
     *
     * @param url           电子发票二维码地址
     * @param total         实收金额
     * @param defaultFormat 发票二维码地址中实收金额的默认格式
     * @return 替换实收金额参数后的电子发票二维码地址
     */
    public static String replaceTotalRecv(String url, double total, String defaultFormat) {
        Matcher matcher = REGEX_TOTAL_RECV_PATTERN.matcher(url);
        if (matcher.find()) {
            String format = matchFirstGroup(matcher, defaultFormat);
            String formatted;
            if (format.equals("d")) {
                // 以“分”为单位
                formatted = String.format(Locale.CHINESE, "%.0f",
                        new BigDecimal(total).multiply(BigDecimal.TEN).multiply(BigDecimal.TEN).doubleValue());
            } else {
                // 以“元”为单位
                if (format.equals("f")) {
                    format = "0f";
                }
                formatted = String.format(Locale.CHINESE, "%." + format, total);
            }
            return matcher.replaceAll(formatted);
        }
        return url;
    }

    /**
     * 返回正则表达式匹配的字符串。
     *
     * @param matcher      正则表达式的匹配结果
     * @param defaultValue 未匹配时返回的默认值
     * @return 匹配结果或默认值
     */
    private static String matchFirstGroup(Matcher matcher, String defaultValue) {
//        if (matcher.matches()) {
        MatchResult result = matcher.toMatchResult();
        if (result.group(1) != null && result.group(2) != null) {
            return result.group(2);
        }
//        }
        return defaultValue;
    }

    public String parseEncryption(ISale sale, Date saleTime) {
        // 解析 DES KEY
        Matcher matcher = REGEX_ENCRYPTION_PATTERN.matcher(this.baseUrl);
        if (matcher.find()) {
            String algorithm = matcher.group(2);
            String key = matcher.group(4);

            // 替换地址中的 DES KEY
            String url2 = matcher.replaceAll("");

            // 解析 {}
            Matcher matcherValue = REGEX_ENCRYPTION_VALUE_PATTERN.matcher(url2);
            if (matcherValue.find()) {
                // 处理 {}
                String encryptFormat = matcherValue.group(1);
                String formatted = this.formatUrl(encryptFormat, sale, saleTime);
                String encrypted = DES.encrypt(key, formatted);
                try {
                    String encoded = URLEncoder.encode(encrypted, "UTF-8");
                    // 替换 {}
                    return matcherValue.replaceAll(encoded);
                } catch (UnsupportedEncodingException ex) {
                    return matcherValue.replaceAll(encrypted);
                }
            } else {
                return this.formatUrl(url2, sale, saleTime);
            }

        } else {
            return this.formatUrl(this.baseUrl, sale, saleTime);
        }
    }

    /**
     * 将POS前台电子发票地址配置规则转换为POS中间件规则。
     *
     * @param baseUrl 电子发票配置地址
     * @return 符合POS中间件规则的电子发票地址配置
     */
    private static String transform(final String baseUrl) {
        String url = baseUrl;

        url = replace(url, "%saleno", "[saleNo]");
        url = replace(url, "%orgcode", "[orgCode]");
        url = replace(url, "%posno", "[posNo]");
        url = replace(url, "%sjsnum", "[random]");
        url = replace(url, "%yyyy", "[date:yyyyMMddHHmmss]");
        url = replace(url, "%jzdate", "[accDate:yyyyMMdd]");
        url = replace(url, "%sstotal", "[totalRecv:2f]");

        return url;
    }
}
