/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.report;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 销售报表。
 *
 * @author zhangsai 2017-10-12 HIPOSAPP-946
 */
public class SaleReport {
    // 合计应收金额
    private double total;
    // 合计优惠金额
    private double totalDsc;
    // 合计实收金额
    private double totalRecv;
    // 合计退货金额
    private double totalRtn;
    // 总交易笔数
    private int totalTrans;
    // 总退货交易笔数
    private int totalTransRtn;

    // 支付信息
    private List<SaleReportPayment> payments;


    /**
     * 初始化一个 {@code SaleReport} 的实例。
     */
    private SaleReport() {
        this.payments = new ArrayList<>();
    }


    /**
     * 返回合计应收金额。
     *
     * @return 合计应收金额
     */
    public double total() {
        return total;
    }

    /**
     * 返回合计优惠金额。
     *
     * @return 合计优惠金额
     */
    public double totalDiscount() {
        return totalDsc;
    }

    /**
     * 返回合计实收金额。
     *
     * @return 合计实收金额
     */
    public double totalReceived() {
        return totalRecv;
    }

    /**
     * 返回合计退货金额。
     *
     * @return 合计退货金额
     */
    public double totalReturned() {
        return totalRtn;
    }

    /**
     * 返回总交易笔数。
     *
     * @return 总交易笔数
     */
    public int totalTrans() {
        return totalTrans;
    }

    /**
     * 返回退货笔数。
     *
     * @return 退货笔数
     */
    public int totalTransReturned() {
        return totalTransRtn;
    }

    /**
     * 返回支付统计项的集合。
     *
     * @return 支付统计项集合
     */
    public List<SaleReportPayment> payments() {
        return Collections.unmodifiableList(payments);
    }

    /**
     * 返回 {@code i} 指定索引位置的支付统计项。
     *
     * @param i 支付统计项的索引位置。
     * @return 指定索引位置的支付统计项。
     */
    public SaleReportPayment payments(int i) {
        return this.payments.get(i);
    }


    /**
     * 添加一个新的 {@code Payment} 支付统计项。
     *
     * @param payment 待添加的支付统计项
     */
    public void addPayment(SaleReportPayment payment) {
        if (payment != null) {
            this.payments.add(payment);
        }
    }


    /**
     * 使用 Builder 模式初始化一个 {@code SaleReport} 的实例。
     */
    public static class Builder {
        // 合计应收金额
        private double total;
        // 合计优惠金额
        private double totalDsc;
        // 合计实收金额
        private double totalRecv;
        // 合计退货金额
        private double totalRtn;
        // 总交易笔数
        private int totalTrans;
        // 总退货交易笔数
        private int totalTransRtn;


        /**
         * 初始化一个 {@code SaleReport.Builder} 的实例。
         *
         * @param trans 总交易笔数
         * @param total 合计应收金额
         */
        public Builder(int trans, double total) {
            this.totalTrans = trans;
            this.total = total;
        }


        /**
         * 设置合计优惠金额。
         *
         * @param discount 合计优惠金额
         * @return {@code SaleReport.Builder} 的实例。
         */
        public Builder discount(double discount) {
            this.totalDsc = discount;
            return this;
        }

        /**
         * 设置合计应收金额。
         *
         * @param received 合计应收金额
         * @return {@code SaleReport.Builder} 的实例。
         */
        public Builder received(double received) {
            this.totalRecv = received;
            return this;
        }

        /**
         * 设置退货笔数和合计退货金额。
         *
         * @param trans    退货笔数
         * @param returned 合计退货金额
         * @return {@code SaleReport.Builder} 的实例。
         */
        public Builder returned(int trans, double returned) {
            this.totalTransRtn = trans;
            this.totalRtn = returned;
            return this;
        }

        /**
         * 构建一个 {@code SaleReport} 的实例。
         *
         * @return {@code SaleReport} 的实例。
         */
        public SaleReport build() {
            SaleReport report = new SaleReport();
            report.total = this.total;
            report.totalDsc = this.totalDsc;
            report.totalRecv = this.totalRecv;
            report.totalRtn = this.totalRtn;
            report.totalTrans = this.totalTrans;
            report.totalTransRtn = this.totalTransRtn;

            return report;
        }
    }
}
