/***************************************************************************************************
 * Copyright (c) 2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.sale.models;

import java.util.Date;

/**
 * 流水号提供程序。
 *
 * @author zhangsai 2018-01-11
 */
public abstract class SaleNoProvider implements ISaleNoProvider {

    // 流水顺序号资源库
    private ISerialNoRepository repository;

    /**
     * 初始化一个 {@code CmpSaleNoProvider} 的实例。
     *
     * @param repository 流水顺序号资源库
     */
    public SaleNoProvider(ISerialNoRepository repository) {
        this.repository = repository;
    }

    /**
     * 返回一个新的流水号。
     *
     * @param orgCode         组织编码
     * @param posNo           收款机号
     * @param saleTime        销售时间
     * @param defaultSerialNo 无法找到顺序号时的默认顺序号
     * @return 流水号
     */
    @Override
    public String next(String orgCode, String posNo, Date saleTime, int defaultSerialNo) {
        int serialNo = this.getSerialNo(orgCode, posNo);

        if (serialNo == -1) {
            // 顺序号不能为负数，默认为 0
            serialNo = defaultSerialNo >= 0 ? defaultSerialNo : 0;
        }

        serialNo = this.next(serialNo);

        return this.format(orgCode, posNo, saleTime, serialNo);
    }

    /**
     * 返回指定收款机的当前流水顺序号。
     *
     * @param orgCode 门店编码
     * @param posNo   收款机号
     * @return 指定收款机的当前流水顺序号。-1 表示没有对应的记录。
     */
    @Override
    public int getSerialNo(String orgCode, String posNo) {
        return this.repository.get(orgCode, posNo);
    }

    /**
     * 保存流水号。
     *
     * @param orgCode 组织编码
     * @param posNo   收款机号
     * @param saleNo  流水号
     */
    @Override
    public void save(String orgCode, String posNo, String saleNo) {
        int serialNo = this.extractSerialNo(saleNo);
        this.repository.save(orgCode, posNo, serialNo);
    }


    /**
     * 根据当前值计算下一个顺序号。
     *
     * @param serialNo 顺序号的当前值
     * @return 下一个顺序号
     */
    protected abstract int next(int serialNo);

    /**
     * 返回一个新的流水号。
     *
     * @param orgCode  组织编码
     * @param posNo    收款机号
     * @param saleTime 销售时间
     * @param serialNo 顺序号
     * @return 流水号
     */
    @Override
    public abstract String format(String orgCode, String posNo, Date saleTime, int serialNo);

    /**
     * 保存流水号。
     *
     * @param saleNo 流水号
     */
    @Override
    public abstract int extractSerialNo(String saleNo);
}
