/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.boc;

import com.boc.aidl.constant.Const;
import com.boc.aidl.rfcard.RFCardConst;
import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.M1Card;
import com.hisense.pos.card.M1CardBlockPasswordCallback;
import com.hisense.pos.card.M1CardReaderProvider;
import com.landi.bocapilibrary.DeviceService;
import com.landi.bocapilibrary.listener.CardReaderListener;
import com.landi.bocapilibrary.utils.BytesUtil;

import java.util.HashMap;

/**
 * 用于读取CRM制卡标准2.0的 Mifare1 非接触式磁卡读取器。
 *
 * @author wangyaogang 2018-11-09 HIPOSAPP-815 支持中行IC卡读卡
 * @author wangyaogang 2018-12-01 HIPOSAPP-814 更新中行刷卡接口
 * @author wangyaogang 2018-12-10 HIPOSAPP-891 过滤错误的磁条卡号
 */
class M1CardReaderV2 extends M1CardReader {

    // 磁卡读取提供程序
    private static M1CardReaderProvider provider;

    // 表示是否正在读卡。由于系统使用线程轮询读卡，如不控制，会多次执行读卡过程。
    // 因此，一旦探测到磁卡后，立即将 isReading 修改为 true，并忽略后续轮询的探测信号。
    // 直到读取卡号，或发生错误，再将 isReading 修改为 false，重新启动探测。
    private volatile boolean isReading = false;

    private HashMap<String, String> blockPasswords;

    /**
     * M1卡 版本
     */
    int version;

    /**
     * 初始化一个 {@code M1CardReaderV2} 的实例。
     *
     * @param version  M1卡 版本
     * @param provider 磁卡读取提供程序
     */
    public M1CardReaderV2(int version, M1CardReaderProvider provider) {
        super();
        this.version = version;
        M1CardReaderV2.provider = provider;
        blockPasswords = new HashMap<>();
    }

    /**
     * 使用读卡密码读取卡内数据。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    @Override
    protected void innerRead(String password, final CardNoCallback callback) {
        if (isReading) {
            return;
        }
        LoggerUtils.getLogger().debug("boc刷卡第一步版本" + version);
        isReading = true;

        if (version == 2) {
            DeviceService.Holder.getInstance().readCard(
                    RFCardConst.RFKeyMode.KEYA_0X00,
                    M1Card.BLOCK_AUTH,
                    M1Card.parseKey(password),
                    10000000, new CardReaderListener() {
                        @Override
                        public void onSuccess(int i, final String icNo) {
                            if (i == Const.OpenCardType.MagicCard) {
                                readMagCard(icNo, callback);
                            } else if (i == Const.OpenCardType.RFCard) {
                                LoggerUtils.getLogger().debug("boc刷卡2AUTH成功" + icNo);
                                if (!"".equals(icNo)) {
                                    // 第二步：根据认证区块数据获取读卡密码
                                    String blockPwd;
                                    if (blockPasswords.containsKey(icNo)) {
                                        blockPwd = blockPasswords.get(icNo);
                                        LoggerUtils.getLogger().debug("boc刷卡2准备读取卡号,已知密码:" + blockPwd);
                                        readCardNo(blockPwd, callback);
                                    } else {
                                        M1CardReaderV2.provider.getBlockPassword(icNo, new M1CardBlockPasswordCallback() {
                                            @Override
                                            public void onSuccess(String blockPwd) {
                                                LoggerUtils.getLogger().debug("boc刷卡2准备读取卡号,获取密码:" + blockPwd);
                                                blockPasswords.put(icNo, blockPwd);
                                                readCardNo(blockPwd, callback);
                                            }
                                        });
                                    }

                                } else {
                                    isReading = false;
                                }
                            }
                        }

                        @Override
                        public void onFail(String s) {
                            isReading = false;
                            LoggerUtils.getLogger().debug("boc刷卡2读卡失败:" + s);
                        }
                    }
            );
        } else {
            String pwd = password + "FFFFFF";
            DeviceService.Holder.getInstance().readCard(
                    RFCardConst.RFKeyMode.KEYA_0X60,
                    M1Card.BLOCK_CARD_NO,
                    M1Card.parseKey(pwd),
                    10000000,
                    new CardReaderListener() {
                        @Override
                        public void onSuccess(int i, String s) {
                            if (i == Const.OpenCardType.MagicCard) {
                                readMagCard(s, callback);
                            } else if (i == Const.OpenCardType.RFCard) {
                                String cardNo = M1Card.decrypt(new String(BytesUtil.hexString2ByteArray(s)));
                                LoggerUtils.getLogger().debug("boc刷卡1==IC卡读卡成功:" + s);
                                LoggerUtils.getLogger().debug("boc刷卡1==IC卡读卡成功解密:" + cardNo);
                                if (!"".equals(cardNo) && !isRead) {
                                    isRead = true;
                                    callback.onSuccess(cardNo);
                                }
                                isReading = false;
                            }
                        }

                        @Override
                        public void onFail(String s) {
                            isReading = false;
                            LoggerUtils.getLogger().debug("boc刷卡1读卡失败" + s);
                        }
                    }
            );
        }
    }

    /**
     * 读取磁条卡卡号
     *
     * @param icNo
     * @param callback
     */
    private void readMagCard(String icNo, CardNoCallback callback) {
        LoggerUtils.getLogger().debug("boc刷卡磁条卡读卡成功原始:" + icNo);
        if (icNo != null && icNo.length() > 3) {
            callback.onSuccess(icNo);
        }
        isReading = false;
    }

    private void readCardNo(String blockPwd, final CardNoCallback callback) {
        if (blockPwd != null && !"".equals(blockPwd)) {
            // 第三步：读取卡号
            DeviceService.Holder.getInstance().readM1Card(
                    RFCardConst.RFKeyMode.KEYA_0X60,
                    M1Card.BLOCK_CARD_NO,
                    M1Card.parseKey(blockPwd),
                    10000000,
                    new CardReaderListener() {

                        @Override
                        public void onSuccess(int i, String cardNo) {
                            LoggerUtils.getLogger().debug("boc刷卡2读卡成功:" + cardNo);
                            if (!"".equals(cardNo) && !isRead) {
                                isRead = true;
                                cardNo = cardNo.substring(0, cardNo.length() - 8);
                                callback.onSuccess(cardNo);
                            }
                            isReading = false;
                        }

                        @Override
                        public void onFail(String s) {
                            isReading = false;
                            LoggerUtils.getLogger().debug("boc刷卡22读卡失败:" + s);
                        }
                    });
        }

        isReading = false;
    }
}
