/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.ccb;

import android.content.Context;
import android.os.ConditionVariable;
import android.os.IBinder;
import android.os.RemoteException;

import com.ccb.deviceservice.aidl.IDeviceService;
import com.ccb.deviceservice.aidl.rfreader.IRFCardReader;
import com.ccb.deviceservice.aidl.rfreader.OnPassListener;
import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardListener;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.card.M1Card;

/**
 * Mifare1 非接触式磁卡读取器的基类。
 *
 * @author zhangsai    2018-04-25
 * @author wangyaogang 2019-09-05 HIPOSAPP-1287 调整线程锁位置并增加日志
 */
abstract class M1CardReader implements ICardReader {

    // 是否正在监听。避免启动多个监听线程。
    private static volatile boolean IS_RUNNING = false;
    // 是否已读取卡号。
    protected volatile boolean isRead = false;

    // 监听程序
    private PiccListener cardListener;


    private IRFCardReader rfCardReader;
    private IDeviceService deviceService;

    // 读卡密码
    private String password;
    // 读取卡号的回调方法
    private CardNoCallback callback;


    public M1CardReader(IDeviceService deviceService) {
        this.deviceService = deviceService;
    }


    /**
     * 打开并初始化读卡器。
     *
     * @param context Context
     */
    @Override
    public void open(Context context) {

    }

    /**
     * 读取卡号。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    @Override
    public void read(String password, CardNoCallback callback) {
        try {
            IBinder binder = this.deviceService.getRFCardReader();
            rfCardReader = IRFCardReader.Stub.asInterface(binder);

            this.password = password;
            this.callback = callback;

            this.start();
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("ccb_M1CardReader_read异常:" + e.toString());
        }
    }

    /**
     * 开始读卡。
     */
    @Override
    public void start() {
        if (rfCardReader == null) {
            LoggerUtils.getLogger().debug("ccb_M1CardReader_start异常:rfCardReader为null");
            return;
        }
        if (IS_RUNNING) { // 如果已启动，则立即退出。避免启动多个监听线程。
            LoggerUtils.getLogger().debug("ccb_M1CardReader_start()_IS_RUNNING返回");
            return;
        }
        LoggerUtils.getLogger().debug("ccb_M1CardReader_start()");
        // 启动立即修改允许状态。避免启动多个监听线程。
        IS_RUNNING = true;
        isRead = false;

        cardListener = new PiccListener(password, callback);
        cardListener.start();
    }

    /**
     * 停止读卡。
     */
    @Override
    public void stop() {
        if (cardListener != null) {
            cardListener.stopListen();
        }
        if (rfCardReader != null) {
            try {
                rfCardReader.halt();
            } catch (RemoteException e) {
                e.printStackTrace();
                LoggerUtils.getLogger().debug("ccb_M1CardReader_stop()_halt()异常:" + e.toString());
            }
        }
        LoggerUtils.getLogger().debug("ccb_M1CardReader_stop()");
        IS_RUNNING = false;
    }

    /**
     * 关闭读卡器。
     */
    @Override
    public void close() {
        this.stop();
    }

    /**
     * 使用读卡密码读取卡内数据。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    protected abstract void innerRead(String password, CardNoCallback callback);

    /**
     * 使用 {@code password} 读取 {@code block} 区块的数据。
     *
     * @param sector   存储区地址
     * @param block    存储块地址
     * @param password 区块的读卡密码
     * @return 存储在 {@code block} 区块的数据
     */
    protected byte[] readData(int sector, int block, String password) {
        final byte[] keyData = M1Card.parseKey(password);
        final byte[] blockData = new byte[16];

        try {
            int result = rfCardReader.authSector(sector, 1, keyData);
            if (result != 0) {
                return null;
            }

            result = rfCardReader.authBlock(block, 1, keyData);
            if (result != 0) {
                return null;
            }

            result = rfCardReader.readBlock(block, blockData);
            if (result != 0) {
                return null;
            }

            return blockData;
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("ccb_M1CardReader_readData异常:" + e.toString());
        }

        return null;
    }

    /**
     * 非接触式磁卡监听程序。
     *
     * @author xieliyan 2017-09-29 HIPOSAPP-160
     * @author zhangsai 2018-04-25
     */
    private class PiccListener extends CardListener {

        // 读取卡号的回调方法
        private CardNoCallback callback;
        // 读卡密码
        private String password;

        private ConditionVariable conditionVariable = new ConditionVariable();
        private int cardType;
        private int sectorNo;

        /**
         * 初始化一个 {@code PiccListener} 的实例。
         *
         * @param password 读卡密码
         * @param callback 读取卡号的回调方法
         */
        public PiccListener(String password, CardNoCallback callback) {
            this.callback = callback;
            this.password = password;
        }

        /**
         * 在一个监听周期后探测并读卡。
         */
        @Override
        protected void onListening() {
            if (isRead) {
                return;
            }
            LoggerUtils.getLogger().debug("ccb_M1CardReader_onListening()");
            final byte[] responseData = new byte[20];

            try {
                LoggerUtils.getLogger().debug("ccb_M1CardReader_准备waitCard");
                rfCardReader.waitCard(new OnPassListener.Stub() {
                    @Override
                    public void onCardPass(int type) throws RemoteException {
                        cardType = type;

                        LoggerUtils.getLogger().debug("ccb_M1CardReader_waitCard_onCardPass");
                        try {
                            if (!rfCardReader.isExist()) {
                                conditionVariable.open();
                                LoggerUtils.getLogger().debug("ccb_M1CardReader_waitCard_onCardPass_卡不在返回");
                                return;
                            }

                            int ret = -1;
                            if (cardType == 0) {
                                ret = rfCardReader.activate("S50", responseData);
                            } else if (cardType == 1) {
                                ret = rfCardReader.activate("S70", responseData);
                            }

                            if (ret != 0) {
                                conditionVariable.open();
                                LoggerUtils.getLogger().debug("ccb_M1CardReader_waitCard_onCardPass_激活失败返回");
                                return;
                            }

                            M1CardReader.this.innerRead(password, callback);
                            conditionVariable.open();
                        } catch (RemoteException e) {
                            e.getMessage();
                            LoggerUtils.getLogger().debug("ccb_M1CardReader_waitCard异常1:" + e.toString());
                            conditionVariable.open();
                        }
                    }

                    @Override
                    public void onFail(int code, String msg) throws RemoteException {
                        LoggerUtils.getLogger().debug("ccb_M1CardReader_waitCard_onFail:" + code + "==" + msg);
                        conditionVariable.open();
                    }
                });
            } catch (RemoteException e) {
                LoggerUtils.getLogger().debug("ccb_M1CardReader_waitCard异常2:" + e.toString());
                conditionVariable.open();
            }
            LoggerUtils.getLogger().debug("ccb_M1CardReader_conditionVariable.block()");
            conditionVariable.block();
        }
    }

}
