/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.hi98;

import android.content.Context;

import com.hisense.pos.card.CardListener;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.card.M1Card;
import com.hisense.pos.picc.PiccCard;
import com.hisense.pos.util.BytesArray;
import com.hisense.pos.util.BytesUtil;

/**
 * Mifare1 非接触式磁卡读取器的基类。
 *
 * @author zhangsai 2018-04-25
 */
abstract class M1CardReader implements ICardReader {

    // 是否正在监听。避免启动多个监听线程。
    private static volatile boolean IS_RUNNING = false;
    // 是否已读取卡号。
    protected volatile boolean isRead = false;

    private PiccCard piccCard;

    // 监听程序
    private PiccListener cardListener;

    // 读卡密码
    private String password;
    // 读取卡号的回调方法
    private CardNoCallback callback;

    /**
     * 打开并初始化读卡器。
     *
     * @param context Context
     */
    @Override
    public void open(Context context) {
        piccCard = new PiccCard();
    }

    /**
     * 读取卡号。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    @Override
    public void read(String password, CardNoCallback callback) {
        if (com.hisense.pos.picc.PiccCard.PICC_OK != piccCard.piccOpen()) {
            return;
        }

        this.password = password;
        this.callback = callback;

        this.start();
    }

    /**
     * 开始读卡。
     */
    @Override
    public void start() {
        if (IS_RUNNING) { // 如果已启动，则立即退出。避免启动多个监听线程。
            return;
        }

        // 启动立即修改允许状态。避免启动多个监听线程。
        IS_RUNNING = true;
        isRead = false;

        cardListener = new PiccListener(password, callback);
        cardListener.start();
    }

    /**
     * 停止读卡。
     */
    @Override
    public void stop() {
        if (cardListener != null) {
            cardListener.stopListen();
        }

        IS_RUNNING = false;
    }

    /**
     * 关闭读卡器。
     */
    @Override
    public void close() {
        this.stop();
    }

    /**
     * 使用读卡密码读取卡内数据。
     *
     * @param password   读卡密码
     * @param serialInfo serialInfo
     * @param callback   读取卡号的回调方法
     */
    protected abstract void innerRead(String password, byte[] serialInfo, CardNoCallback callback);

    /**
     * 使用 {@code password} 读取 {@code block} 区块的数据。
     *
     * @param block      磁卡区块
     * @param password   区块的读卡密码
     * @param serialInfo serialInfo
     * @return 存储在 {@code block} 区块的数据
     */
    protected byte[] readData(int block, String password, byte[] serialInfo) {
        byte authType = 0x00;      // 认证类型
        byte blockAddress = (byte) block;
        byte[] keyData = M1Card.parseKey(password);
        byte[] restInfo = BytesUtil.subBytes(serialInfo, 1, 0xFF & serialInfo[0]);
        int authResult = piccCard.mfAuthBlock(authType, blockAddress, keyData, restInfo);

        if (0 == authResult) {
            BytesArray data = new BytesArray();
            if (0 == piccCard.mfReadBlock(blockAddress, data)) {
                return data.getData();
            }
        }

        return null;
    }

    /**
     * 非接触式磁卡监听程序。
     *
     * @author xieliyan 2017-09-29 HIPOSAPP-160
     * @author zhangsai 2018-04-25
     */
    private class PiccListener extends CardListener {

        // 读取卡号的回调方法
        private CardNoCallback callback;
        // 读卡密码
        private String password;

        // PICC卡读卡所需参数
        private byte cardTypes[];
        private byte readCardType[];
        private byte serialInfo[];
        private byte cid[];
        private byte other[];

        /**
         * 初始化一个 {@code PiccListener} 的实例。
         *
         * @param password 读卡密码
         * @param callback 读取卡号的回调方法
         */
        public PiccListener(String password, CardNoCallback callback) {
            this.callback = callback;
            this.password = password;

            // 初始化读卡所需数据
            cardTypes = new byte[]{(byte) 'm', 0x00};
            readCardType = new byte[5];
            serialInfo = new byte[10];
            cid = new byte[100];
            other = new byte[100];
        }

        /**
         * 在一个监听周期后探测并读卡。
         */
        @Override
        protected void onListening() {
            if (isRead) {
                return;
            }

            for (byte cardType : cardTypes) {
                int result = piccCard.piccDetect(cardType, readCardType, serialInfo, cid, other);
                if (result == PiccCard.PICC_OK) {
                    if (readCardType[0] == 0x10 || readCardType[0] == 0x20 || readCardType[0] == 0x40) {
                        M1CardReader.this.innerRead(password, serialInfo, callback);
                    }
                } else if (result != PiccCard.PICC_FAIL_FOUNDCARD) {
                    break;
                }
            }
        }
    }

}
