/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.hi98;

import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.M1Card;
import com.hisense.pos.card.M1CardBlockPasswordCallback;
import com.hisense.pos.card.M1CardReaderProvider;
import com.hisense.pos.util.BytesUtil;

/**
 * 用于读取CRM制卡标准2.0的 Mifare1 非接触式磁卡读取器。
 *
 * @author zhangsai 2018-04-25
 */
class M1CardReaderV2 extends M1CardReader {

    // 磁卡读取提供程序
    private M1CardReaderProvider provider;

    // 表示是否正在读卡。由于系统使用线程轮询读卡，如不控制，会多次执行读卡过程。
    // 因此，一旦探测到磁卡后，立即将 isReading 修改为 true，并忽略后续轮询的探测信号。
    // 直到读取卡号，或发生错误，再将 isReading 修改为 false，重新启动探测。
    private volatile boolean isReading = false;

    /**
     * 初始化一个 {@code M1CardReaderV2} 的实例。
     *
     * @param provider 磁卡读取提供程序
     */
    public M1CardReaderV2(M1CardReaderProvider provider) {
        this.provider = provider;
    }

    /**
     * 使用读卡密码读取卡内数据。
     *
     * @param password   读卡密码
     * @param serialInfo serialInfo
     * @param callback   读取卡号的回调方法
     */
    @Override
    protected void innerRead(String password, final byte[] serialInfo, final CardNoCallback callback) {
        if (isReading) {
            return;
        }

        try {
            isReading = true;

            // 第一步：从认证区块读取数据
            byte[] authData = super.readData(M1Card.BLOCK_AUTH, password, serialInfo);
            String icNo = BytesUtil.bytes2HexString(authData);

            if (!"".equals(icNo)) {
                // 第二步：根据认证区块数据获取读卡密码
                this.provider.getBlockPassword(icNo, new M1CardBlockPasswordCallback() {
                    @Override
                    public void onSuccess(String blockPwd) {
                        if (blockPwd != null && !"".equals(blockPwd)) {
                            // 第三步：读取卡号
                            byte[] cardNoData = M1CardReaderV2.super.readData(M1Card.BLOCK_CARD_NO, blockPwd, serialInfo);
                            String cardNo = BytesUtil.bytes2HexString(cardNoData);
                            if (!"".equals(cardNo) && !isRead) {
                                isRead = true;
                                cardNo = cardNo.substring(0, cardNo.length() - 8);
                                callback.onSuccess(cardNo);
                            }
                        }

                        isReading = false;
                    }
                });
            } else {
                isReading = false;
            }

        } catch (Exception ex) {
            isReading = false;
        }
    }

}
