/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.hk316;

import android.content.Context;
import android.os.ConditionVariable;
import android.os.RemoteException;

import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardListener;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.card.M1Card;
import com.hisense.pos.print.hk316.HI316DeviceHelper;
import com.histonepos.npsdk.picccard.IPiccCardService;
import com.histonepos.npsdk.picccard.IPiccEventListener;

/**
 * Mifare1 非接触式磁卡读取器的基类。
 *
 * @author wangyaogang 2020-01-10 HIAPOSCMP-482 支持磁条卡/非接卡刷卡功能
 */
abstract class M1CardReader implements ICardReader {

    // 是否正在监听。避免启动多个监听线程。
    private static volatile boolean IS_RUNNING = false;
    // 是否已读取卡号。
    protected volatile boolean isRead = false;

    // 监听程序
    private PiccListener cardListener;


    private IPiccCardService rfCardReader;

    // 读卡密码
    private String password;
    // 读取卡号的回调方法
    private CardNoCallback callback;

    private byte[] resetinfo = new byte[0]; //验证时需要的数据, 激活时获得

    public M1CardReader() {

    }


    /**
     * 打开并初始化读卡器。
     *
     * @param context Context
     */
    @Override
    public void open(Context context) {
        try {
            rfCardReader = HI316DeviceHelper.getInstance().getPiccCard();
            int iret = rfCardReader.piccOpen();
            if (iret != 0) {
                LoggerUtils.getLogger().debug("316_M1CardReader_open失败:" + iret);
            }
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("316_M1CardReader_open异常:" + e.toString());
        }
    }

    /**
     * 读取卡号。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    @Override
    public void read(String password, CardNoCallback callback) {

        this.password = password;
        this.callback = callback;

        this.start();
    }

    /**
     * 开始读卡。
     */
    @Override
    public void start() {
        if (rfCardReader == null) {
            LoggerUtils.getLogger().debug("316_M1CardReader_start异常:rfCardReader为null");
            return;
        }
        if (IS_RUNNING) { // 如果已启动，则立即退出。避免启动多个监听线程。
            LoggerUtils.getLogger().debug("316_M1CardReader_start()_IS_RUNNING返回");
            return;
        }
        // 启动立即修改允许状态。避免启动多个监听线程。
        IS_RUNNING = true;
        isRead = false;

        cardListener = new PiccListener(password, callback);
        cardListener.start();
    }

    /**
     * 停止读卡。
     */
    @Override
    public void stop() {
        if (cardListener != null) {
            cardListener.stopListen();
        }
        IS_RUNNING = false;
    }

    /**
     * 关闭读卡器。
     */
    @Override
    public void close() {
        this.stop();
        LoggerUtils.getLogger().debug("316_M1CardReader_close");
       /* new Thread(new Runnable() {
            @Override
            public void run() {
                if (rfCardReader != null) {
                    try {
                        rfCardReader.stopCheck();
                        rfCardReader.piccClose();
                        LoggerUtils.getLogger().debug("316_M1CardReader_close");
                    } catch (Exception e) {
                        e.printStackTrace();
                        LoggerUtils.getLogger().debug("316_M1CardReader_close异常:" + e.toString());
                    }
                }
            }
        }).start();*/
    }

    /**
     * 使用读卡密码读取卡内数据。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    protected abstract void innerRead(String password, CardNoCallback callback);

    /**
     * 使用 {@code password} 读取 {@code block} 区块的数据。
     *
     * @param sector   存储区地址
     * @param block    存储块地址
     * @param password 区块的读卡密码
     * @return 存储在 {@code block} 区块的数据
     */
    protected byte[] readData(int sector, int block, String password) {
        final byte[] keyData = M1Card.parseKey(password);
        final byte[] blockData = new byte[16];

        try {
            int result = rfCardReader.piccMfAuthBlock((byte) sector, (byte) block, keyData, resetinfo);
            if (result != 0) {
                LoggerUtils.getLogger().debug("316_M1CardReader_piccMfAuthBlock失败:" + result);
                return null;
            }

            result = rfCardReader.piccMfReadBlock((byte) block, blockData);
            if (result != 0) {
                LoggerUtils.getLogger().debug("316_M1CardReader_piccMfReadBlock失败:" + result);
                return null;
            }

            return blockData;
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("316_M1CardReader_readData异常:" + e.toString());
        }

        return null;
    }

    /**
     * 非接触式磁卡监听程序。
     *
     * @author xieliyan 2017-09-29 HIPOSAPP-160
     * @author zhangsai 2018-04-25
     */
    private class PiccListener extends CardListener {

        // 读取卡号的回调方法
        private CardNoCallback callback;
        // 读卡密码
        private String password;

        private ConditionVariable conditionVariable = new ConditionVariable();

        /**
         * 初始化一个 {@code PiccListener} 的实例。
         *
         * @param password 读卡密码
         * @param callback 读取卡号的回调方法
         */
        public PiccListener(String password, CardNoCallback callback) {
            this.callback = callback;
            this.password = password;
        }

        /**
         * 在一个监听周期后探测并读卡。
         */
        @Override
        protected void onListening() {
            if (isRead) {
                return;
            }
            try {
                rfCardReader.setTimeout(60 * 60 * 24);
                rfCardReader.setPiccListener(new IPiccEventListener.Stub() {
                    @Override
                    public void onCardActivate(byte[] mSerialInfo) throws RemoteException {
                        resetinfo = mSerialInfo;
                        M1CardReader.this.innerRead(password, callback);
                        conditionVariable.open();
                    }

                    @Override
                    public void onTimeout() throws RemoteException {
                        rfCardReader.piccClose();
                        conditionVariable.open();
                    }
                });
            } catch (Exception e) {
                conditionVariable.open();
                e.printStackTrace();
                LoggerUtils.getLogger().debug("316_M1CardReader_waitCard异常2:" + e.toString());
            }
            conditionVariable.block();
        }
    }

}
