/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.icbc;

import android.content.Context;
import android.os.ConditionVariable;
import android.os.RemoteException;

import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardListener;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.card.M1Card;
import com.icbc.smartpos.deviceservice.aidl.IDeviceService;
import com.icbc.smartpos.deviceservice.aidl.IRFCardReader;
import com.icbc.smartpos.deviceservice.aidl.RFSearchListener;

/**
 * Mifare1 非接触式磁卡读取器的基类。
 *
 * @author lizhixin 2018-06-22 HIPOSAPP-596 新增工行收单功能-读卡
 * @author lizhixin 2018-08-07 HIPOSAPP-712 支持艾体威尔设备类型
 */
abstract class M1CardReader implements ICardReader {

    private static final String TAG = "M1CardReader";

    // 是否正在监听。避免启动多个监听线程。
    private static volatile boolean IS_RUNNING = false;
    // 是否已读取卡号。
    protected volatile boolean isRead = false;
    //设备类型 LANDI 联迪 AISINO 艾体威尔
    private String deviceType;

    // 监听程序
    private PiccListener cardListener;


    private IRFCardReader rfCardReader;
    private IDeviceService deviceService;

    // 读卡密码
    private String password;
    // 读取卡号的回调方法
    private CardNoCallback callback;


    public M1CardReader(IDeviceService deviceService, String deviceType) {
        this.deviceService = deviceService;
        this.deviceType = deviceType;
    }


    /**
     * 打开并初始化读卡器。
     *
     * @param context Context
     */
    @Override
    public void open(Context context) {

    }

    /**
     * 读取卡号。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    @Override
    public void read(String password, CardNoCallback callback) {
        try {
            rfCardReader = deviceService.getRFCardReader();
            /*IRFCardReader binder = this.deviceService.getRFCardReader();
            rfCardReader = IRFCardReader.Stub.asInterface(binder);*/

            this.password = password;
            this.callback = callback;

            start();
        } catch (RemoteException e) {
        }
    }

    /**
     * 开始读卡。
     */
    @Override
    public void start() {
        if (rfCardReader == null) {
            LoggerUtils.getLogger().debug("-------------------irfCardReader is NULL-------------------------");
            return;
        }
        if (IS_RUNNING) { // 如果已启动，则立即退出。避免启动多个监听线程。
            return;
        }

        // 启动立即修改允许状态。避免启动多个监听线程。
        IS_RUNNING = true;
        isRead = false;

        cardListener = new PiccListener(password, callback);
        cardListener.start();
    }

    /**
     * 停止读卡。
     */
    @Override
    public void stop() {
        if (cardListener != null) {
            cardListener.stopListen();

            if ("AISINO".equals(deviceType)) {
                //艾体威尔设备需要停止监听
                try {
                    rfCardReader.stopSearch();
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
            }


        }

        IS_RUNNING = false;
    }

    /**
     * 关闭读卡器。
     */
    @Override
    public void close() {
        this.stop();
    }

    /**
     * 使用读卡密码读取卡内数据。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    protected abstract void innerRead(String password, CardNoCallback callback);

    /**
     * 使用 {@code password} 读取 {@code block} 区块的数据。
     *
     * @param sector   存储区地址
     * @param block    存储块地址
     * @param password 区块的读卡密码
     * @return 存储在 {@code block} 区块的数据
     */
    protected byte[] readData(int sector, int block, String password) {
        final byte[] keyData = M1Card.parseKey(password);
        final byte[] blockData = new byte[16];

        try {
            int result = rfCardReader.authSector(sector, 1, keyData);
            if (result != 0) {
                return null;
            }

            result = rfCardReader.authBlock(block, 1, keyData);
            if (result != 0) {
                return null;
            }

            result = rfCardReader.readBlock(block, blockData);
            if (result != 0) {
                return null;
            }

            return blockData;
        } catch (RemoteException e) {
        }

        return null;
    }

    /**
     * 非接触式磁卡监听程序。
     *
     * @author xieliyan 2017-09-29 HIPOSAPP-160
     * @author zhangsai 2018-04-25
     */
    private class PiccListener extends CardListener {

        // 读取卡号的回调方法
        private CardNoCallback callback;
        // 读卡密码
        private String password;

        private ConditionVariable conditionVariable = new ConditionVariable();
        private int cardType;

        /**
         * 初始化一个 {@code PiccListener} 的实例。
         *
         * @param password 读卡密码
         * @param callback 读取卡号的回调方法
         */
        public PiccListener(String password, CardNoCallback callback) {
            this.callback = callback;
            this.password = password;
        }

        /**
         * 在一个监听周期后探测并读卡。
         */
        @Override
        protected void onListening() {
            if (isRead) {
                return;
            }

            final byte[] responseData = new byte[1024];

            try {
                rfCardReader.searchCard(new RFSearchListener.Stub() {
                    @Override
                    public void onCardPass(int cType) {
                        cardType = cType;
                        conditionVariable.open();
                        try {
                            if (!rfCardReader.isExist()) {
                                return;
                            }

                            int ret = -1;
                            if (cardType == 0) {
                                ret = rfCardReader.activate("S50", responseData);
                            } else if (cardType == 1) {
                                ret = rfCardReader.activate("S70", responseData);
                            }

                            if (ret != 0) {
                                return;
                            }

                            M1CardReader.this.innerRead(password, callback);

                        } catch (RemoteException e) {
                            e.getMessage();
                        }
                    }

                    @Override
                    public void onFail(int error, String message) {
                        conditionVariable.open();
                    }
                });

                /*new OnPassListener.Stub() {
                    @Override
                    public void onCardPass(int type) throws RemoteException {
                        cardType = type;
                        conditionVariable.open();
                        try {
                            if (!rfCardReader.isExist()) {
                                return;
                            }

                            int ret = -1;
                            if (cardType == 0) {
                                ret = rfCardReader.activate("S50", responseData);
                            } else if (cardType == 1) {
                                ret = rfCardReader.activate("S70", responseData);
                            }

                            if (ret != 0) {
                                return;
                            }

                            M1CardReader.this.innerRead(password, callback);

                        } catch (RemoteException e) {
                            e.getMessage();
                        }
                    }

                    @Override
                    public void onFail(int code, String msg) throws RemoteException {
                        conditionVariable.open();
                    }
                }*/
            } catch (RemoteException e) {
            }
            conditionVariable.block();
        }
    }

}
