/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.icbc;

import com.hisense.pos.payment.PayResult;

import java.math.BigDecimal;

/**
 * 工行收单返回的交易结果。
 *
 * @author zhangsai 2018-11-28 HIPOSAPP-888
 */
public class IcbcPayResult extends PayResult {

    private BaseResult baseResult;          // 基本结果数据
    private PurchaseTransResult transResult;// 交易结果数据
    private PurchaseExtraInfo extraInfo;    // 交易附加数据
    private double amount;                  // 交易金额

    /**
     * 初始化一个 {@code IcbcPayResult} 的实例。
     *
     * @param baseResult  基本结果数据
     * @param transResult 交易结果数据
     * @param extraInfo   交易附加数据
     */
    IcbcPayResult(BaseResult baseResult, PurchaseTransResult transResult, PurchaseExtraInfo extraInfo) {
        this.baseResult = baseResult;
        this.transResult = transResult;
        this.extraInfo = extraInfo;
        this.amount = parseAmount(transResult.amount());
    }

    /**
     * 解析交易金额。
     *
     * @param amount 交易金额。单位：分
     * @return 正常单位的交易金额。单位：元
     */
    static double parseAmount(long amount) {
        BigDecimal p = new BigDecimal(amount);
        BigDecimal q = new BigDecimal(100);
        return p.divide(q, 2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    /**
     * 返回交易金额。
     *
     * @return 交易金额
     */
    @Override
    public double amount() {
        return this.amount;
    }

    /**
     * 返回交易参考号。
     *
     * @return 交易参考号
     */
    @Override
    public String refNo() {
        return this.transResult.refNo();
    }

    /**
     * 返回交易凭证号。
     *
     * @return 交易凭证号
     */
    @Override
    public String traceNo() {
        // 工行当日撤销使用REF_NO，而不是交易凭证号。此处使用REF_NO覆盖交易凭证号
        return this.transResult.refNo();
    }

    /**
     * 返回交易卡号或二维码消费订单号。
     *
     * @return 交易卡号或二维码消费订单号
     */
    @Override
    public String cardNo() {
        return (transResult.QRCodeOrder() != null) ? transResult.QRCodeOrder() : extraInfo.pan();
    }

    @Override
    public String outTradeNo() {
        return null;
    }

    @Override
    public String mchId() {
        return null;
    }

    /**
     * 返回终端号。
     *
     * @return 终端号
     */
    @Override
    public String termNo() {
        return baseResult.termNo();
    }


    @Override
    public String payType() {
        return null;
    }

    /**
     * 返回当前交易是否为消费。
     *
     * @return true 表示消费，false 表示其它。
     */
    @Override
    public boolean isPayment() {
        // 工行收单使用回调方式处理接口调用结果，无需区分消费/撤销/退货。此处不做处理。
        return false;
    }

    /**
     * 返回当前交易是否为消费撤销。
     *
     * @return true 表示消费撤销，false 表示其它。
     */
    @Override
    public boolean isRefunding() {
        // 工行收单使用回调方式处理接口调用结果，无需区分消费/撤销/退货。此处不做处理。
        return false;
    }

    /**
     * 返回当前交易是否为查询。
     *
     * @return true 表示查询，false 表示其它。
     */
    @Override
    public boolean isQuery() {
        return false;
    }

    /**
     * 返回当前交易是否为退货。
     *
     * @return true 表示退货，false 表示其它。
     */
    @Override
    public boolean isCancellation() {
        // 工行收单使用回调方式处理接口调用结果，无需区分消费/撤销/退货。此处不做处理。
        return false;
    }
}
