/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.icbc;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.RemoteException;

import com.hisense.pos.payment.PayRequest;
import com.hisense.pos.payment.PayResult;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;
import com.icbc.smartpos.transservice.aidl.TransHandler;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;
import java.util.Map;

/**
 * 工行收单应用支付提供程序的基类。
 *
 * @author zhangsai 2018-11-27 HIPOSAPP-888
 */
public abstract class IcbcPaymentProvider extends IcbcProviderBase {

    private final static String TRANS_DATE_FORMAT = "yyyyMMdd";
    private final static SimpleDateFormat TRANS_DATE_FORMATTER = new SimpleDateFormat(TRANS_DATE_FORMAT, Locale.CHINESE);

    /**
     * 初始化一个 {@code IcbcPaymentProvider} 的实例。
     *
     * @param activity Activity 活动
     */
    IcbcPaymentProvider(Activity activity) {
        super(activity);
    }

    /**
     * 返回是否支持交易查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    public boolean supportPayQuery() {
        return false;
    }

    /**
     * 返回是否支持撤销查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    public boolean supportCancelQuery() {
        return false;
    }

    /**
     * 返回是否支持退款查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    public boolean supportRefundQuery() {
        return false;
    }

    /**
     * 银行标准化金额
     *
     * @param amount 金额
     * @return amount.substring() 标准化后的金额
     */
    static String makeFieldAmount(String amount) {
        String[] part = amount.split("\\.");
        switch (part.length) {
            case 1:
                return ("0000000000" + amount + "00").substring(amount.length());
            case 2:
                String digit = part[1].length() >= 2 ? part[1].substring(0, 2)
                        : (part[1] + "00").substring(0, 2);
                amount = "000000000000" + part[0] + digit;
                return amount.substring(amount.length() - 12);
        }
        System.out.println(part.length);
        return null;
    }

    /**
     * 处理工行收单回调的返回结果。
     *
     * @param args 返回结果集合
     * @return 支付结果
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    public PayResult handleResult(Object... args) throws PaymentProviderException {
        if (args == null) {
            throw new PaymentProviderException("无交易返回数据");
        }

        if (args.length != 3) {
            throw new PaymentProviderException("交易返回数据的参数个数不正确");
        }

        BaseResult baseResult = this.handleBaseResult((Bundle) args[0]);
        PurchaseTransResult transResult = this.handleResult(PurchaseTransResult.class, (Bundle) args[1]);
        PurchaseExtraInfo extraInfo = this.handleResult(PurchaseExtraInfo.class, (Bundle) args[2]);

        return new IcbcPayResult(baseResult, transResult, extraInfo);
    }

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 支付结果
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    public PayResult handleResult(Intent data) throws PaymentProviderException {
        BaseResult baseResult = this.handleBaseResult(data.getBundleExtra("baseResult"));
        PurchaseTransResult transResult = this.handleResult(PurchaseTransResult.class,
                data.getBundleExtra("transResult"));
        PurchaseExtraInfo extraInfo = this.handleResult(PurchaseExtraInfo.class,
                data.getBundleExtra("extraInfo"));

        return new IcbcPayResult(baseResult, transResult, extraInfo);
    }

    /**
     * 收银。
     *
     * @param transListType 交易类型
     * @param payRequest    交易金额
     * @param handler       交易结果处理程序
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    protected void pay(String transListType, PayRequest payRequest, final PaymentHandler handler)
            throws PaymentProviderException {
        Bundle transData = new Bundle();
        transData.putLong("AMOUNT", Long.valueOf(makeFieldAmount(String.valueOf(payRequest.getAmount()))));
//        transData.putLong("INS_TIMES", 10);//分期付款期数，可以在工行收单应用内设置

        //如果有消费类型则放入
        ArrayList<String> transList = new ArrayList<>();
        transList.add(transListType);
        transData.putStringArrayList("TRANS_LIST", transList);

        //打印第二联将持卡人签名改为“右对齐大字体”屏蔽下面三行第一联和第二联相同
        /*ArrayList<String> g = new ArrayList<>();
        g.add("{\"type\":1,\"text\":[{\"align\":2,\"font\":2,\"data\":\"右对齐大字体\"}]}");
        transData.putStringArrayList("ADDITIONAL_PRINT", g);*/

        //调用消费服务
        IcbcService.startTrans(TransType.MULTI_PURCHASE, transData, new TransHandler.Stub() {
            @Override
            public void onFinish(final Bundle baseResult, final Bundle transResult, final Bundle extraInfo) throws RemoteException {
                if (handler != null) {
                    activity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            handler.onFinish(baseResult, transResult, extraInfo);
                        }
                    });
                }
            }
        });
    }

    /**
     * 隔日退货。
     *
     * @param transType  交易类型
     * @param payRequest 退货金额
     * @param refNo      交易参考号
     * @param transDate  交易日期
     * @param params     附加参数
     * @param handler    退货结果处理程序
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    protected void refund(String transType, PayRequest payRequest, String refNo, Date transDate,
                          Map<String, String> params, final PaymentHandler handler) throws PaymentProviderException {
        Bundle transData = new Bundle();
        transData.putLong("AMOUNT", Long.valueOf(makeFieldAmount(String.valueOf(payRequest.getAmount()))));
        transData.putString("OLD_REF_NO,", refNo);
        transData.putString("OLD_TERM_NO,", params.get("TERM_NO"));
        transData.putString("OLD_TRANS_DATE,", TRANS_DATE_FORMATTER.format(transDate));
        IcbcService.startTrans(transType, transData, new TransHandler.Stub() {
            @Override
            public void onFinish(final Bundle baseResult, final Bundle transResult, final Bundle extraInfo) throws RemoteException {
                if (handler != null) {
                    activity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            handler.onFinish(baseResult, transResult, extraInfo);
                        }
                    });
                }
            }
        });
    }

    /**
     * 当日撤销。
     *
     * @param payRequest 撤销金额
     * @param traceNo    交易凭证号
     * @param params     附加参数
     * @param handler    撤销结果处理程序
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    public void cancel(PayRequest payRequest, String traceNo, Map<String, String> params,
                       final PaymentHandler handler) throws PaymentProviderException {
        Bundle transData = new Bundle();
        transData.putString("OLD_REF_NO", traceNo);
        //打印第二联将持卡人签名改为条形码，（条形码内容为1234567）屏蔽下面三行第一联和第二联相同
        /*ArrayList<String> g = new ArrayList<>();
        g.add("{\"type\":3,\"barcode\":{\"width\":280,\"height\":60,\"data\":\"1234567\"}}");
        transData.putStringArrayList("ADDITIONAL_PRINT", g);*/

        IcbcService.startTrans(TransType.POS_VOID, transData, new TransHandler.Stub() {
            @Override
            public void onFinish(final Bundle baseResult, final Bundle transResult, final Bundle extraInfo) throws RemoteException {
                if (handler != null) {
                    activity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            handler.onFinish(baseResult, transResult, extraInfo);
                        }
                    });
                }
            }
        });
    }

    /**
     * 目前工行不支持查询
     *
     * @param uniqueNo
     * @param handler
     */
    public void payQuery(String uniqueNo, PaymentHandler handler) {

    }
}
