/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.ums;

import android.text.TextUtils;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.payment.PayResult;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.regex.Pattern;

/**
 * 银联商务应用返回的交易结果。
 *
 * @author zhangsai    2018-11-24 HIPOSAPP-888
 * @author wangyaogang 2019-07-22 HIPOSAPP-1221 银联商务支付返回参数增加空格,易收银接收后统一去除
 * @author wangyaogang 2019-07-24 HIPOSAPP-1223 银联商务交易返回金额统一取绝对值处理
 * @author wangyaogang 2019-08-19 HIPOSAPP-1225 银联商务增加支付查询
 * @author wangyaogang 2020-04-14 HIAPOSCMP-654 增加利客来银联商务支付异常（超时等）查询功能
 * @author guolina     2020-05-12 HIAPOSCMP-750 利客来银商聚合支付退款payno记录
 */
 public class UmsPayResult extends PayResult {

    private String appName;         // 应用名称
    private String transId;         // 交易ID
    private JSONObject transData;   // 交易结果
    private double amount;          // 交易金额

    /**
     * 初始化一个 {@code UmsPayResult} 的实例。
     *
     * @param appName   应用名称
     * @param transId   交易ID
     * @param transData 交易结果
     */
    UmsPayResult(String appName, String transId, JSONObject transData) {
        this.appName = appName;
        this.transId = transId.replaceAll("\\s*", "");

        if (transData != null) {
            this.transData = transData;
        } else {
            this.transData = new JSONObject();
        }

        this.amount = Math.abs(this.parseAmount("amt"));
    }


    /**
     * 返回交易金额。
     *
     * @return 交易金额
     */
    @Override
    public double amount() {
        return this.amount;
    }

    /**
     * 返回卡号。
     * (卡号全为0 ，则取orderNo)
     *
     * @return 卡号
     */
    @Override
    public String cardNo() {
        String cardNo = this.getString("cardNo");
        if (TransId.POS_MASTER.equals(this.transId) ||
                TransId.POS_MASTER2.equals(this.transId)
                || (isValidNumber(cardNo) && BigDecimalUtils.isEquals0(cardNo))) {
            try {
                JSONObject memInfo = new JSONObject(this.transData.get("memInfo").toString());
                return memInfo.getString("orderNo");
            } catch (JSONException ex) {
                LoggerUtils.getLogger().debug(ex.getMessage().toString());
            }
        }
        if (TransId.REFUNDING.equals(this.transId)) {
            try {
                JSONObject chnSpecInfo = new JSONObject(this.transData.get("chnSpecInfo").toString());
                return chnSpecInfo.getString("orderNo");
            } catch (JSONException ex) {
                LoggerUtils.getLogger().debug(ex.getMessage().toString());
            }
        }

        return this.getString("cardNo");
    }

    /**
     * 是否纯数字
     *
     * @param cardNo
     * @return
     */
    private boolean isValidNumber(String cardNo) {
        if (TextUtils.isEmpty(cardNo) || cardNo.trim().length() == 0) {
            return false;
        }
        Pattern pattern = Pattern.compile("^[-+]?[\\d]*$");
        return pattern.matcher(cardNo).matches();
    }

    /**
     * 返回参考号。
     *
     * @return 参考号
     */
    @Override
    public String refNo() {
        return this.getString("refNo");
    }

    /**
     * 返回交易凭证号。
     *
     * @return 交易凭证号
     */
    @Override
    public String traceNo() {
        return this.getString("traceNo");
    }

    /**
     * 返回终端号。
     *
     * @return 终端号
     */
    @Override
    public String termNo() {
        return null;
    }


    @Override
    public String outTradeNo() {
        return null;
    }

    @Override
    public String mchId() {
        return null;
    }

    @Override
    public String payType() {
        try {
            if (TransId.REFUNDING.equals(this.transId)) {
                JSONObject chnSpecInfo = new JSONObject(this.transData.get("chnSpecInfo").toString());
                return chnSpecInfo.getString("channelName");
            } else {
                JSONObject memInfo = new JSONObject(this.transData.get("memInfo").toString());
                return memInfo.getString("channelName");
            }
        } catch (JSONException ex) {
            LoggerUtils.getLogger().debug(ex.getMessage().toString());
        }
        return null;
    }

    /**
     * 返回当前交易是否为消费。
     *
     * @return true 表示消费，false 表示其它。
     */
    @Override
    public boolean isPayment() {
        return TransId.CONSUME_BY_PASSWORD.equals(this.transId) ||
                TransId.BANK_CARD_CONSUME.equals(this.transId) ||
                TransId.POS_MASTER.equals(this.transId) ||
                TransId.POS_MASTER2.equals(this.transId) ||
                TransId.CONSUME.equals(this.transId) ||
                TransId.WALLET_COUPON.equals(this.transId);
    }

    /**
     * 返回当前交易是否为消费撤销。
     *
     * @return true 表示消费撤销，false 表示其它。
     */
    @Override
    public boolean isCancellation() {
        return TransId.CANCELLATION.equals(this.transId);
    }

    /**
     * 返回当前交易是否为退货。
     *
     * @return true 表示退货，false 表示其它。
     */
    @Override
    public boolean isRefunding() {
        return TransId.REFUNDING.equals(this.transId);
    }

    /**
     * 返回当前交易是否为查询
     *
     * @return true 表示查询，false 表示其它。
     */
    @Override
    public boolean isQuery() {
        return TransId.TRANS_DETAIL.equals(this.transId);
    }


    /**
     * 返回对应 {@code name} 的值。如果不存在，则返回 null。
     *
     * @param name 键
     * @return 如果存在，则返回对应 {@code name} 的值。如果不存在，则返回 null。
     */
    private String getString(String name) {
        return this.getString(name, "");
    }

    /**
     * 返回对应 {@code name} 的值。如果不存在，则返回 {@code defaultValue} 默认值。
     *
     * @param name         键
     * @param defaultValue 默认值
     * @return 如果存在，则返回对应 {@code name} 的值。如果不存在，则返回 {@code defaultValue} 默认值。
     */
    private String getString(String name, String defaultValue) {
        try {
            if (transData.has(name)) {
                return transData.getString(name);
            } else {
                return defaultValue;
            }
        } catch (JSONException ex) {
            return defaultValue;
        }
    }

    /**
     * 解析并返回对应 {@code name} 的交易金额。
     *
     * @param name 键
     * @return 如果存在则返回对应的交易金额。如果不存在，则返回 0。
     */
    private double parseAmount(String name) {
        String amt = this.getString(name, "0");
//        BigDecimal p = new BigDecimal(amt);
//        BigDecimal q = new BigDecimal(100);
//        return p.divide(q, 2, BigDecimal.ROUND_HALF_UP).doubleValue();
        return Double.valueOf(amt);
    }
}
