/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.ums;

import android.app.Activity;
import android.content.Intent;

import com.hisense.pos.payment.PayRequest;
import com.hisense.pos.payment.PayResult;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProvider;
import com.hisense.pos.payment.PaymentProviderException;
import com.ums.AppHelper;

import org.json.JSONException;
import org.json.JSONObject;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * 银联商务收单应用支付提供程序的基类。
 *
 * @author zhangsai    2018-11-24 HIPOSAPP-888
 * @author zhangsai    2018-02-01 HIPOSAPP-898  增加 supportCancellation() 是否支持当日撤销属性
 * @author wangyaogang 2019-08-19 HIPOSAPP-1225 银联商务增加支付查询
 * @author wangyaogang 2020-04-14 HIAPOSCMP-654 增加利客来银联商务支付异常（超时等）查询功能
 */
abstract class UmsPaymentProvider extends PaymentProvider {

    private final static String REFUND_DATE_FORMAT = "MMdd";
    private final static SimpleDateFormat REFUND_DATE_FORMATTER = new SimpleDateFormat(REFUND_DATE_FORMAT, Locale.CHINESE);
    private final static DecimalFormat AMOUNT_FORMATTER = new DecimalFormat("########.##");

    private String appName; // 应用名称
    private String appId; // 开放平台,客户申请,
    private String transId; // 当前交易类型,

    /**
     * 初始化一个 {@code UmsPaymentProvider} 的实例。
     *
     * @param activity Activity 活动
     * @param appName  应用名称
     */
    UmsPaymentProvider(Activity activity, String appName) {
        super(activity);

        this.appName = appName;
        this.appId = AppId.getAppId(activity.getPackageName());
    }


    /**
     * 格式化交易金额。
     *
     * @param amt 交易金额
     * @return 符合银联商务请求标准的交易金额
     */
    private static String formatAmount(double amt) {
        BigDecimal p = new BigDecimal(amt);
        BigDecimal q = new BigDecimal(100);
        return AMOUNT_FORMATTER.format(p.multiply(q).doubleValue());
    }

    /**
     * 调用 POS 交易接口。
     *
     * @param transId 交易ID
     * @param map     扩展参数
     */
    private void callPosTrans(String transId, Map map) {
        this.transId = transId;
        AppHelper.callTrans(super.activity, this.appName, transId, new JSONObject(map));
    }

    /**
     * 调用公共资源接口。
     *
     * @param transId 交易ID
     * @param map     扩展参数
     */
    private void callPublicResource(String transId, Map map) {
        this.transId = transId;
        AppHelper.callTrans(super.activity, AppName.PUBLIC_RESOURCE, transId, new JSONObject(map));
    }

    /**
     * 返回 Android 请求编码。
     *
     * @return Android 请求编码
     */
    public int requestCode() {
        return AppHelper.TRANS_REQUEST_CODE;
    }

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 支付结果
     * @throws PaymentProviderException 支付异常
     */
    public PayResult handleResult(Intent data) throws PaymentProviderException {
        if (null == data) {
            if (TransId.CANCELLATION.equals(this.transId)) {
                throw new PaymentProviderException("未返回任何数据");
            } else {
                throw new PaymentProviderException(PayResult.CODE_NO_DATA, "未返回任何数据");
            }
        }

        Map<String, String> map = AppHelper.filterTransResult(data);

        /**
         * 接口调用结果（与交易无关）：
         * “0”表示调用接口成功；
         * “-1”表示未找到appName对应的资源包；
         * “-2”表示未找transId对应的业务，
         * “-3”表示交易结果返回失败，需要调用交易明细/重打印接口，
         * (实际不替换jar包，不更换调用方式的情况下，-3不可能返回，其处理和上述的返回为空一样)
         */
        if (!"0".equals(map.get("resultCode"))) {
            if ("-3".equals(map.get("resultCode"))) {
                throw new PaymentProviderException(PayResult.CODE_NO_DATA, "未返回任何数据");
            } else {
                throw new PaymentProviderException("resultMsg:" + map.get("resultMsg"));
            }
        }

        if (null == map.get("transData")) {
            throw new PaymentProviderException("返回transData为空");
        }

        try {
            String sJson = map.get("transData");
            JSONObject transData = new JSONObject(sJson);

            if (transData.get("resCode") == null) {
                throw new PaymentProviderException("返回resCode为空");
            }

            if (!transData.getString("resCode").equals("00")) {
                String resCode = transData.getString("resCode");
                String resDesc = transData.getString("resDesc");
                throw new PaymentProviderException(resCode, resDesc);
            }

            return new UmsPayResult(map.get("appName"), map.get("transId"), transData);
        } catch (JSONException ex) {
            throw new PaymentProviderException("JSONException" + ex);
        }
    }

    /**
     * 返回是否支持撤销。
     *
     * @return true 表示支持撤销，false 表示不支持撤销
     */
    public boolean supportCancellation() {
        return true;
    }

    /**
     * 返回是否支持交易查询。
     * 后一个判断用于区分 通用银联商务 还是通用工行,通用工行不支持
     * (两者包名一致,选择的支付通道一致)
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    public boolean supportPayQuery() {
        return this.appId != null && "com.ums.transcontroller.call".equals(AppHelper.PAY_ACTION);
    }

    /**
     * 返回是否支持撤销查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    public boolean supportCancelQuery() {
        return false;
    }

    /**
     * 返回是否支持退款查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    public boolean supportRefundQuery() {
        return false;
    }

    /**
     * 消费。
     *
     * @param transId    交易ID
     * @param payRequest 交易金额
     * @param tradeType  交易类型
     */
    protected void pay(String transId, PayRequest payRequest, TradeType tradeType) {
        HashMap<String, String> map = new HashMap<>();
        map.put("amt", formatAmount(payRequest.getAmount()));
        if (this.appId != null) {
            map.put("extOrderNo", payRequest.getUniqueNo());
            map.put("appId", this.appId);
        }
        this.callPosTrans(transId, map);
    }

    /**
     * 交易查询方法
     * sNeedPrintReceipt：当该值为true表示POS不打单，当该值为false或者不存在时表示由POS打单
     * isShowDetailPage：当该值为true时，表示调起交易明细页面，当该值为false或者不存在时表示为重打印
     * <p>
     * 实际 isShowDetailPage为true,停在明细界面,应该传false
     *
     * @param uniqueNo
     * @param handler
     * @throws PaymentProviderException
     */
    public void payQuery(String uniqueNo, PaymentHandler handler) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("extOrderNo", uniqueNo);
        map.put("appId", this.appId);
        map.put("isNeedPrintReceipt", true);
        map.put("isShowDetailPage", false);
        this.callPublicResource(TransId.TRANS_DETAIL, map);
    }

    /**
     * 退货。
     *
     * @param payRequest 退款金额
     * @param refNo      参考号
     * @param transDate  交易日期
     * @param params     扩展参数
     */
    public void refund(PayRequest payRequest, String refNo, Date transDate, Map<String, String> params, PaymentHandler handler) {
        HashMap<String, String> map = new HashMap<>();
        map.put("amt", String.valueOf(payRequest.getAmount()));
        map.put("refNo", refNo);
        map.put("date", REFUND_DATE_FORMATTER.format(transDate));
        map.put("orderNo", params.get("PAY_NO"));
        this.callPublicResource(TransId.REFUNDING, map);
    }

    /**
     * 撤销。
     *
     * @param payRequest 退款金额
     * @param traceNo    原交易凭证号
     * @param params     扩展参数
     */
    public void cancel(PayRequest payRequest, String traceNo, Map<String, String> params, PaymentHandler handler) {
        HashMap<String, String> map = new HashMap<>();
        map.put("orgTraceNo", traceNo);
        this.callPublicResource(TransId.CANCELLATION, map);
    }

}