/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.ums;

import android.app.Activity;
import android.content.Intent;
import android.text.TextUtils;

import com.hisense.pos.payment.IPaymentServiceProvider;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProvider;
import com.hisense.pos.payment.PaymentProviderException;
import com.ums.AppHelper;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

/**
 * 银联商务支付功能提供程序。
 *
 * @author zhangsai    2018-11-23 HIPOSAPP-888
 * @author wangyaogang 2020-04-15 HIAPOSCMP-673 银联商务支持班次查询
 * @author wangyaogang 2020-04-21 HIAPOSCMP-709 银联商务打开换班功能直接返回时提示“用户取消”
 * @author wangyaogang 2020-04-22 HIAPOSCMP-657 对接银商POS通订单查询接口
 * @author wangyaogang 2020-04-22 HIAPOSCMP-658 对接银商交易明细查询接口
 */
public class UmsPaymentServiceProvider extends PaymentProvider implements IPaymentServiceProvider {

    private final static String PROVIDER_NAME = "银联商务";
    private String transId; // 当前交易类型,
    private String appId; // 开放平台,客户申请,

    /**
     * 初始化一个 {@code UmsPaymentServiceProvider} 的实例。
     *
     * @param activity Activity 活动
     */
    public UmsPaymentServiceProvider(Activity activity) {
        super(activity);
        this.appId = AppId.getAppId(activity.getPackageName());
    }

    /**
     * 返回 Android 请求编码。
     *
     * @return Android 请求编码
     */
    @Override
    public int requestCode() {
        return AppHelper.TRANS_REQUEST_CODE;
    }

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 返回结果字典
     * @throws PaymentProviderException 支付服务异常
     */
    @Override
    public Map handleResult(Intent data) throws PaymentProviderException {
        if (data == null) {
            if (TransId.SHIFT.equals(this.transId)) {
                throw new PaymentProviderException("用户取消");
            } else {
                throw new PaymentProviderException("没有返回数据");
            }
        }

        Map map = AppHelper.filterTransResult(data);
        if (map.get("transData") == null) {
            throw new PaymentProviderException("返回数据transData为空");
        }

        String sJson = (String) map.get("transData");

        try {
            JSONObject jsonObject = new JSONObject(sJson);
            if (jsonObject.get("resCode") == null) {
                throw new PaymentProviderException("返回参数resCode为空");
            }

            if (!jsonObject.get("resCode").toString().equals("00")) {
                String resultMsg = jsonObject.get("resDesc").toString();
                throw new PaymentProviderException(resultMsg);
            }
        } catch (JSONException ex) {
            throw new PaymentProviderException(ex);
        }

        return map;
    }

    /**
     * 返回提供程序名称。
     *
     * @return 提供程序名称
     */
    @Override
    public String providerName() {
        return PROVIDER_NAME;
    }

    /**
     * 重打印签购单。
     *
     * @param traceNo 交易凭证号
     */
    @Override
    public void reprintReceipt(String traceNo, PaymentHandler handler) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("traceNo", traceNo);
        map.put("isNeedPrintReceipt", false);
        this.callPublicResource(TransId.REPRINT_RECEIPT, map);
    }

    /**
     * 重打印结算单。
     */
    @Override
    public void reprintSettlement() {
        // 银联商务未提供【重打印结算单】接口
    }

    /**
     * 打印交易汇总。
     */
    @Override
    public void printTransSummary(PaymentHandler handler) {
        this.callPublicResource(TransId.PRINT_TRANS_SUMMARY);
    }

    /**
     * 查询银行卡余额。
     */
    @Override
    public void queryBalance(PaymentHandler handler) {
        this.callPublicResource(TransId.QUERY_CARD_BALANCE);
    }

    /**
     * 结算。
     */
    @Override
    public void settle(PaymentHandler handler) {
        // 银联商务未提供【结算】接口
    }

    /**
     * 换班。
     */
    @Override
    public void shift() {
        this.callPublicResource(TransId.SHIFT);
    }


    /**
     * 调用公共资源。
     *
     * @param transId 请求ID
     */
    private void callPublicResource(String transId) {
        this.callPublicResource(transId, new HashMap());
    }

    /**
     * 调用公共资源。
     *
     * @param transId 请求ID
     * @param map     请求参数
     */
    private void callPublicResource(String transId, HashMap map) {
        this.transId = transId;
        AppHelper.callTrans(super.activity, AppName.PUBLIC_RESOURCE, transId, new JSONObject(map));
    }

    /**
     * 调用 POS 交易接口。
     *
     * @param transId 交易ID
     * @param map     扩展参数
     */
    private void callPosTrans(String appName, String transId, Map map) {
        this.transId = transId;
        AppHelper.callTrans(super.activity, appName, transId, new JSONObject(map));
    }

    /**
     * 班次查询
     */
    @Override
    public void queryBatchNo() {
        this.callPublicResource(TransId.BATCH_NO);
    }


    /**
     * 交易查询
     *
     * @param traceNo            凭证号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单；
     *                           false或者不存在时表示由POS打单
     * @param handler
     * @throws PaymentProviderException
     */
    @Override
    public void tradeQuery(String traceNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException {

        if (TextUtils.isEmpty(this.appId)) {
            throw new PaymentProviderException("appId为空，不支持查询");
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("traceNo", traceNo);
        map.put("appId", this.appId);
        map.put("isNeedPrintReceipt", isNeedPrintReceipt);
        map.put("isShowDetailPage", true);
        this.callPublicResource(TransId.TRANS_DETAIL, map);
    }

    /**
     * @param orderNo            商户订单号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单
     *                           false或者不存在时表示由POS打单
     *                           <p>
     *                           tradeType:订单号输入方式
     *                           all：同时支持手输、扫码（应用停留在订单号输入界面，用户在POS上选择输入方式）
     *                           useScan：扫码方式，直接调起摄像头
     *                           manual：外部传入订单号方式，此时orderNo参数不能为空
     * @param handler
     * @throws PaymentProviderException
     */
    @Override
    public void scanTradeQuery(String orderNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException {
        if (TextUtils.isEmpty(this.appId)) {
            throw new PaymentProviderException("appId为空，不支持查询");
        }
        this.transId = TransId.QUERY_ORDER;
        HashMap<String, Object> map = new HashMap<>();
        map.put("appId", this.appId);
        map.put("isNeedPrintReceipt", isNeedPrintReceipt);
        map.put("tradeType", "all");
        map.put("orderNo", orderNo);
        this.callPosTrans(AppName.POS_MASTER, TransId.QUERY_ORDER, map);
    }
}
