/***************************************************************************************************
 * Copyright (c) 2003-2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.database.sqlite;

import android.content.Context;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.TreeMap;

/**
 * SQLite 数据库结构提供程序的抽象基类。
 *
 * @author zhangsai 2019-05-29 HIPOSAPP-1174
 */
public abstract class SQLiteDatabaseProvider implements ISQLiteDatabaseProvider {

    private Context context;     // Context
    private String databaseName; // 数据库名称
    private int version;         // 数据库版本号

    /**
     * 初始化一个 {@code SQLiteDatabaseProvider} 的实例。
     *
     * @param context      Context
     * @param databaseName 数据库名称
     * @param version      数据库版本号
     */
    public SQLiteDatabaseProvider(Context context, String databaseName, int version) {
        this.context = context;
        this.databaseName = databaseName;
        this.version = version;
    }

    /**
     * Context
     */
    @Override
    public Context context() {
        return this.context;
    }

    /**
     * 获取数据库名称。
     */
    @Override
    public String getDatabaseName() {
        return this.databaseName;
    }

    /**
     * 获取数据库版本号。
     */
    @Override
    public int version() {
        return this.version;
    }

    /**
     * 返回 onCreate(SQLiteDatabase) 时执行的 SQL 语句。
     *
     * @param provider SQLite 数据库结构提供程序
     * @return 执行的 SQL 语句
     */
    public static Collection<String> getOnCreateSql(ISQLiteDatabaseProvider provider) {
        Method[] declaredMethods = provider.getClass().getDeclaredMethods();

        // 查找 onCreate SQL 语句
        ArrayList<String> onCreateSql = new ArrayList<>();
        for (Method method : declaredMethods) {
            try {
                OnCreate onCreateSqlProvider = method.getAnnotation(OnCreate.class);
                if (onCreateSqlProvider != null) {
                    Object createSql = method.invoke(provider);
                    if (createSql instanceof String) {
                        onCreateSql.add((String) createSql);
                    } else if (createSql instanceof Collection<?>) {
                        onCreateSql.addAll((Collection<? extends String>) createSql);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return onCreateSql;
    }

    /**
     * 返回 onUpgrade(SQLiteDatabase, int, int) 时执行的 SQL 语句。
     *
     * @param provider   SQLite 数据库结构提供程序
     * @param oldVersion 升级前版本
     * @param newVersion 升级后版本
     * @return 执行的 SQL 语句及其数据库版本
     */
    public static Map<Integer, Collection<String>> getOnUpgradeSql(ISQLiteDatabaseProvider provider,
                                                                   int oldVersion, int newVersion) {
        Method[] declaredMethods = provider.getClass().getDeclaredMethods();

        // 查找 onUpgrade SQL 语句，并按 version 排序
        TreeMap<Integer, Collection<String>> upgradeSqlMap = new TreeMap<>();
        for (Method method : declaredMethods) {
            try {
                OnUpgrade onUpgradeSqlProvider = method.getAnnotation(OnUpgrade.class);
                if (onUpgradeSqlProvider != null) {
                    if (oldVersion < onUpgradeSqlProvider.version() && onUpgradeSqlProvider.version() <= newVersion) {
                        Object upgradeSql = method.invoke(provider);

                        // 添加单条 SQL 语句
                        if (upgradeSql instanceof String) {
                            if (upgradeSqlMap.containsKey(onUpgradeSqlProvider.version())) {
                                upgradeSqlMap.get(onUpgradeSqlProvider.version()).add((String) upgradeSql);
                            } else {
                                ArrayList<String> sqlList = new ArrayList<>();
                                sqlList.add((String) upgradeSql);
                                upgradeSqlMap.put(onUpgradeSqlProvider.version(), sqlList);
                            }
                        } else if (upgradeSql instanceof Collection<?>) {
                            // 添加 SQL 语句集合
                            if (upgradeSqlMap.containsKey(onUpgradeSqlProvider.version())) {
                                upgradeSqlMap.get(onUpgradeSqlProvider.version()).addAll((Collection<? extends String>) upgradeSql);
                            } else {
                                ArrayList<String> sqlList = new ArrayList<>();
                                sqlList.addAll((Collection<? extends String>) upgradeSql);
                                upgradeSqlMap.put(onUpgradeSqlProvider.version(), sqlList);
                            }
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return upgradeSqlMap;
    }
}
