/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.database.sqlite;

import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteStatement;
import android.util.Log;

/**
 * SQLite 资源库的基类。
 *
 * @author zhangsai 2018-01-11
 * @author zhangsai 2018-07-18 HIPOSAPP-675 直接使用 DBHelper，不再通过 DataBaseManager 中转
 * @author zhangsai 2018-12-22 HIPOSAPP-931 新增执行事务/读取数据的方法
 * @author zhangsai 2019-01-08 HIPOSAPP-966 新增 getDouble() 方法
 */
public class SQLiteRepository {
    private SQLiteOpenHelper dbHelper;

    /**
     * 初始化一个 {@code SQLiteRepository} 的实例。
     *
     * @param provider SQLiteOpenHelper
     */
    public SQLiteRepository(ISQLiteDatabaseProvider provider) {
        this.dbHelper = new SQLiteHelper(provider);
    }

    /**
     * 初始化一个 {@code SQLiteRepository} 的实例。
     *
     * @param helper SQLiteOpenHelper
     */
    public SQLiteRepository(SQLiteOpenHelper helper) {
        this.dbHelper = helper;
    }

    /**
     * 返回一个 {@code SQLiteStatement} 的实例。
     *
     * @param db   SQLite 数据库实例
     * @param sql  要执行的 SQL 语句
     * @param args 要执行的 SQL 语句的参数
     * @return {@code SQLiteStatement} 的实例
     */
    private static SQLiteStatement prepareStatement(SQLiteDatabase db, String sql, String... args) {
        SQLiteStatement statement = db.compileStatement(sql);
        if (args != null) {
            for (int i = 0; i < args.length; i++) {
                if (args[i] != null) {
                    statement.bindString(i + 1, args[i]);
                }
            }
        }
        return statement;
    }

    /**
     * 执行 SQL 语句。
     *
     * @param sql  要执行的 SQL 语句
     * @param args SQL 语句的参数集合
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    protected void execute(String sql, String... args) throws SQLException {
        SQLiteDatabase db = null;
        synchronized (this) {
            try {
                db = dbHelper.getReadableDatabase();
                this.execute(db, sql, args);
            } finally {
                if (db != null) {
                    db.close();
                }
            }
        }
    }

    /**
     * 执行 SQL 语句。
     *
     * @param database 打开的 SQL 数据库
     * @param sql      要执行的 SQL 语句
     * @param args     SQL 语句的参数集合
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    protected void execute(SQLiteDatabase database, String sql, String... args) throws SQLException {
        SQLiteStatement statement = null;
        synchronized (this) {
            try {
                if (database.isOpen()) {
                    statement = prepareStatement(database, sql, args);
                    statement.execute();
                }
            } finally {
                if (statement != null) {
                    statement.close();
                }
            }
        }
    }

    /**
     * 执行 SQL 事务。
     *
     * @param callback 事务处理的回调方法
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    protected void execute(TransactionCallback callback) throws SQLException {
        SQLiteDatabase db = null;

        try {
            db = dbHelper.getReadableDatabase();
            if (db.isOpen()) {
                db.beginTransaction();
                if (callback != null) {
                    callback.onTransaction(db);
                }
                db.setTransactionSuccessful();
            } else {
                Log.i("info", "Update Failure");
            }
        } finally {
            if (db != null) {
                db.endTransaction();
                db.close();
            }
        }
    }

    /**
     * 执行 SQL 语句并在 {@code QueryCallback} 中返回查询到的 {@code Cursor}。
     *
     * @param sql      要执行的 SQL 语句
     * @param callback 有查询结果时的回调方法
     * @throws SQLException 执行 SQL 语句时产生的异常
     */
    protected void executeCursor(String sql, QueryCallback callback) throws SQLException {
        this.executeCursor(sql, null, callback);
    }

    /**
     * @param sql       要执行的 SQL 语句
     * @param args      SQL 语句的参数集合
     * @param callback  读取数据的回调方法
     * @param <TResult> 读取数据的类型
     * @return {@code TResult} 的实例或 null
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    protected <TResult> TResult readSingle(String sql, String[] args, ReadCallback<TResult> callback) throws SQLException {
        SQLiteDatabase db = null;
        Cursor cursor = null;
        TResult result = null;

        try {
            db = dbHelper.getReadableDatabase();
            if (db.isOpen()) {
                cursor = db.rawQuery(sql, args);
                if (cursor != null && cursor.getCount() > 0) {
                    cursor.moveToFirst();
                    result = callback.onSuccess(cursor);
                }
            }
        } finally {
            if (cursor != null) {
                cursor.close();
            }
            if (db != null) {
                db.close();
            }
        }

        return result;
    }

    /**
     * 执行 SQL 语句并在 {@code QueryCallback} 中返回查询到的 {@code Cursor}。
     *
     * @param sql      要执行的 SQL 语句
     * @param args     SQL 语句的参数集合
     * @param callback 有查询结果时的回调方法
     * @throws SQLException 执行 SQL 语句时产生的异常
     */
    protected void executeCursor(String sql, String[] args, QueryCallback callback) throws SQLException {
        synchronized (this) {
            SQLiteDatabase db = null;
            Cursor cursor = null;
            try {
                db = dbHelper.getReadableDatabase();
                if (db.isOpen()) {
                    cursor = db.rawQuery(sql, args);
                    if (cursor != null && cursor.getCount() > 0) {
                        cursor.moveToFirst();
                        callback.onSuccess(cursor);
                    }
                }
            } finally {
                if (cursor != null) {
                    cursor.close();
                }
                if (db != null) {
                    db.close();
                }
            }
        }
    }

    /**
     * 执行并返回首行首列的数据。
     *
     * @param sql  要执行的 SQL 语句
     * @param args SQL 语句的参数集合
     * @return 首行首列的数据。如果数据库无记录，则返回 null。
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    protected String executeScalar(String sql, String... args) throws SQLException {
        synchronized(this) {
            String result = null;
            SQLiteDatabase db = null;
            Cursor cursor = null;

            try {
                db = dbHelper.getReadableDatabase();
                if (db.isOpen()) {
                    cursor = db.rawQuery(sql, args);
                    if (cursor != null && cursor.getCount() > 0) {
                        cursor.moveToFirst();
                        result = cursor.getString(0);
                    }
                }
            } finally {
                if (cursor != null) {
                    cursor.close();
                }
                if (db != null) {
                    db.close();
                }
            }

            return result;
        }
    }

    protected void executeDelete(String sql, String... args) throws SQLException {
        SQLiteDatabase db = null;
        SQLiteStatement statement = null;

        try {
            db = dbHelper.getReadableDatabase();
            if (db.isOpen()) {
                statement = prepareStatement(db, sql, args);
                statement.execute();
            } else {
                Log.i("info", "delete fail");
            }
        } finally {
            if (statement != null) {
                statement.close();
            }
            if (db != null) {
                db.close();
            }
        }
    }

    /**
     * 将字符串转换为 {@code Integer} 类型。
     *
     * @param value        要转换的字符串
     * @param defaultValue 默认值
     * @return 如果字符串为空，则返回默认值。其它情况下，返回转换后的值。
     */
    protected double getDouble(String value, double defaultValue) {
        if (value == null || value.length() == 0) {
            return defaultValue;
        }

        return Double.valueOf(value);
    }

    /**
     * 将字符串转换为 {@code Integer} 类型。
     *
     * @param value        要转换的字符串
     * @param defaultValue 默认值
     * @return 如果字符串为空，则返回默认值。其它情况下，返回转换后的值。
     */
    protected int getInteger(String value, int defaultValue) {

        if (value == null || value.length() == 0) {
            return defaultValue;
        }

        return Integer.valueOf(value);
    }

    /**
     * 读取 {@param cursor} 游标所在行指定 {@param columnName} 的列数据。
     *
     * @param cursor     当前查询的游标
     * @param columnName 列名
     * @return double 类型的列数据
     */
    protected double getDouble(Cursor cursor, String columnName) {
        return cursor.getDouble(cursor.getColumnIndex(columnName));
    }

    /**
     * 读取 {@param cursor} 游标所在行指定 {@param columnName} 的列数据。
     *
     * @param cursor     当前查询的游标
     * @param columnName 列名
     * @return int 类型的列数据
     */
    protected int getInt(Cursor cursor, String columnName) {
        return cursor.getInt(cursor.getColumnIndex(columnName));
    }

    /**
     * 读取 {@param cursor} 游标所在行指定 {@param columnName} 的列数据。
     *
     * @param cursor     当前查询的游标
     * @param columnName 列名
     * @return String 类型的列数据
     */
    protected String getString(Cursor cursor, String columnName) {
        return cursor.getString(cursor.getColumnIndex(columnName));
    }
}
